using System;
using System.Collections.Generic;
using System.Web;
using System.Text;
using System.IO;
using Vevo.Shared.Utilities;
using Vevo.Shared.Domain.SystemServices;

namespace Vevo.Shared.Domain.Logging
{
    public class ExceptionLog
    {
        #region Public Constances

        public const string LogFilePath = "~/App_Data/Logs/";

        #endregion


        #region Private

        private const int HttpResourceNotFoundErrorCode = 404;

        private Exception _ex;
        private IFileManager _fileManager;
        private DateTime _dateTime;

        private StringBuilder _sb;

        private void SaveErrorLog()
        {
            _sb = new StringBuilder();

            WriteExceptionHeader();

            Exception currentEx = _ex;
            while (currentEx != null)
            {
                if (IsInformativeException( currentEx ) ||
                    IsLastException( currentEx ))
                {
                    WriteExceptionDetails( currentEx );
                }

                currentEx = currentEx.InnerException;
            }

            string body = _sb.ToString();

            WriteLogFile( body );
        }

        private string GetLogFilename()
        {
            return String.Format( "{0}log_{1}-{2:00}-{3:00}.txt", 
                LogFilePath, _dateTime.Year, _dateTime.Month, _dateTime.Day );
        }

        private void WriteLogFile( string text )
        {
            TextWriter fp = null;
            try
            {

                string logFilename = GetLogFilename();
                fp = _fileManager.CreateTextWriter( logFilename, true );
                fp.WriteLine( "-------------------------" );
                fp.WriteLine( text );
                fp.WriteLine( "" );
            }
            catch
            {
            }
            finally
            {
                if (fp != null)
                    fp.Close();
            }
        }

        private void WriteExceptionHeader()
        {
            string dateTime = _dateTime.ToLongDateString() + ", at "
                            + _dateTime.ToShortTimeString();

            _sb.Append( "Exception generated on " );
            _sb.Append( dateTime );
            _sb.AppendLine();

            _sb.Append( "Page location: " );
            string rawUrl;
            try
            {
                rawUrl = HttpContext.Current.Request.RawUrl;
            }
            catch
            {
                rawUrl = "N/A";
            }
            _sb.Append( rawUrl );
        }

        private bool IsHttpPageNotFound( Exception ex )
        {
            return (ex is HttpException &&
                ((HttpException) ex).GetHttpCode() == HttpResourceNotFoundErrorCode);
        }

        private static string GetVisitorIP()
        {
            try
            {
                string ip = HttpContext.Current.Request.ServerVariables["HTTP_X_FORWARDED_FOR"];
                if (String.IsNullOrEmpty( ip ))
                    ip = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"];

                return ip;
            }
            catch
            {
                return "N/A";
            }
        }

        private void WriteExceptionDetails( Exception ex )
        {
            _sb.AppendLine();
            _sb.Append( "------------------------------" );

            if (ex is HttpException)
            {
                _sb.AppendLine();
                _sb.Append( "Http Code: " );
                _sb.Append( ((HttpException) ex).GetHttpCode() );
            }

            _sb.AppendLine();
            _sb.Append( "Message: " );
            _sb.Append( ex.Message );
            _sb.AppendLine();
            _sb.Append( "Visitor IP address: " );
            _sb.Append( GetVisitorIP() );
            _sb.AppendLine();
            _sb.Append( "Exception Type: " );
            _sb.Append( ex.GetType().ToString() );
            _sb.AppendLine();
            _sb.Append( "Method: " );
            _sb.Append( ex.TargetSite );

            if (!IsHttpPageNotFound( ex ))
            {
                _sb.AppendLine();
                _sb.Append( "Stack Trace: " );
                _sb.AppendLine();
                _sb.Append( ex.StackTrace );
            }
        }

        private bool IsInformativeException( Exception ex )
        {
            return ex.GetType() != typeof( HttpUnhandledException );
        }

        private bool IsLastException( Exception ex )
        {
            return ex.InnerException == null;
        }

        #endregion


        #region Constructors

        public ExceptionLog( Exception ex, IFileManager fileManager )
        {
            _ex = ex;
            _fileManager = fileManager;
            _dateTime = DateTime.Now;
        }

        public ExceptionLog( Exception ex, IFileManager fileManager, DateTime dateTime )
        {
            _ex = ex;
            _fileManager = fileManager;
            _dateTime = dateTime;
        }

        #endregion


        #region Public Methods

        public void Save()
        {
            SaveErrorLog();
        }

        #endregion

    }
}
