using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.Contents;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ContentMenuItemRepositoryNoCache : IContentMenuItemRepository
    {
        #region Private

        private const string AllFields =
            " Name, IsEnabled, ContentMenuItem.ContentMenuItemID, ContentMenuItem.ContentMenuID, ContentID, ReferringMenuID, SortOrder, Other1, Other2, Other3, UrlName, MenuPosition, CultureID,  Description , StoreID ";

        private const string AllContentMenuItemLocaleFields = "ContentMenuItemID, CultureID, Name, Description, UrlName ";

        private UrlNameAccessHelper _urlNameHelper = new UrlNameAccessHelper( "ContentMenuItemLocale", "ContentMenuItemID" );

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "contentmenuitemid" ))
                return "ORDER BY ContentMenuItem." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "name" ))
                return "ORDER BY L." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "sortorder" ))
                return "ORDER BY ContentMenuItem." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "contentmenuitemid":
                    return "ContentMenuItem.ContentMenuItemID";
                default:
                    return fieldName;
            }
        }

        private ContentMenuItemLocale CreateOneLocaleFromRow( DataRow row )
        {
            ContentMenuItemLocale locale = new ContentMenuItemLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.Name = row["Name"].ToString();
            locale.Description = row["Description"].ToString();
            locale.UrlName = row["UrlName"].ToString();

            return locale;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private IList<ContentMenuItem> ConvertContentMenuItemTableToList( Culture culture, DataTable table )
        {
            IList<ContentMenuItem> list = new List<ContentMenuItem>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneFromRow( culture, row ) );

            return list;
        }

        private void DeleteLocales( string contentmenuitemID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ContentMenuItemLocale " +
                "WHERE ContentMenuItemID = @ContentMenuItemID; ",
                DataAccess.CreateParameterString( contentmenuitemID ) );
        }

        private void InsertLocales( ContentMenuItem contentmenuitem )
        {
            foreach (ContentMenuItemLocale locale in contentmenuitem.Locales)
            {
                locale.UrlName = _urlNameHelper.CreateLocaleUrlName( locale.Name );
                DataAccess.ExecuteNonQuery(
                "INSERT INTO ContentMenuItemLocale ( ContentMenuItemID, CultureID, Name, Description, UrlName ) " +
                "VALUES ( @ContentMenuItemID, @CultureID, @Name, @Description, @UrlName ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( contentmenuitem.ContentMenuItemID ),
                DataAccess.CreateParameterString( locale.CultureID ),
                DataAccess.CreateParameterString( locale.Name ),
                DataAccess.CreateParameterString( locale.Description ),
                DataAccess.CreateParameterString( locale.UrlName )
                );
            }
        }

        private ContentMenuItem Create( ContentMenuItem contentmenuitem )
        {
            int recCount = int.Parse( DataAccess.ExecuteScalar(
                "SELECT COUNT(*) FROM ContentMenuItem WHERE ContentMenuID = @ContentMenuID;",
                DataAccess.CreateParameterString( contentmenuitem.ContentMenuID.ToString() ) ) );
            recCount = recCount + 1;
            string contentmenuitemID = DataAccess.ExecuteScalar(
                "INSERT INTO ContentMenuItem ( ContentMenuID, ContentID, ReferringMenuID, SortOrder, " +
                "IsEnabled, Other1, Other2, Other3, MenuPosition, StoreID ) " +
                "VALUES ( @ContentMenuID, @ContentID, @ReferringMenuID, @SortOrder, @IsEnabled, " +
                "@Other1, @Other2, @Other3, @MenuPosition, @StoreID ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( contentmenuitem.ContentMenuID ),
                DataAccess.CreateParameterString( contentmenuitem.ContentID ),
                DataAccess.CreateParameterString( contentmenuitem.ReferringMenuID ),
                DataAccess.CreateParameterInt32( recCount ),
                DataAccess.CreateParameterBool( contentmenuitem.IsEnabled ),
                DataAccess.CreateParameterString( contentmenuitem.Other1 ),
                DataAccess.CreateParameterString( contentmenuitem.Other2 ),
                DataAccess.CreateParameterString( contentmenuitem.Other3 ),
                DataAccess.CreateParameterString( contentmenuitem.MenuPosition.ToString() ),
                DataAccess.CreateParameterString( contentmenuitem.StoreID ) );

            contentmenuitem.ContentMenuItemID = contentmenuitemID;

            InsertLocales( contentmenuitem );


            return contentmenuitem;
        }


        private ContentMenuItem Update( ContentMenuItem contentmenuitem )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE  ContentMenuItem " +
                "SET ContentMenuID = @ContentMenuID,ContentID = @ContentID, " +
                "ReferringMenuID = @ReferringMenuID, IsEnabled = @IsEnabled, " +
                "Other1 = @Other1, Other2 = @Other2, Other3 = @Other3,  StoreID = @StoreID " +
                "WHERE ContentMenuItemID = @ContentMenuItemID; ",
                DataAccess.CreateParameterString( contentmenuitem.ContentMenuID ),
                DataAccess.CreateParameterString( contentmenuitem.ContentID ),
                DataAccess.CreateParameterString( contentmenuitem.ReferringMenuID ),
                DataAccess.CreateParameterBool( contentmenuitem.IsEnabled ),
                DataAccess.CreateParameterString( contentmenuitem.Other1 ),
                DataAccess.CreateParameterString( contentmenuitem.Other2 ),
                DataAccess.CreateParameterString( contentmenuitem.Other3 ),
                DataAccess.CreateParameterString( contentmenuitem.StoreID ),
                DataAccess.CreateParameterString( contentmenuitem.ContentMenuItemID ) );

            DeleteLocales( contentmenuitem.ContentMenuItemID );
            InsertLocales( contentmenuitem );


            return contentmenuitem;
        }

        private ContentMenuItem CreateOneFromRow( Culture culture, DataRow row )
        {
            ContentMenuItem contentmenuitem = new ContentMenuItem( culture );

            contentmenuitem.ContentMenuItemID = row["ContentMenuItemID"].ToString();
            contentmenuitem.ContentMenuID = row["ContentMenuID"].ToString();
            contentmenuitem.ContentID = row["ContentID"].ToString();
            contentmenuitem.ReferringMenuID = row["ReferringMenuID"].ToString();
            contentmenuitem.SortOrder = ConvertUtilities.ToInt32( row["SortOrder"] );
            contentmenuitem.IsEnabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );
            contentmenuitem.Other1 = row["Other1"].ToString();
            contentmenuitem.Other2 = row["Other2"].ToString();
            contentmenuitem.Other3 = row["Other3"].ToString();
            contentmenuitem.Name = row["Name"].ToString();
            contentmenuitem.Description = row["Description"].ToString();
            contentmenuitem.UrlName = row["UrlName"].ToString();
            contentmenuitem.MenuPosition =
                EnumUtilities<ContentMenuItem.MenuPositionType>.Parse( row["MenuPosition"].ToString() );
            contentmenuitem.StoreID = row["StoreID"].ToString();

            return contentmenuitem;
        }

        private IList<ContentMenuItem> ConvertTableToList( Culture culture, DataTable table )
        {
            return ConvertTableToList( culture, table, 0, table.Rows.Count );
        }

        private IList<ContentMenuItem> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<ContentMenuItem> list = new List<ContentMenuItem>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }


        private string GetCultureID( Culture culture )
        {
            return culture.CultureID;
        }

        private void CopyContentMenuItemLocales( ContentMenuItem contentmenuitem, IList<ILocale> locales )
        {
            foreach (ContentMenuItemLocale locale in locales)
            {
                contentmenuitem.Locales.Add( locale );
            }
        }

        #endregion

        #region Public Methods

        public ContentMenuItem Save( ContentMenuItem contentmenuitem )
        {
            ContentMenuItem result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (contentmenuitem.ContentMenuItemID == "0")
                    result = Create( contentmenuitem );
                else
                    result = Update( contentmenuitem );

                transaction.Complete();
            }

            return result;
        }

        public ContentMenuItem GetOne( Culture culture, string contentmenuitemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM ContentMenuItem " +
                "LEFT JOIN (SELECT * FROM ContentMenuItemLocale WHERE CultureID = @CultureID) L " +
                "ON ContentMenuItem.ContentMenuItemID = L.ContentMenuItemID " +
                "WHERE  ContentMenuItem.ContentMenuItemID = @ContentMenuItemID ; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( contentmenuitemID )
                );

            if (table.Rows.Count > 0)
            {
                ContentMenuItem contentMenuItem = CreateOneFromRow( culture, table.Rows[0] );
                CopyContentMenuItemLocales( contentMenuItem, GetLocales( contentmenuitemID ) );
                return contentMenuItem;
            }
            else
                return ContentMenuItem.Null;

        }

        public IList<ContentMenuItem> GetByContentMenuID( Culture culture, string contentMenuID, string sortBy, BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM ContentMenuItem LEFT JOIN " +
                "    (SELECT * FROM ContentMenuItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON ContentMenuItem.ContentMenuItemID = L.ContentMenuItemID " +
                "WHERE ContentMenuID = @ContentMenuID " +
                FlagFilterHelper.CreateFlagFilterWithAnd(
                "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) + CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( contentMenuID ) );

            return ConvertContentMenuItemTableToList( culture, table );
        }

        public IList<ContentMenuItem> GetByContentMenuID(
            Culture culture,
            string contentMenuID,
            string sortBy,
            BoolFilter isEnabled,
            BoolFilter contentIsEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT Name, ContentMenuItem.IsEnabled, ContentMenuItem.ContentMenuItemID, ContentMenuItem.ContentMenuID, " +
                "ContentMenuItem.ContentID, ReferringMenuID, SortOrder, ContentMenuItem.Other1, ContentMenuItem.Other2, ContentMenuItem.Other3, UrlName, MenuPosition, CultureID,  " +
                "Description , StoreID " +
                "FROM ContentMenuItem LEFT JOIN " +
                "    (SELECT * FROM ContentMenuItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON ContentMenuItem.ContentMenuItemID = L.ContentMenuItemID " +
                "LEFT JOIN (SELECT * FROM Content) AS C ON ContentMenuItem.ContentID = C.ContentID " +
                "WHERE ContentMenuID = @ContentMenuID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "ContentMenuItem.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                FlagFilterHelper.CreateFlagFilterWithAnd( "C.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( contentIsEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( contentMenuID ) );

            return ConvertContentMenuItemTableToList( culture, table );
        }

        public IList<ContentMenuItem> GetByPosition( Culture culture, string menuPosition, string sortBy, BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM ContentMenuItem LEFT JOIN " +
                "    (SELECT * FROM ContentMenuItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON ContentMenuItem.ContentMenuItemID = L.ContentMenuItemID " +
                "WHERE MenuPosition = @MenuPosition " +
                FlagFilterHelper.CreateFlagFilterWithAnd(
                "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) + CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( menuPosition ) );

            return ConvertContentMenuItemTableToList( culture, table );
        }

        public string GetContentMenuItemIDFromUrlName( string urlName )
        {
            string contentMenuItemID = DataAccess.ExecuteScalar(
                "SELECT ContentMenuItemID " +
                "FROM ContentMenuItemLocale " +
                "WHERE UrlName = @urlName; ",
                DataAccess.CreateParameterString( urlName ) );

            return contentMenuItemID;
        }

        public void Delete( string contentmenuitemID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ContentMenuItem " +
                "WHERE ContentMenuItemID = @ContentMenuItemID; ",
                DataAccess.CreateParameterString( contentmenuitemID ) );

                DeleteLocales( contentmenuitemID );
                transaction.Complete();
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ContentMenuItemLocale " +
                "WHERE CultureID = @CultureID ;",
                DataAccess.CreateParameterString( cultureID ) );
        }

        public IList<ILocale> GetLocales( string contentmenuitemID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
               "SELECT " + AllContentMenuItemLocaleFields +
               "FROM ContentMenuItemLocale " +
               "WHERE ContentMenuItemID = @ContentMenuItemID; ",
               DataAccess.CreateParameterString( contentmenuitemID ) );

            return ConvertLocaleTableToList( localeTable );
        }


        public IList<ContentMenuItem> SearchContentMenuItem(
                Culture culture,
                string menuPosition,
                string sortBy,
                SearchFilter searchFilter )
        {
            string sqlFieldName = ConvertFieldName( searchFilter.FieldName );
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, sqlFieldName, parameterList );
            parameterList.Add( DataAccess.CreateParameterString( menuPosition ) );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT " + AllFields + " , 0 AS [Level]" +
                "FROM ContentMenuItem LEFT JOIN " +
                "(SELECT * FROM ContentMenuItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON ContentMenuItem.ContentMenuItemID = L.ContentMenuItemID " +
                commandSearch +
                "AND MenuPosition = @MenuPosition " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );


            DataColumn[] keys = new DataColumn[] { full.Columns["ContentMenuItem.ContentMenuItemID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( culture, full );
        }

        public IList<ContentMenuItem> GetAll( Culture culture, BoolFilter enabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields + "FROM ContentMenuItem LEFT JOIN " +
                "(SELECT * FROM ContentMenuItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON ContentMenuItem.ContentMenuItemID = L.ContentMenuItemID " +
                FlagFilterHelper.CreateFlagFilterWithWhere(
                "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( enabled ) ) +
                "ORDER BY ContentMenuItem.ContentMenuItemID",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) )
                );
            return ConvertTableToList( culture, table );
        }


        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM ContentMenuItem INNER JOIN ContentMenuItemLocale L " +
                "ON ContentMenuItem.ContentMenuItemID = L.ContentMenuItemID WHERE  1 = 0; "
                );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public string GetMaxSortOrder( string contentMenuID )
        {
            string max = DataAccess.ExecuteScalar(
                "SELECT MAX(SortOrder) FROM ContentMenuItem WHERE ContentMenuID = @ContentMenuID;"
            ,
               DataAccess.CreateParameterString( contentMenuID ) );

            if (max == "")
                return "0";
            else
                return max;
        }

        public void ReArrangeSortOrderByContentMenuID( Culture culture, string parentContentMenuID,
           BoolFilter enabled )
        {
            IList<ContentMenuItem> list = GetByContentMenuID( culture, parentContentMenuID,
                "SortOrder", enabled );
            string[] result = new string[list.Count];
            int recCount = 0;
            foreach (ContentMenuItem item in list)
            {
                result[recCount] = item.ContentMenuItemID;
                recCount++;
            }
            UpdateSortOrder( result );
        }

        public void ReArrangeSortOrder( Culture culture, string parentContentMenuID,
            string contentMenuItemID, BoolFilter enabled )
        {
            IList<ContentMenuItem> list = GetByContentMenuID( culture, parentContentMenuID,
                "SortOrder", enabled );
            int recCount = 0;
            string[] result = new string[list.Count + 1];
            foreach (ContentMenuItem item in list)
            {
                result[recCount] = item.ContentMenuItemID;
                recCount++;
            }
            result[recCount] = contentMenuItemID;
            UpdateSortOrder( result );
        }

        public void UpdateSortOrder( string[] sortList )
        {
            int sortCount = 0;
            foreach (string items in sortList)
            {
                sortCount++;
                DataAccess.ExecuteNonQuery(
                    "UPDATE ContentMenuItem SET SortOrder = @SortOrder WHERE ContentMenuItemID = @ContentMenuItemID;",
                    DataAccess.CreateParameterString( sortCount.ToString() ),
                    DataAccess.CreateParameterString( items ) );
            }
        }

        public IList<ContentMenuItem> GetByStoreID( Culture culture, string storeID, string sortBy, BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM ContentMenuItem LEFT JOIN " +
                "    (SELECT * FROM ContentMenuItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON ContentMenuItem.ContentMenuItemID = L.ContentMenuItemID " +
                "WHERE StoreID = @StoreID " +
                FlagFilterHelper.CreateFlagFilterWithAnd(
                "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) + CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ) );

            return ConvertContentMenuItemTableToList( culture, table );
        }
        #endregion
    }
}
