using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Users;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;

namespace Vevo.Data
{
    public class CountryRepositoryNoCache : ICountryRepository
    {
        #region Private

        private const string allFields = "CountryCode, CommonName, FormalName, CountryType, CountrySubType, " +
                "Sovereignty, Capital, CurrencyCode, CurrencyName, TelephoneCode, CountryCode3, CountryNumber, " +
                "InternetCountryCode, SortOrder, Enabled ";

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private Country CreateCountry( Country country )
        {
            DataAccess.ExecuteScalar(
            "INSERT INTO Country( CountryCode, CommonName, FormalName, CountryType, CountrySubType, " +
            "Sovereignty, Capital, CurrencyCode, CurrencyName, " +
            "TelephoneCode, CountryCode3, CountryNumber, " +
            "InternetCountryCode, SortOrder, Enabled " +
            ") " +
            "VALUES ( @CountryCode, @CommonName, @FormalName, @CountryType, @CountrySubType, " +
            "@Sovereignty, @Capital, @CurrencyCode, @CurrencyName, " +
            "@TelephoneCode, @CountryCode3, @CountryNumber, " +
            "@InternetCountryCode, @SortOrder, @Enabled " +
            ");" +
            "SELECT @@Identity; ",
            DataAccess.CreateParameterString( country.CountryCode ),
            DataAccess.CreateParameterString( country.CommonName ),
            DataAccess.CreateParameterString( country.FormalName ),
            DataAccess.CreateParameterString( country.CountryType ),
            DataAccess.CreateParameterString( country.CountrySubType ),
            DataAccess.CreateParameterString( country.Sovereignty ),
            DataAccess.CreateParameterString( country.Capital ),
            DataAccess.CreateParameterString( country.CurrencyCode ),
            DataAccess.CreateParameterString( country.CurrencyName ),
            DataAccess.CreateParameterString( country.TelephoneCode ),
            DataAccess.CreateParameterString( country.CountryCode3 ),
            DataAccess.CreateParameterString( country.CountryNumber ),
            DataAccess.CreateParameterString( country.InternetCountryCode ),
            DataAccess.CreateParameterInt32( country.SortOrder ),
            DataAccess.CreateParameterBool( country.Enabled ) );

            return country;
        }

        private Country UpdateCountry( Country country, string newCountryCode )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Country SET CountryCode = @CountryCodeNew, " +
                "CommonName = @CommonName, " +
                "FormalName = @FormalName, " +
                "CountryType = @CountryType, " +
                "CountrySubType = @CountrySubType, " +
                "Sovereignty = @Sovereignty, " +
                "Capital = @Capital, " +
                "CurrencyCode = @CurrencyCode, " +
                "CurrencyName = @CurrencyName, " +
                "TelephoneCode = @TelephoneCode, " +
                "CountryCode3 = @CountryCode3, " +
                "CountryNumber = @CountryNumber, " +
                "InternetCountryCode = @InternetCountryCode, " +
                "SortOrder = @SortOrder, " +
                "Enabled = @Enabled " +
                "WHERE CountryCode = @CountryCodeOld; ",
                DataAccess.CreateParameterString( newCountryCode ),
                DataAccess.CreateParameterString( country.CommonName ),
                DataAccess.CreateParameterString( country.FormalName ),
                DataAccess.CreateParameterString( country.CountryType ),
                DataAccess.CreateParameterString( country.CountrySubType ),
                DataAccess.CreateParameterString( country.Sovereignty ),
                DataAccess.CreateParameterString( country.Capital ),
                DataAccess.CreateParameterString( country.CurrencyCode ),
                DataAccess.CreateParameterString( country.CurrencyName ),
                DataAccess.CreateParameterString( country.TelephoneCode ),
                DataAccess.CreateParameterString( country.CountryCode3 ),
                DataAccess.CreateParameterString( country.CountryNumber ),
                DataAccess.CreateParameterString( country.InternetCountryCode ),
                DataAccess.CreateParameterInt32( country.SortOrder ),
                DataAccess.CreateParameterBool( country.Enabled ),
                DataAccess.CreateParameterString( country.CountryCode ) );

            country.CountryCode = newCountryCode;
            return country;
        }

        private Country CreateOneFromRow( DataRow row )
        {
            Country country = new Country();
            country.CommonName = row["CommonName"].ToString();
            country.FormalName = row["FormalName"].ToString();
            country.CountryType = row["CountryType"].ToString();
            country.CountrySubType = row["CountrySubType"].ToString();
            country.Sovereignty = row["Sovereignty"].ToString();
            country.Capital = row["Capital"].ToString();
            country.CurrencyCode = row["CurrencyCode"].ToString();
            country.CurrencyName = row["CurrencyName"].ToString();
            country.TelephoneCode = row["TelephoneCode"].ToString();
            country.CountryCode = row["CountryCode"].ToString();
            country.CountryCode3 = row["CountryCode3"].ToString();
            country.CountryNumber = row["CountryNumber"].ToString();
            country.InternetCountryCode = row["InternetCountryCode"].ToString();
            country.SortOrder = ConvertUtilities.ToInt32( row["SortOrder"] );
            country.Enabled = ConvertUtilities.ToBoolean( row["Enabled"] );

            return country;
        }

        private IList<Country> ConvertTableToList( DataTable table )
        {
            IList<Country> list = new List<Country>();

            for (int i = 0; i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }
        #endregion


        #region Public Methods

        public Country Create( Country country )
        {
            return CreateCountry( country );
        }

        public Country Update( Country country )
        {
            return UpdateCountry( country, country.CountryCode );
        }

        public Country Update( Country country, string newCountryCode )
        {
            return UpdateCountry( country, newCountryCode );
        }

        public Country GetOne( string countryCode )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM Country WHERE CountryCode = @CountryCode;",
                DataAccess.CreateParameterString( countryCode ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return Country.Null;
        }

        public Country GetOneByName( string countryName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM Country WHERE CommonName = @CommonName;",
                DataAccess.CreateParameterString( countryName ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return Country.Null;
        }

        public void Delete( string countryCode )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM Country WHERE CountryCode = @CountryCode; ",
                DataAccess.CreateParameterString( countryCode ) );
        }

        public IList<Country> GetAll( BoolFilter enabled, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM Country WHERE 1 = 1 " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "Enabled", RepositoryUtilities.ConvertToFlagFilter( enabled ) ) +
                CreateSortByText( sortBy ) );

            return ConvertTableToList( table );
        }

        public void DeleteAll()
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM Country;" );
        }


        #endregion
    }
}
