﻿using System.Collections.Generic;
using System.Data;
using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;

namespace Vevo.Data.Implementation
{
    internal class ProductKitItemAccess
    {
        #region Private

        private ProductKitItem CreateOneFromRow( DataRow row )
        {
            ProductKitItem productKitItem = new ProductKitItem();
            productKitItem.ProductKitGroupID = row["ProductKitGroupID"].ToString();
            productKitItem.ProductID = row["ProductID"].ToString();
            productKitItem.Quantity = ConvertUtilities.ToInt32( row["Quantity"] );
            productKitItem.IsUserDefinedQuantity = ConvertUtilities.ToBoolean( row["IsUserDefinedQuantity"] );
            productKitItem.SortOrder = ConvertUtilities.ToInt32( row["SortOrder"] );
            productKitItem.IsDefault = ConvertUtilities.ToBoolean( row["IsDefault"] );

            return productKitItem;
        }

        private IList<ProductKitItem> ConvertProductKitItemTableToList( DataTable table )
        {
            IList<ProductKitItem> list = new List<ProductKitItem>();

            foreach (DataRow row in table.Rows)
            {
                ProductKitItem productKitItem = new ProductKitItem();
                productKitItem.ProductKitGroupID = row["ProductKitGroupID"].ToString();
                productKitItem.ProductID = row["ProductID"].ToString();
                productKitItem.Quantity = ConvertUtilities.ToInt32( row["Quantity"] );
                productKitItem.IsUserDefinedQuantity = ConvertUtilities.ToBoolean( row["IsUserDefinedQuantity"] );
                productKitItem.SortOrder = ConvertUtilities.ToInt32( row["SortOrder"] );
                productKitItem.IsDefault = ConvertUtilities.ToBoolean( row["IsDefault"] );

                list.Add( productKitItem );
            }

            return list;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.ToLower() == "productid")
                return " ORDER BY ProductKitItem." + sortBy;
            else
                return " ORDER BY " + sortBy;
        }

        #endregion


        #region Public Methods

        public int DeleteByProductID( string productID )
        {
            return DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductKitItem " +
                "WHERE ProductID = @ProductID ;",
                DataAccess.CreateParameterString( productID ) );
        }

        public int DeleteByProductKitGroupID( string productKitGroupID )
        {
            return DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductKitItem " +
                "WHERE ProductKitGroupID = @ProductKitGroupID ;",
                DataAccess.CreateParameterString( productKitGroupID ) );
        }

        public int DeleteProductKitItem( string productID, string productKitGroupID )
        {
            return DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductKitItem " +
                "WHERE ProductKitGroupID = @ProductKitGroupID AND ProductID = @ProductID ;",
                DataAccess.CreateParameterString( productKitGroupID ),
                DataAccess.CreateParameterString( productID ) );
        }

        private void InsertProductKitItem( string productKitGroupID, IList<ProductKitItem> productKitItemList )
        {
            foreach (ProductKitItem productKitItem in productKitItemList)
            {
                int result = DataAccess.ExecuteNonQuery(
                    "INSERT INTO ProductKitItem (ProductKitGroupID ,ProductID, Quantity, IsUserDefinedQuantity, SortOrder, IsDefault ) " +
                    "VALUES ( @ProductKitGroupID, @ProductID, @Quantity, @IsUserDefinedQuantity, @SortOrder, @IsDefault );",
                    DataAccess.CreateParameterString( productKitGroupID ),
                    DataAccess.CreateParameterString( productKitItem.ProductID ),
                    DataAccess.CreateParameterInt32( productKitItem.Quantity ),
                    DataAccess.CreateParameterBool( productKitItem.IsUserDefinedQuantity ),
                    DataAccess.CreateParameterInt32( productKitItem.SortOrder ),
                    DataAccess.CreateParameterBool( productKitItem.IsDefault ) );
            }
        }

        public void SaveAll( string productKitGroupID, IList<ProductKitItem> productKitItemList )
        {
            DeleteByProductKitGroupID( productKitGroupID );
            InsertProductKitItem( productKitGroupID, productKitItemList );
        }

        public ProductKitItem GetOne( string productKitGroupID, string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ProductKitGroupID ,ProductID, Quantity, IsUserDefinedQuantity, SortOrder, IsDefault " +
                "FROM ProductKitItem " +
                "WHERE ProductKitGroupID = @ProductKitGroupID AND ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productKitGroupID ),
                DataAccess.CreateParameterString( productID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return ProductKitItem.Null;
        }

        public IList<ProductKitItem> GetAll()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ProductKitGroupID ,ProductID, Quantity, IsUserDefinedQuantity, SortOrder, IsDefault " +
                "FROM ProductKitItem " );
            return ConvertProductKitItemTableToList( table );
        }

        public IList<ProductKitItem> GetByProductKitGroupID( string productKitGroupID, Culture culture )
        {
            return GetByProductKitGroupID( productKitGroupID, "SortOrder", culture );
        }

        public IList<ProductKitItem> GetByProductKitGroupID( string productKitGroupID, string sortBy, Culture culture )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ProductKitGroupID ,ProductKitItem.ProductID, Quantity, IsUserDefinedQuantity, SortOrder, IsDefault, Name " +
                "FROM ProductKitItem " +
                "LEFT JOIN " +
                "   (SELECT Product.ProductID, Name FROM Product " +
                "   LEFT JOIN (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L ON Product.ProductID = L.ProductID ) AS N " +
                "ON N.ProductID = ProductKitItem.ProductID " +
                "WHERE ProductKitGroupID = @ProductKitGroupID " +
                CreateSortByText( sortBy ),
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( productKitGroupID ) );

            return ConvertProductKitItemTableToList( table );
        }

        public ProductKitItem GetByProductID( string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ProductKitGroupID ,ProductID, Quantity, IsUserDefinedQuantity, SortOrder, IsDefault " +
                "FROM ProductKitItem " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );

            IList<ProductKitItem> list = ConvertProductKitItemTableToList( table );
            if (list.Count > 0)
                return list[0];
            else
                return ProductKitItem.Null;
        }

        public void UpdateSortOrder( string productKitGroup, string[] sortedProductKitItems )
        {
            int sortCount = 0;
            foreach (string item in sortedProductKitItems)
            {
                sortCount++;
                DataAccess.ExecuteNonQuery(
                    "UPDATE ProductKitItem SET SortOrder = @SortOrder WHERE ProductID = @ProductID AND ProductKitGroupID = @ProductKitGroupID;",
                    DataAccess.CreateParameterString( sortCount.ToString() ),
                    DataAccess.CreateParameterString( item ),
                    DataAccess.CreateParameterString( productKitGroup ) );
            }
        }
        #endregion
    }
}
