﻿using System.Collections.Generic;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ProductKitGroupRepository : IProductKitGroupRepository
    {
        #region Private

        private const string RepositoryName = "ProductKitGroupRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetOneItemSignature = "GetOneItem";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetAllSignature = "GetAll";
        private const string GetAllExceptBaseProductSingnature = "GetAllExceptBaseProduct";
        private const string GetProductKitItemsSignature = "GetProductKitItems";
        private const string SearchProductKitGroupSignature = "SearchProductKitGroup";
        private const string GetProductKitItemSignature = "GetProductKitItem";
        private const string GetAllProductKitItemsSignature = "GetAllProductKitItems";

        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "ProductKitGroup", "ProductKitGroupLocale", "ProductKit" } );

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ProductKitGroupRepositoryNoCache _repository = new ProductKitGroupRepositoryNoCache();

        #endregion

        #region Constructors

        public ProductKitGroupRepository()
        {
        }

        public ProductKitGroupRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public ProductKitGroup Save( ProductKitGroup productKitGroup )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            ProductKitGroup newProductKitGroup = _repository.Save( productKitGroup );
            _dataCache.ClearRepositoryCache();
            return newProductKitGroup;
        }

        public void Delete( string productKitGroupID )
        {
            _repository.Delete( productKitGroupID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public ProductKitGroup GetOne( Culture culture, string productKitGroupID )
        {
            ProductKitGroup result = (ProductKitGroup) _dataCache.GetValue(
                GetOneSignature,
                _cacheDependency,
                culture.CultureID,
                productKitGroupID );

            if (result == null)
            {
                result = _repository.GetOne( culture, productKitGroupID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, productKitGroupID );
            }

            return result;
        }

        public IList<ILocale> GetLocales( string productKitGroupID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, productKitGroupID );

            if (list == null)
            {
                list = _repository.GetLocales( productKitGroupID );
                _dataCache.SetValue( GetLocalesSignature, list, productKitGroupID );
            }

            return list;
        }

        public IList<ProductKitGroup> GetAll( Culture culture )
        {
            IList<ProductKitGroup> result = (IList<ProductKitGroup>) _dataCache.GetValue(
                GetAllSignature, _cacheDependency, culture.CultureID
                );

            if (result == null)
            {
                result = new List<ProductKitGroup>();
                result = _repository.GetAll( culture );
                _dataCache.SetValue(
                    GetAllSignature,
                    result,
                    culture.CultureID
                    );
            }
            return result;
        }

        public IList<ProductKitGroup> GetAll( Culture culture, string sortBy )
        {
            IList<ProductKitGroup> result = (IList<ProductKitGroup>) _dataCache.GetValue(
                GetAllSignature, _cacheDependency, culture.CultureID, sortBy
                );

            if (result == null)
            {
                result = new List<ProductKitGroup>();
                result = _repository.GetAll( culture, sortBy );
                _dataCache.SetValue(
                    GetAllSignature,
                    result,
                    culture.CultureID,
                    sortBy
                    );
            }
            return result;
        }

        public IList<ProductKitGroup> GetAllByProductID( Culture culture, string productID, string sortBy )
        {
            IList<ProductKitGroup> result = (IList<ProductKitGroup>) _dataCache.GetValue(
                GetAllSignature, _cacheDependency, culture.CultureID, productID, sortBy );

            if (result == null)
            {
                result = new List<ProductKitGroup>();
                result = _repository.GetAllByProductID( culture, productID, sortBy );
                _dataCache.SetValue(
                    GetAllSignature,
                    result,
                    culture.CultureID,
                    productID,
                    sortBy
                    );
            }
            return result;
        }

        public IList<ProductKitGroup> GetAllExceptBaseProduct( Culture culture, string productID )
        {
            IList<ProductKitGroup> result = (IList<ProductKitGroup>) _dataCache.GetValue(
                GetAllExceptBaseProductSingnature, _cacheDependency, culture.CultureID, productID
                );

            if (result == null)
            {
                result = new List<ProductKitGroup>();
                result = _repository.GetAllExceptBaseProduct( culture, productID );
                _dataCache.SetValue(
                    GetAllExceptBaseProductSingnature,
                    result,
                    culture.CultureID,
                    productID
                    );
            }
            return result;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public IList<ProductKitGroup> SearchProductKitGroup(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchProductKitGroupSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchProductKitGroup( culture, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchProductKitGroupSignature,
                    result,
                    culture.CultureID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<ProductKitGroup>) result.First;
        }

        public ProductKitItem GetOneItem( string productKitGroupID, string productID )
        {
            ProductKitItem productKitItem =
                (ProductKitItem) _dataCache.GetValue(
                    GetOneItemSignature,
                    _cacheDependency,
                    productKitGroupID,
                    productID );

            if (productKitItem == null)
            {
                productKitItem = _repository.GetOneItem( productKitGroupID, productID );
                _dataCache.SetValue( GetOneItemSignature, productKitItem, productKitGroupID, productID );
            }

            return productKitItem;
        }

        public IList<ProductKitItem> GetProductKitItems( string productKitGroupID, Culture culture )
        {
            IList<ProductKitItem> productKitItems =
                (IList<ProductKitItem>) _dataCache.GetValue(
                    GetProductKitItemsSignature,
                    _cacheDependency,
                    productKitGroupID,
                    culture.CultureID );

            if (productKitItems == null)
            {
                productKitItems = _repository.GetProductKitItems( productKitGroupID, culture );
                _dataCache.SetValue( GetProductKitItemsSignature, productKitItems, productKitGroupID, culture.CultureID );
            }

            return productKitItems;
        }

        public IList<ProductKitItem> GetProductKitItems( string productKitGroupID, string sortBy, Culture culture )
        {
            IList<ProductKitItem> productKitItems =
                (IList<ProductKitItem>) _dataCache.GetValue(
                    GetProductKitItemsSignature,
                    _cacheDependency,
                    productKitGroupID,
                    sortBy,
                    culture.CultureID );

            if (productKitItems == null)
            {
                productKitItems = _repository.GetProductKitItems( productKitGroupID, sortBy, culture );
                _dataCache.SetValue( GetProductKitItemsSignature, productKitItems, productKitGroupID, sortBy, culture.CultureID );
            }

            return productKitItems;
        }

        public void DeleteProductKitItemByGroupID( string productKitGroupID )
        {
            _repository.DeleteProductKitItemByGroupID( productKitGroupID );
            _dataCache.ClearRepositoryCache();
        }
        public ProductKitItem GetProductKitItem( string productID )
        {
            ProductKitItem productKitItem =
                (ProductKitItem) _dataCache.GetValue(
                    GetProductKitItemSignature,
                    _cacheDependency,
                    productID );

            if (productKitItem == null)
            {
                productKitItem = _repository.GetProductKitItem( productID );
                _dataCache.SetValue( GetProductKitItemSignature, productKitItem, productID );
            }

            return productKitItem;
        }

        public IList<ProductKitItem> GetAllProductKitItems()
        {
            IList<ProductKitItem> result = (IList<ProductKitItem>) _dataCache.GetValue(
                GetAllProductKitItemsSignature, _cacheDependency );

            if (result == null)
            {
                result = new List<ProductKitItem>();
                result = _repository.GetAllProductKitItems();
                _dataCache.SetValue( GetAllProductKitItemsSignature, result );
            }
            return result;
        }

        public void DeleteProductKitItem( string productID, string productKitGroupID )
        {
            _repository.DeleteProductKitItem( productID, productKitGroupID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteProductKitItemByProductID( string productID )
        {
            _repository.DeleteProductKitItemByProductID( productID );
            _dataCache.ClearRepositoryCache();
        }

        public void UpdateProductKitSortOrder( string productKit, string[] sortedProductKitGroup )
        {
            _repository.UpdateProductKitSortOrder( productKit, sortedProductKitGroup );
            _dataCache.ClearRepositoryCache();
        }

        public void UpdateProductKitItemSortOrder( string productKitGroup, string[] sortedProductKitItems )
        {
            _repository.UpdateProductKitItemSortOrder( productKitGroup, sortedProductKitItems );
            _dataCache.ClearRepositoryCache();
        }

        #endregion
    }
}
