using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Web;
using Vevo.Data.Implementation;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Domain.Stores;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ProductRepositoryNoCache : IProductRepository
    {
        #region Private

        private string GenerateSqlSelectByOptionGroup(
            string cultureID, string optionGroupID, string alias )
        {
            return String.Format( "(SELECT OptionItem.OptionItemID, Name FROM OptionItem INNER JOIN " +
                "OptionItemLocale ON OptionItem.OptionItemID = OptionItemLocale.OptionItemID " +
                "WHERE OptionGroupID = {0} AND CultureID = {2}) AS {1}", optionGroupID, alias, cultureID );
        }

        private const string AllProductLocaleFields = "ProductID, CultureID, Name, ShortDescription, LongDescription, UrlName ";
        private const string AllProductImageLocaleFields = "ProductImageID, CultureID, AltTag, TitleTag ";

        private const string AllProductMetaInformationFields = "ProductID, CultureID, StoreID, MetaKeyword, MetaDescription, UseDefaultMetaKeyword, UseDefaultMetaDescription ";

        private const string SQLSelectProductString =
                "SELECT Product.ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, Model, " +
                "    ImageSecondary, SumStock, Weight, FixedShippingCost, FreeShippingCost, " +
                "    ShippingCost, Manufacturer, " +
                "    Keywords, " +
                "    IsTodaySpecial, MinQuantity, MaxQuantity, IsDownloadable, DownloadPath, Product.IsEnabled, RelatedProducts, " +
                "    MerchantRating, Product.DiscountGroupID as ProductDiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, IsParentVisible, " +
                "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, " +
                "    UseInventory, ProductDetailsLayoutPath, IsAffiliate, IsCallForPrice, Brand, ReturnTime, IsProductKit, IsDynamicProductKitPrice, IsDynamicProductKitWeight,CreateDate,ViewCount, " +
                "    Length, Width, Height, " +
            // For Recurring
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount, M.MetaKeyword AS MetaKeyword, M.MetaDescription AS MetaDescription, " +
            // For Product Custom Price
                "    IsCustomPrice, DefaultPrice, MinimumPrice, " +
            // For QuickBook
                "    QBListID, IsQBExported " +
                "FROM " +
                "    ((((Product INNER JOIN " +
                "        (SELECT ProductID, " +
                "            SUM( Stock ) AS SumStock FROM ProductStock GROUP By ProductID) AS S " +
                "        ON Product.ProductID = S.ProductID) " +
                "    LEFT JOIN (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "        ON Product.ProductID = L.ProductID) " +
            // For Product Custom Price
                "    LEFT JOIN ProductCustomPrice CP ON Product.ProductID = CP.ProductID) " +
            // for Recurring
                "    LEFT JOIN ProductRecurring E ON Product.ProductID = E.ProductID) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "    WHERE CultureID = @CultureID2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID ";


        private const string AllProductFields =
            " Product.ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, Manufacturer, " +
            " Model, ImageSecondary, SumStock, Weight, FixedShippingCost, ShippingCost, " +
            " PP.WholesalePrice, PP.Price, PP.RetailPrice, Keywords, Other1, Other2, Other3, Other4, Other5, " +
            " IsTodaySpecial, MinQuantity, MaxQuantity, IsDownloadable, DownloadPath, Product.IsEnabled, RelatedProducts," +
            " DiscountGroupID, PP.WholesalePrice2, PP.WholesalePrice3, " +
            " PP.UseDefaultPrice, PP.UseDefaultRetailPrice, PP.UseDefaultWholeSalePrice, PP.UseDefaultWholeSalePrice2, PP.UseDefaultWholeSalePrice3, " +
            " IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, IsParentVisible, UseInventory ," +
            " M.MetaKeyword AS MetaKeyword, M.MetaDescription AS MetaDescription, M.UseDefaultMetaKeyword AS UseDefaultMetaKeyword, M.UseDefaultMetaDescription AS UseDefaultMetaDescription, " +
            " QBListID, IsQBExported, IsCustomPrice, IsCallForPrice, Brand, ReturnTime, IsProductKit, IsDynamicProductKitPrice, IsDynamicProductKitWeight, CreateDate,ViewCount, " +
            " Length, Width, Height ";

        private CategoryHelperRepository _categoryHelper = new CategoryHelperRepository( "Category" );
        private CategoryHelperRepository _departmentHelper = new CategoryHelperRepository( "Department" );
        private IList<Product> ExecuteQuickSearch(
            Culture culture,
            string searchTerm,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootCategoryID,
            int startIndex,
            int endIndex,
            string searchMode
            )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterString( rootCategoryID ) );

            string query = "SELECT ROW_NUMBER() over ( ORDER BY " + sortBy + ") as rownum, " +
                    " * FROM (" +
                    "SELECT DISTINCT Product.ProductID, Name, ShortDescription, LongDescription, UrlName, " +
                    "    Sku, Model, ImageSecondary, SumStock, Weight, MinQuantity, MaxQuantity, " +
                    "    FixedShippingCost, ShippingCost, Manufacturer, PP.WholesalePrice, PP.Price, " +
                    "    PP.RetailPrice, Keywords, IsDownloadable, DownloadPath, " +
                    "    Product.IsEnabled, PP.WholesalePrice2, PP.WholesalePrice3, " +
                    "    PP.UseDefaultPrice, PP.UseDefaultRetailPrice, PP.UseDefaultWholeSalePrice, PP.UseDefaultWholeSalePrice2, PP.UseDefaultWholeSalePrice3, " +
                    "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                    "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, " +
                    "    UseInventory, Product.DiscountGroupID As ProductDiscountGroupID, " +
                    "    D.DiscountGroupID As CategoryDiscountGroupID, IsAffiliate, IsCallForPrice, Brand, ReturnTime, IsProductKit, IsDynamicProductKitPrice, IsDynamicProductKitWeight, CreateDate,ViewCount, " +
                // For Product Custom Price
                    "    IsCustomPrice, DefaultPrice, MinimumPrice, " +
                // Edit for Recurring.
                    "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                    "    RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                // Product Locale
                    "    M.MetaKeyword AS MetaKeyword, M.MetaDescription AS MetaDescription, M.UseDefaultMetaKeyword AS UseDefaultMetaKeyword, M.UseDefaultMetaDescription AS UseDefaultMetaDescription " +
                    "FROM " +
                    "    (((((( (Product LEFT JOIN ProductLocale L ON Product.ProductID = L.ProductID) " +
                    "    LEFT JOIN ProductCategory AS C ON Product.ProductID = C.ProductID ) " +
                    "    INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock " +
                    "    GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                    "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                    "    LEFT JOIN Category D ON C.CategoryID = D.CategoryID ) " +
                    "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID) " +
                // For Product Custom Price
                    "    LEFT JOIN ProductCustomPrice CP ON Product.ProductID = CP.ProductID) " +
                    "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                    "    WHERE CultureID = @CultureID AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
                    "WHERE L.CultureID = @CultureID2 " +
                    "AND Product.IsEnabled = @IsEnabled " +
                    "AND IsParentVisible = @IsParentVisible " +
                    "AND D.RootID = @RootID " +
                    ConditionSearchField( searchTerm, searchFieldList, parameters, searchMode ) +
                    ") AS Product ";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery, DataAccess.ConvertArrayListToDbParameters( parameters ) );

            IList<Product> productList = ConvertProductTableToList( culture, table, storeID );
            IList<Product> newProductList = new List<Product>();

            foreach (Product product in productList)
            {
                if (IsCategoryVisible( product.ProductID, rootCategoryID, storeID, culture ))
                {
                    newProductList.Add( product );
                }
            }

            return newProductList;
        }

        private string ConditionSearchField(
            string searchTerm,
            string[] searchFieldList,
            ArrayList parameters,
            string searchType )
        {
            string query = "";

            if (!String.IsNullOrEmpty( searchTerm ))
            {
                switch (searchType)
                {
                    case "all":
                        query = GetAllWordsSearch( searchTerm, searchFieldList, parameters );
                        break;
                    case "exact":
                        query = GetExactPhaseSearch( searchTerm, searchFieldList, parameters );
                        break;
                    default:
                        query = GetAnyWordSearch( searchTerm, searchFieldList, parameters );
                        break;
                }

                return query;
            }
            else
                return "";
        }

        private string GetAllWordsSearch( string searchTerm, string[] searchFieldList, ArrayList parameters )
        {
            string[] arKeys = StringUtilities.SplitString( searchTerm, new char[] { ' ' } );
            string query = " AND (";

            for (int i = 0; i < searchFieldList.Length; i++)
            {
                for (int j = 0; j < arKeys.Length; j++)
                {
                    if (j == 0)
                    {
                        query += " ( ";
                    }
                    else
                    {
                        query += " AND ";
                    }

                    string strColumn = searchFieldList[i];
                    query += CreateColumnName( strColumn );

                    parameters.Add( DataAccess.CreateParameterString( "%" + arKeys[j] + "%" ) );
                    query += " LIKE @" + strColumn + j;

                    if (j == arKeys.Length - 1)
                    {
                        query += " ) ";
                    }
                }

                if (i != searchFieldList.Length - 1)
                {
                    query += " OR ";
                }
            }

            query += " ) ";

            return query;
        }

        private string GetExactPhaseSearch( string searchTerm, string[] searchFieldList, ArrayList parameters )
        {
            string query = " AND (";

            for (int i = 0; i < searchFieldList.Length; i++)
            {
                string strColumn = searchFieldList[i];

                if (i == 0)
                    query += " ";
                else
                    query += " OR ";

                query += CreateColumnName( strColumn );

                parameters.Add( DataAccess.CreateParameterString( "%" + searchTerm + "%" ) );
                query += " LIKE @" + strColumn;
            }

            query += " ) ";

            return query;
        }

        private string GetAnyWordSearch( string searchTerm, string[] searchFieldList, ArrayList parameters )
        {
            string[] arKeys = StringUtilities.SplitString( searchTerm, new char[] { ' ', '@', ';', '\'' } );
            string query = "";

            for (int i = 0; i < arKeys.Length; i++)
            {
                if (i == 0)
                {
                    query = " AND (";
                }

                for (int j = 0; j < searchFieldList.Length; j++)
                {
                    string strColumn = searchFieldList[j];
                    if (j == 0 && i == 0)
                        query += " ";
                    else
                        query += " OR ";

                    query += CreateColumnName( strColumn );

                    parameters.Add( DataAccess.CreateParameterString( "%" + arKeys[i] + "%" ) );
                    query += " LIKE @" + strColumn + i;
                }

                if (i == arKeys.Length - 1)
                    query += " ) ";
            }

            return query;
        }

        private string EnabledCategoryListCondition( Culture culture, string rootID )
        {
            IList<Category> categoryList = GetEnabledLeafCategoryList( culture, rootID );

            string categoryListCondition = " and ( category.categoryid = 0 ) ";

            if (categoryList.Count > 0)
            {
                categoryListCondition = " and ( ";

                for (int i = 0; i < categoryList.Count; i++)
                {
                    categoryListCondition += " category.categoryid = " + categoryList[i].CategoryID;

                    if (i != categoryList.Count - 1)
                        categoryListCondition += " or ";
                }

                categoryListCondition += " ) ";
            }

            return categoryListCondition;
        }

        private string EnabledDepartmentListCondition( Culture culture, string rootID )
        {
            IList<Department> departmentList = GetEnabledLeafDepartmentList( culture, rootID );

            string departmentListCondition = " and ( department.departmentid = 0 ) ";

            if (departmentList.Count > 0)
            {
                departmentListCondition = " and ( ";

                for (int i = 0; i < departmentList.Count; i++)
                {
                    departmentListCondition += " department.departmentid = " + departmentList[i].DepartmentID;

                    if (i != departmentList.Count - 1)
                        departmentListCondition += " or ";
                }

                departmentListCondition += " ) ";
            }

            return departmentListCondition;
        }

        private IList<Category> GetEnabledLeafCategoryList( Culture culture, string rootID )
        {
            IList<Category> categoryList =
                DataAccessContext.CategoryRepository.GetAll( culture, "CategoryID", BoolFilter.ShowTrue );

            IList<Category> enabledLeafCategoryList = new List<Category>();

            foreach (Category category in categoryList)
            {
                if (DataAccessContext.CategoryRepository.IsLeaf( category.CategoryID ))
                {
                    if (IsParentCategoryVisible( category, rootID, culture ))
                        enabledLeafCategoryList.Add( category );
                }
            }

            return enabledLeafCategoryList;
        }

        private IList<Department> GetEnabledLeafDepartmentList( Culture culture, string rootID )
        {
            IList<Department> departmentList =
                DataAccessContext.DepartmentRepository.GetAll( culture, "DepartmentID", BoolFilter.ShowTrue );

            IList<Department> enabledLeafDepartmentList = new List<Department>();

            foreach (Department department in departmentList)
            {
                if (DataAccessContext.DepartmentRepository.IsLeaf( department.DepartmentID ))
                {
                    if (IsParentDepartmentVisible( department, rootID, culture ))
                        enabledLeafDepartmentList.Add( department );
                }
            }

            return enabledLeafDepartmentList;
        }

        private string CreateColumnName( string columnName )
        {
            if (String.Compare( columnName, "Other1", true ) == 0 ||
                String.Compare( columnName, "Other2", true ) == 0 ||
                String.Compare( columnName, "Other3", true ) == 0 ||
                String.Compare( columnName, "Other4", true ) == 0 ||
                String.Compare( columnName, "Other5", true ) == 0)
            {
                return "Product." + columnName;
            }
            else
            {
                return columnName;
            }
        }

        private string ConditionCategoryID( string categoryIDList, ArrayList parameters )
        {
            if (!String.IsNullOrEmpty( categoryIDList ))
            {
                string query = " AND C.CategoryID IN (";

                string[] categoryIDs = categoryIDList.Trim().Split( ' ' );
                for (int i = 0; i < categoryIDs.Length; i++)
                {
                    if (!string.IsNullOrEmpty( categoryIDs[i].Trim() ))
                    {
                        query += "@CategoryID" + i + ",";
                        parameters.Add( DataAccess.CreateParameterString( categoryIDs[i] ) );
                    }
                }
                query = query.TrimEnd( ',' ) + ") ";

                return query;
            }
            else
                return "";
        }

        private string ConditionDepartmentID( string departmentIDList, ArrayList parameters )
        {
            if (!String.IsNullOrEmpty( departmentIDList ))
            {
                string query = " AND D.DepartmentID IN (";

                string[] departmentIDs = departmentIDList.Trim().Split( ' ' );
                for (int i = 0; i < departmentIDs.Length; i++)
                {
                    if (!string.IsNullOrEmpty( departmentIDs[i].Trim() ))
                    {
                        query += "@DepartmentID" + i + ",";
                        parameters.Add( DataAccess.CreateParameterString( departmentIDs[i] ) );
                    }
                }
                query = query.TrimEnd( ',' ) + ") ";

                return query;
            }
            else
                return "";
        }

        private string ConditionPrice( string price1, string price2, string prefix, ArrayList parameters )
        {
            if (!String.IsNullOrEmpty( price1 ) && String.IsNullOrEmpty( price2 ))
            {
                parameters.Add( DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( price1 ) ) );

                return " AND " + prefix + "[Price] >= @Price1 ";
            }
            else if (String.IsNullOrEmpty( price1 ) && !String.IsNullOrEmpty( price2 ))
            {
                parameters.Add( DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( price2 ) ) );

                return " AND " + prefix + "[Price] <= @Price2 ";
            }
            else if (!String.IsNullOrEmpty( price1 ) && !String.IsNullOrEmpty( price2 ))
            {
                parameters.Add( DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( price1 ) ) );
                parameters.Add( DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( price2 ) ) );

                return " AND " + prefix + "[Price] BETWEEN @Price1 AND @Price2 ";
            }
            else
                return "";
        }

        private IList<Product> ExecuteAdvancedSearch(
            Culture culture,
            string categoryIDs,
            string departmentIDs,
            string manufacturerID,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootCategoryID,
            string rootDepartmentID,
            int startIndex,
            int endIndex,
            bool isIncludeProductKit,
            string searchType
            )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( rootCategoryID ) );
            parameters.Add( DataAccess.CreateParameterString( rootDepartmentID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            string conditionCategory = string.Empty;
            if (!String.IsNullOrEmpty( categoryIDs ))
            {
                conditionCategory = " AND PC.CategoryID IN (@CategoryID) ";
                parameters.Add( DataAccess.CreateParameterString( categoryIDs ) );
            }

            string conditionDepartment = String.Empty;
            if (!String.IsNullOrEmpty( departmentIDs ))
            {
                conditionDepartment = " AND PD.DepartmentID IN (@DepartmentID) ";
                parameters.Add( DataAccess.CreateParameterString( departmentIDs ) );
            }

            string conditionManufacturer = String.Empty;
            if (!String.IsNullOrEmpty( manufacturerID ))
            {
                conditionManufacturer = " AND Product.Manufacturer = @ManufacturerID ";
                parameters.Add( DataAccess.CreateParameterString( manufacturerID ) );
            }

            string query = "SELECT ROW_NUMBER() over ( ORDER BY " + sortBy + ") as rownum, " +
                    " * FROM ( " +
                    " SELECT DISTINCT Product.ProductID, Product.Sku, Product.MinQuantity, Product.MaxQuantity, Product.IsCustomPrice, Product.IsCallForPrice, " +
                    " Product.FreeShippingCost, Product.IsGiftCertificate, Product.UseInventory, Product.IsFixedPrice, Product.DiscountGroupID as ProductDiscountGroupID, " +
                    " PL.Name, PL.UrlName, PP.Price, " +
                    " PP.RetailPrice,PP.WholesalePrice,PP.WholesalePrice2,PP.WholesalePrice3, PP.UseDefaultPrice, " +
                    " PP.UseDefaultRetailPrice, PP.UseDefaultWholeSalePrice, PP.UseDefaultWholeSalePrice2,PP.UseDefaultWholeSalePrice3, " +
                    " PM.MetaKeyword, PM.MetaDescription " +
                    " FROM Product " +
                    " INNER JOIN (SELECT Category.CategoryID, ProductCategory.ProductID FROM Category, ProductCategory " +
                    " WHERE RootID = @RootCategoryID AND Category.CategoryID = ProductCategory.CategoryID) as PC  " +
                    " ON Product.ProductID = PC.ProductID " +
                    " LEFT JOIN (SELECT Department.DepartmentID, ProductDepartment.ProductID FROM Department, ProductDepartment " +
                    " WHERE RootID = @RootDepartmentID AND Department.DepartmentID = ProductDepartment.DepartmentID) as PD " +
                    " ON Product.ProductID = PD.ProductID " +
                    " LEFT JOIN (SELECT * FROM ProductLocale WHERE cultureID = @CultureID) as PL " +
                    " ON Product.ProductID = PL.ProductID " +
                    " LEFT JOIN (SELECT * FROM ProductMetaInformation WHERE CultureID = @CultureID1 AND StoreID = @StoreID ) as PM " +
                    " ON Product.ProductID = PM.ProductID " +
                    " LEFT JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP " +
                    " ON Product.ProductID = PP.ProductID " +
                    " WHERE Product.IsEnabled = @IsEnabled AND Product.IsParentVisible = @IsParentVisible " +
                    conditionCategory +
                    conditionDepartment +
                    conditionManufacturer +
                    ConditionPrice( price1, price2, "PP.", parameters ) +
                    ConditionSearchField( searchTerm, searchFieldList, parameters, searchType ) +
                    ") AS Product";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery,
            DataAccess.ConvertArrayListToDbParameters( parameters ) );

            IList<Product> productList = ConvertProductTableToList( culture, table, storeID );

            IList<Product> newProductList = new List<Product>();

            foreach (Product product in productList)
            {
                if (IsCategoryVisible( product.ProductID, rootCategoryID, storeID, culture ))
                {
                    if (!isIncludeProductKit)
                    {
                        if (!product.IsProductKit)
                            newProductList.Add( product );
                    }
                    else
                        newProductList.Add( product );
                }
            }

            return newProductList;
        }

        private IList<Product> ExecuteAdvancedSearch(
            Culture culture,
            string categoryIDs,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootCategoryID,
            int startIndex,
            int endIndex,
            bool isIncludeProductKit,
            string searchType
            )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterString( rootCategoryID ) );

            string query = "SELECT ROW_NUMBER() over ( ORDER BY " + sortBy + ") as rownum, " +
                    " * FROM (" +
                    "SELECT DISTINCT Product.ProductID, Name, ShortDescription, LongDescription, UrlName, " +
                    "    Sku, Model," +
                    "    ImageSecondary, SumStock, Weight, FixedShippingCost, FreeShippingCost, ShippingCost, Manufacturer, " +
                    "    C.WholesalePrice, C.Price, C.RetailPrice, Keywords, MinQuantity, MaxQuantity, " +
                    "    Product.IsEnabled, " +
                    "    C.WholesalePrice2, C.WholesalePrice3, " +
                    "    C.UseDefaultPrice, C.UseDefaultRetailPrice, C.UseDefaultWholeSalePrice, C.UseDefaultWholeSalePrice2, C.UseDefaultWholeSalePrice3, " +
                    "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, " +
                    "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                    "    D.DiscountGroupID As CategoryDiscountGroupID, " +
                    "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, IsAffiliate, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                    "    IsDynamicProductKitPrice, IsDynamicProductKitWeight,CreateDate,ViewCount, " +
                // For Product Custom Price
                    "    IsCustomPrice, DefaultPrice, MinimumPrice, " +
                // Edit for Recurring.
                    "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles ," +
                    "    RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                // Product Locale
                    "    M.MetaKeyword AS MetaKeyword, M.MetaDescription AS MetaDescription, M.UseDefaultMetaKeyword AS UseDefaultMetaKeyword, M.UseDefaultMetaDescription AS UseDefaultMetaDescription " +
                    "FROM " +
                    "    ((((( (Product LEFT JOIN ProductLocale L ON Product.ProductID = L.ProductID) " +
                    "    LEFT JOIN (SELECT PC.ProductID, CategoryID, SortOrder, StoreID, Price, RetailPrice, " +
                    "    WholesalePrice, WholesalePrice2, WholesalePrice3, " +
                    "    UseDefaultPrice, UseDefaultRetailPrice, UseDefaultWholeSalePrice, UseDefaultWholeSalePrice2, UseDefaultWholeSalePrice3 " +
                    "    FROM (Productcategory PC INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP " +
                    "    ON PC.ProductID = PP.ProductID )) AS C ON Product.ProductID = C.ProductID ) " +
                    "    INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock " +
                    "    GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                    "    LEFT JOIN Category D ON C.CategoryID = D.CategoryID ) " +
                    "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID) " +
                // For Product Custom Price
                    "    LEFT JOIN ProductCustomPrice CP ON Product.ProductID = CP.ProductID) " +
                    "    LEFT JOIN (SELECT * FROM ProductMetaInformation AS N " +
                    "    WHERE N.CultureID = @CultureID AND N.StoreID = @StoreID2) AS M ON Product.ProductID = M.ProductID " +
                    "WHERE L.CultureID = @CultureID2 " +
                    "AND Product.IsEnabled = @IsEnabled " +
                    "AND IsParentVisible = @IsParentVisible " +
                    "AND D.RootID = @RootID " +
                    ConditionCategoryID( categoryIDs, parameters ) +
                    ConditionPrice( price1, price2, "C.", parameters ) +
                    ConditionSearchField( searchTerm, searchFieldList, parameters, searchType ) +
                    ") AS Product";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery,
            DataAccess.ConvertArrayListToDbParameters( parameters ) );

            IList<Product> productList = ConvertProductTableToList( culture, table, storeID );
            IList<Product> newProductList = new List<Product>();

            foreach (Product product in productList)
            {
                if (IsCategoryVisible( product.ProductID, rootCategoryID, storeID, culture ))
                {
                    if (!isIncludeProductKit)
                    {
                        if (!product.IsProductKit)
                            newProductList.Add( product );
                    }
                    else
                        newProductList.Add( product );
                }
            }

            return newProductList;
        }

        private IList<Product> ExecuteAdvancedSearchDepartment(
            Culture culture,
            string departmentIDs,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootDepartmentID,
            int startIndex,
            int endIndex,
            string searchType
            )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterString( rootDepartmentID ) );

            string query = "SELECT ROW_NUMBER() over ( ORDER BY " + sortBy + ") as rownum, " +
                    " * FROM (" +
                    "SELECT DISTINCT Product.ProductID, Name, ShortDescription, LongDescription, UrlName, " +
                    "    Sku, Model," +
                    "    ImageSecondary, SumStock, Weight, FixedShippingCost, FreeShippingCost, ShippingCost, Manufacturer, " +
                    "    C.WholesalePrice, C.Price, C.RetailPrice, Keywords, MinQuantity, MaxQuantity, " +
                    "    Product.IsEnabled, " +
                    "    C.WholesalePrice2, C.WholesalePrice3, " +
                    "    C.UseDefaultPrice, C.UseDefaultRetailPrice, C.UseDefaultWholeSalePrice, C.UseDefaultWholeSalePrice2, C.UseDefaultWholeSalePrice3, " +
                    "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, " +
                    "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                    "    D.DiscountGroupID As DepartmentDiscountGroupID, " +
                    "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                    "    IsDynamicProductKitPrice, IsDynamicProductKitWeight,CreateDate,ViewCount, " +
                // For Product Custom Price
                    "    IsCustomPrice, DefaultPrice, MinimumPrice, " +
                // Edit for Recurring.
                    "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles ," +
                    "    RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                // Product Locale
                    "    M.MetaKeyword AS MetaKeyword, M.MetaDescription AS MetaDescription, M.UseDefaultMetaKeyword AS UseDefaultMetaKeyword, M.UseDefaultMetaDescription AS UseDefaultMetaDescription " +
                    "FROM " +
                    "    ((((( (Product LEFT JOIN ProductLocale L ON Product.ProductID = L.ProductID) " +
                    "    LEFT JOIN (SELECT PC.ProductID, DepartmentID, SortOrder, StoreID, Price, RetailPrice, " +
                    "    WholesalePrice, WholesalePrice2, WholesalePrice3, " +
                    "    UseDefaultPrice, UseDefaultRetailPrice, UseDefaultWholeSalePrice, UseDefaultWholeSalePrice2, UseDefaultWholeSalePrice3 " +
                    "    FROM (Productdepartment PC INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP " +
                    "    ON PC.ProductID = PP.ProductID )) AS C ON Product.ProductID = C.ProductID ) " +
                    "    INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock " +
                    "    GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                    "    LEFT JOIN Department D ON C.DepartmentID = D.DepartmentID ) " +
                    "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID) " +
                // For Product Custom Price
                    "    LEFT JOIN ProductCustomPrice CP ON Product.ProductID = CP.ProductID) " +
                    "    LEFT JOIN (SELECT * FROM ProductMetaInformation AS N " +
                    "    WHERE N.CultureID = @CultureID AND N.StoreID = @StoreID2) AS M ON Product.ProductID = M.ProductID " +
                    "WHERE L.CultureID = @CultureID2 " +
                    "AND Product.IsEnabled = @IsEnabled " +
                    "AND IsParentVisible = @IsParentVisible " +
                    "AND D.RootID = @RootID " +
                    ConditionDepartmentID( departmentIDs, parameters ) +
                    ConditionPrice( price1, price2, "C.", parameters ) +
                    ConditionSearchField( searchTerm, searchFieldList, parameters, searchType ) +
                    ") AS Product";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery,
            DataAccess.ConvertArrayListToDbParameters( parameters ) );

            IList<Product> productList = ConvertProductTableToList( culture, table, storeID );
            IList<Product> newProductList = new List<Product>();

            foreach (Product product in productList)
            {
                if (IsDepartmentVisible( product.ProductID, rootDepartmentID, storeID, culture ))
                {
                    newProductList.Add( product );
                }
            }

            return newProductList;
        }

        private int CountAdvanceSearchResult(
            Culture culture,
            string categoryIDs,
            string departmentIDs,
            string manufacturerID,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootCategoryID,
            string rootDepartmentID,
            string searchType )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( rootCategoryID ) );
            parameters.Add( DataAccess.CreateParameterString( rootDepartmentID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );

            string conditionCategory = String.Empty;
            if (!String.IsNullOrEmpty( categoryIDs ))
            {
                conditionCategory = " AND PC.CategoryID IN (@CategoryID) ";
                parameters.Add( DataAccess.CreateParameterString( categoryIDs ) );
            }

            string conditionDepartment = String.Empty;
            if (!String.IsNullOrEmpty( departmentIDs ))
            {
                conditionDepartment = " AND PD.DepartmentID IN (@DepartmentID) ";
                parameters.Add( DataAccess.CreateParameterString( departmentIDs ) );
            }

            string conditionManufacturer = String.Empty;
            if (!String.IsNullOrEmpty( manufacturerID ))
            {
                conditionManufacturer = " AND Product.Manufacturer = @ManufacturerID ";
                parameters.Add( DataAccess.CreateParameterString( manufacturerID ) );
            }

            string query = "SELECT COUNT(*) FROM ( " +
                    " SELECT DISTINCT Product.ProductID, Product.Sku, Product.MinQuantity, Product.MaxQuantity, Product.IsCustomPrice, Product.IsCallForPrice, " +
                    " Product.FreeShippingCost, Product.IsGiftCertificate, Product.IsFixedPrice, Product.DiscountGroupID as ProductDiscountGroupID, " +
                    " PL.Name, PL.UrlName, PP.Price, " +
                    " PP.RetailPrice,PP.WholesalePrice,PP.WholesalePrice2,PP.WholesalePrice3, PP.UseDefaultPrice, " +
                    " PP.UseDefaultRetailPrice, PP.UseDefaultWholeSalePrice, PP.UseDefaultWholeSalePrice2,PP.UseDefaultWholeSalePrice3, " +
                    " PM.MetaKeyword, PM.MetaDescription " +
                    " FROM Product " +
                    " INNER JOIN (SELECT Category.CategoryID, ProductCategory.ProductID FROM Category, ProductCategory " +
                    " WHERE RootID = @RootCategoryID AND Category.CategoryID = ProductCategory.CategoryID) as PC  " +
                    " ON Product.ProductID = PC.ProductID " +
                    " LEFT JOIN (SELECT Department.DepartmentID, ProductDepartment.ProductID FROM Department, ProductDepartment " +
                    " WHERE RootID = @RootDepartmentID AND Department.DepartmentID = ProductDepartment.DepartmentID) as PD " +
                    " ON Product.ProductID = PD.ProductID " +
                    " LEFT JOIN (SELECT * FROM ProductLocale WHERE cultureID = @CultureID) as PL " +
                    " ON Product.ProductID = PL.ProductID " +
                    " LEFT JOIN (SELECT * FROM ProductMetaInformation WHERE CultureID = @CultureID1 AND StoreID = @StoreID ) as PM " +
                    " ON Product.ProductID = PM.ProductID " +
                    " LEFT JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP " +
                    " ON Product.ProductID = PP.ProductID " +
                    " WHERE Product.IsEnabled = @IsEnabled AND Product.IsParentVisible = @IsParentVisible " +
                    conditionCategory +
                    conditionDepartment +
                    conditionManufacturer +
                    ConditionPrice( price1, price2, "PP.", parameters ) +
                    ConditionSearchField( searchTerm, searchFieldList, parameters, searchType ) +
                    ") AS row";

            string result = DataAccess.ExecuteScalar( query,
                DataAccess.ConvertArrayListToDbParameters( parameters ) );

            return Convert.ToInt32( result );
        }

        private int CountAdvanceSearchResult(
            Culture culture,
            string categoryIDs,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootCategoryID,
            string searchType
            )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterString( rootCategoryID ) );

            string query = "SELECT COUNT(*) FROM ( " +
                    "SELECT DISTINCT Product.ProductID " +
                    "FROM " +
                    "    ((((( (Product LEFT JOIN ProductLocale L ON Product.ProductID = L.ProductID) " +
                    "    LEFT JOIN (SELECT PC.ProductID, CategoryID, SortOrder, StoreID, Price, RetailPrice, " +
                    "    WholesalePrice, WholesalePrice2, WholesalePrice3, " +
                    "    UseDefaultPrice, UseDefaultRetailPrice, UseDefaultWholeSalePrice, UseDefaultWholeSalePrice2, UseDefaultWholeSalePrice3 " +
                    "    FROM (Productcategory PC INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP " +
                    "    ON PC.ProductID = PP.ProductID )) AS C ON Product.ProductID = C.ProductID ) " +
                    "    INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock " +
                    "    GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                    "    LEFT JOIN Category D ON C.CategoryID = D.CategoryID ) " +
                    "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID) " +
                // For Product Custom Price
                    "    LEFT JOIN ProductCustomPrice CP ON Product.ProductID = CP.ProductID) " +
                    "    LEFT JOIN (SELECT * FROM ProductMetaInformation AS N " +
                    "    WHERE N.CultureID = @CultureID AND N.StoreID = @StoreID2) AS M ON Product.ProductID = M.ProductID " +
                    "WHERE L.CultureID = @CultureID2 " +
                    "AND Product.IsEnabled = @IsEnabled " +
                    "AND IsParentVisible = @IsParentVisible " +
                    "AND D.RootID = @RootID " +
                    ConditionCategoryID( categoryIDs, parameters ) +
                    ConditionPrice( price1, price2, "C.", parameters ) +
                    ConditionSearchField( searchTerm, searchFieldList, parameters, searchType ) +
                    " ) AS row";

            string result = DataAccess.ExecuteScalar( query,
                DataAccess.ConvertArrayListToDbParameters( parameters ) );

            return Convert.ToInt32( result );
        }

        public int CountQuickSearchResult(
            Culture culture,
            string searchTerm,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootCategoryID,
            string searchType
            )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterString( rootCategoryID ) );

            string query = "SELECT COUNT(*) FROM (" +
                    "SELECT DISTINCT Product.ProductID " +
                    "FROM " +
                    "    (((((( (Product LEFT JOIN ProductLocale L ON Product.ProductID = L.ProductID) " +
                    "    LEFT JOIN ProductCategory AS C ON Product.ProductID = C.ProductID ) " +
                    "    INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock " +
                    "    GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                    "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                    "    LEFT JOIN Category D ON C.CategoryID = D.CategoryID ) " +
                    "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID) " +
                // For Product Custom Price
                    "    LEFT JOIN ProductCustomPrice CP ON Product.ProductID = CP.ProductID) " +
                    "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                    "    WHERE CultureID = @CultureID AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
                    "WHERE L.CultureID = @CultureID2 " +
                    "AND Product.IsEnabled = @IsEnabled " +
                    "AND IsParentVisible = @IsParentVisible " +
                    "AND D.RootID = @RootID " +
                    ConditionSearchField( searchTerm, searchFieldList, parameters, searchType ) +
                    ") AS Row";

            string count = DataAccess.ExecuteScalar( query, DataAccess.ConvertArrayListToDbParameters( parameters ) );

            return Convert.ToInt32( count );
        }

        private int CountAdvanceSearchDepartmentResult(
            Culture culture,
            string departmentIDs,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootDepartmentID,
            string searchType
            )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterString( rootDepartmentID ) );

            string query = "SELECT COUNT(*) FROM ( " +
                    "SELECT DISTINCT Product.ProductID " +
                    "FROM " +
                    "    ((((( (Product LEFT JOIN ProductLocale L ON Product.ProductID = L.ProductID) " +
                    "    LEFT JOIN (SELECT PC.ProductID, DepartmentID, SortOrder, StoreID, Price, RetailPrice, " +
                    "    WholesalePrice, WholesalePrice2, WholesalePrice3, " +
                    "    UseDefaultPrice, UseDefaultRetailPrice, UseDefaultWholeSalePrice, UseDefaultWholeSalePrice2, UseDefaultWholeSalePrice3 " +
                    "    FROM (Productdepartment PC INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP " +
                    "    ON PC.ProductID = PP.ProductID )) AS C ON Product.ProductID = C.ProductID ) " +
                    "    INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock " +
                    "    GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                    "    LEFT JOIN Department D ON C.DepartmentID = D.DepartmentID ) " +
                    "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID) " +
                // For Product Custom Price
                    "    LEFT JOIN ProductCustomPrice CP ON Product.ProductID = CP.ProductID) " +
                    "    LEFT JOIN (SELECT * FROM ProductMetaInformation AS N " +
                    "    WHERE N.CultureID = @CultureID AND N.StoreID = @StoreID2) AS M ON Product.ProductID = M.ProductID " +
                    "WHERE L.CultureID = @CultureID2 " +
                    "AND Product.IsEnabled = @IsEnabled " +
                    "AND IsParentVisible = @IsParentVisible " +
                    "AND D.RootID = @RootID " +
                    ConditionDepartmentID( departmentIDs, parameters ) +
                    ConditionPrice( price1, price2, "C.", parameters ) +
                    ConditionSearchField( searchTerm, searchFieldList, parameters, searchType ) +
                    " ) AS row";

            string result = DataAccess.ExecuteScalar( query,
                DataAccess.ConvertArrayListToDbParameters( parameters ) );

            return Convert.ToInt32( result );
        }

        private UrlNameAccessHelper _urlNameHelper = new UrlNameAccessHelper( "ProductLocale", "ProductID" );

        private ProductGiftCertificateAccess _productGiftCertificateAccess = new ProductGiftCertificateAccess();
        private ProductRecurringAccess _productRecurringAccess = new ProductRecurringAccess();
        private ProductCustomPriceAccess _productCustomPriceAccess = new ProductCustomPriceAccess();

        private ProductCategoryAccess _productCategoryAccess = new ProductCategoryAccess();
        private ProductDepartmentAccess _productDepartmentAccess = new ProductDepartmentAccess();
        private ProductOptionGroupAccess _productOptionGroupAccess = new ProductOptionGroupAccess();
        private ProductStockAccess _productStockAccess = new ProductStockAccess();
        private ProductShippingCostAccess _productShippingCostAccess = new ProductShippingCostAccess();
        private ProductImageAccess _productImageAccess = new ProductImageAccess();
        private CustomerReviewAccess _customerReviewAccess = new CustomerReviewAccess();
        private ProductPriceAccess _productPriceAccess = new ProductPriceAccess();
        private ProductSpecificationAccess _productSpecificationAccess = new ProductSpecificationAccess();
        private ProductKitAccess _productKitAccess = new ProductKitAccess();
        private ProductKitItemAccess _productKitItemAccess = new ProductKitItemAccess();

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "productid":
                    return "Product.ProductID";
                case "stock":
                    return "SumStock";

                default:
                    return fieldName;
            }
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "productid" ))
                return " ORDER BY Product." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "sortorder" ))
                return " ORDER BY C." + sortBy.Trim() + " ";
            else
                return " ORDER BY " + sortBy;
        }

        private string CreateAdvancedSearchSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "name" ))
                return " ORDER BY L." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "sortorder" ))
                return " ORDER BY C." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "price" ))
                return " ORDER BY C." + sortBy.Trim() + " ";
            else
                return " ORDER BY " + sortBy;
        }

        private string CreateCategoryInnerJoinSql( string categoryID, string rootID )
        {
            if (String.IsNullOrEmpty( categoryID ) && String.IsNullOrEmpty( rootID ))
                return String.Empty;

            string sqlString = String.Empty;

            if (!String.IsNullOrEmpty( rootID ))
                sqlString = "INNER JOIN (SELECT CategoryID, RootID FROM Category WHERE RootID = " + rootID +
                    " ) AS D ON PC.CategoryID = D.CategoryID ";
            else
                sqlString = "INNER JOIN (SELECT CategoryID, RootID FROM Category ) AS D ON PC.CategoryID = D.CategoryID ";

            if (!String.IsNullOrEmpty( categoryID ))
                return " INNER JOIN ( (SELECT DISTINCT * FROM ProductCategory WHERE CategoryID = " + categoryID + " ) AS PC " +
                    sqlString + " ) " + " ON Product.ProductID = PC.ProductID ";
            else
                return " INNER JOIN ( (SELECT DISTINCT * FROM ProductCategory ) AS PC " + sqlString + " ) " +
                    " ON Product.ProductID = PC.ProductID ";
        }

        private string GetCultureID( Culture culture )
        {
            return culture.CultureID;
        }

        private Product CreateOneProductFromRow( Culture culture, DataRow row, string storeID )
        {
            ProductFactory factory = new ProductFactory();
            factory.DefaultCulture = culture;
            factory.BaseData = DataAccessConverter.ConvertDataRowToHashtable( row );
            factory.ParentCategoryIDs = _productCategoryAccess.GetByProductID( row["ProductID"].ToString() );
            factory.ParentDepartmentIDs = _productDepartmentAccess.GetByProductID( row["ProductID"].ToString() );

            if (factory.RequiresGiftCertificateData)
            {
                factory.GiftCertificateData =
                    _productGiftCertificateAccess.GetDetails( row["ProductID"].ToString() );
            }

            return factory.CreateProduct( storeID );
        }

        private Product CreateOneLargeProductFromRow( Culture culture, DataRow row, string storeID )
        {
            ProductFactory factory = new ProductFactory();
            factory.DefaultCulture = culture;
            factory.BaseData = DataAccessConverter.ConvertDataRowToHashtable( row );
            //      factory.ParentCategoryIDs = _productCategoryAccess.GetByProductID(row["ProductID"].ToString());
            //     factory.ParentDepartmentIDs = _productDepartmentAccess.GetByProductID(row["ProductID"].ToString());

            if (factory.RequiresGiftCertificateData)
            {
                factory.GiftCertificateData =
                    _productGiftCertificateAccess.GetDetails( row["ProductID"].ToString() );
            }

            return factory.CreateProduct( storeID );
        }
        private bool CheckParentCategoryVisibility( string categoryID )
        {
            if (categoryID == "0")
                return true;

            Category category = DataAccessContext.CategoryRepository.GetOne( Culture.Null, categoryID );

            if (category == null)
            {
                return false;
            }
            else if (!category.IsEnabled)
            {
                return false;
            }
            else if (category.ParentCategoryID == "0")
            {
                return category.IsEnabled;
            }
            else
            {
                return CheckParentCategoryVisibility( category.ParentCategoryID );
            }
        }

        private bool CheckParentDepartmentVisibility( string departmentID )
        {
            if (departmentID == "0")
                return true;

            Department department = DataAccessContext.DepartmentRepository.GetOne( Culture.Null, departmentID );

            if (department == null)
            {
                return false;
            }
            else if (!department.IsEnabled)
            {
                return false;
            }
            else if (department.ParentDepartmentID == "0")
            {
                return department.IsEnabled;
            }
            else
            {
                return CheckParentDepartmentVisibility( department.ParentDepartmentID );
            }
        }

        private bool EvaluateParentVisibility( string productID )
        {
            foreach (string categoryID in _productCategoryAccess.GetByProductID( productID ))
            {
                // This product is visible if any of the parent categories is visible.

                if (CheckParentCategoryVisibility( categoryID ))
                    return true;
            }

            return false;
        }

        private bool EvaluateParentVisibility( IList<string> categoryIDs, ArrayList categoryVisibleIDList )
        {
            foreach (string categoryID in categoryIDs)
            {
                // This product is visible if any of the parent categories is visible.
                if (categoryVisibleIDList.Contains( categoryID ))
                    return true;
            }

            return false;
        }

        private void RefreshIsParentVisible( string productID )
        {
            bool isParentVisible = EvaluateParentVisibility( productID );

            DataAccess.ExecuteNonQuery(
                "UPDATE Product " +
                "SET IsParentVisible = @IsParentVisible " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterBool( isParentVisible ),
                DataAccess.CreateParameterString( productID ) );
        }

        private void RefreshIsParentVisible( string productID, bool isParentVisible )
        {
            //     bool isParentVisible = EvaluateParentVisibility(productID);

            DataAccess.ExecuteNonQuery(
                "UPDATE Product " +
                "SET IsParentVisible = @IsParentVisible " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterBool( isParentVisible ),
                DataAccess.CreateParameterString( productID ) );
        }

        private void RefreshIsParentVisible( string productID, IList<string> categoryIDs, ArrayList categoryVisibleIDList )
        {
            bool isParentVisible = EvaluateParentVisibility( categoryIDs, categoryVisibleIDList );

            DataAccess.ExecuteNonQuery(
                "UPDATE Product " +
                "SET IsParentVisible = @IsParentVisible " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterBool( isParentVisible ),
                DataAccess.CreateParameterString( productID ) );
        }

        private bool IsParentCategoryVisible( Category category, string rootID, Culture culture )
        {
            if (category.RootID == rootID)
            {
                if (category.IsEnabled)
                {
                    if (category.ParentCategoryID != rootID)
                    {
                        Category parentCategory = DataAccessContext.CategoryRepository.GetOne( culture, category.ParentCategoryID );
                        if (IsParentCategoryVisible( parentCategory, rootID, culture ))
                            return true;
                    }
                    else
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        private bool IsParentDepartmentVisible( Department department, string rootID, Culture culture )
        {
            if (department.RootID == rootID)
            {
                if (department.IsEnabled)
                {
                    if (department.ParentDepartmentID != rootID)
                    {
                        Department parentDepartment = DataAccessContext.DepartmentRepository.GetOne( culture, department.ParentDepartmentID );
                        if (IsParentDepartmentVisible( parentDepartment, rootID, culture ))
                            return true;
                    }
                    else
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        private bool IsCategoryVisible( string productID, string rootID, string storeID, Culture culture )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( culture, productID, storeID );
            IList<string> categoryIDs = product.CategoryIDs;

            foreach (string categoryID in categoryIDs)
            {
                Category category = DataAccessContext.CategoryRepository.GetOne( culture, categoryID );
                if (IsParentCategoryVisible( category, rootID, culture ))
                    return true;
            }

            return false;
        }

        private bool IsDepartmentVisible( string productID, string rootID, string storeID, Culture culture )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( culture, productID, storeID );
            IList<string> departmentIDs = product.DepartmentIDs;

            foreach (string departmentID in departmentIDs)
            {
                Department department = DataAccessContext.DepartmentRepository.GetOne( culture, departmentID );
                if (IsParentDepartmentVisible( department, rootID, culture ))
                    return true;
            }

            return false;
        }

        private void LocalesSaveAll( string productID, LocaleCollection<ProductLocale> productLocales )
        {
            productLocales.Load();
            DeleteLocales( productID );
            InsertLocales( productID, productLocales );
        }

        private void MetaInformationsSaveAll( string productID, MetaInformationCollection<ProductMetaInformation> productMetaInformations )
        {
            productMetaInformations.Load();
            DeleteMetaInformations( productID );
            InsertMetaInformations( productID, productMetaInformations );
        }

        private void DeleteLocales( string productID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductLocale " +
                "WHERE ProductID = @ProdutID; ",
                DataAccess.CreateParameterString( productID ) );
        }

        private void DeleteMetaInformations( string productID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductMetaInformation " +
                "WHERE ProductID = @ProdutID; ",
                DataAccess.CreateParameterString( productID ) );
        }


        private void InsertLocales( string productID, LocaleCollection<ProductLocale> productLocales )
        {
            foreach (ProductLocale locale in productLocales)
            {
                InsertLocale( productID, locale );
            }
        }

        private void InsertLocale( string productID, ProductLocale locale )
        {
            locale.UrlName = _urlNameHelper.CreateLocaleUrlName( locale.Name );

            DataAccess.ExecuteNonQuery(
                    "INSERT INTO ProductLocale (ProductID, CultureID, " +
                    "    Name, ShortDescription, LongDescription, UrlName " +
                    "     ) " +
                    "VALUES (@ProductID, @CultureID, " +
                    "    @Name, @ShortDescription, @LongDescription, @UrlName " +
                    "     ); ",

                    DataAccess.CreateParameterString( productID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.Name ),
                    DataAccess.CreateParameterString( locale.ShortDescription ),
                    DataAccess.CreateParameterString( locale.LongDescription ),
                    DataAccess.CreateParameterString( locale.UrlName )
                    );
        }

        private void InsertMetaInformations( string productID, LocaleCollection<ProductMetaInformation> productMetaInformations )
        {
            foreach (ProductMetaInformation meta in productMetaInformations)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO ProductMetaInformation (ProductID, CultureID,StoreID " +
                    "    ,MetaKeyword, MetaDescription , UseDefaultMetaKeyword, UseDefaultMetaDescription " +
                    "     ) " +
                    "VALUES (@ProductID, @CultureID, @StoreID, " +
                    "    @MetaKeyword, @MetaDescription ,@UseDefaultMetaKeyword, @UseDefaultMetaDescription" +
                    "     ); ",

                    DataAccess.CreateParameterString( productID ),
                    DataAccess.CreateParameterString( meta.CultureID ),
                    DataAccess.CreateParameterString( meta.StoreID ),
                    DataAccess.CreateParameterString( meta.MetaKeyword ),
                    DataAccess.CreateParameterString( meta.MetaDescription ),
                    DataAccess.CreateParameterBool( meta.UseDefaultMetaKeyword ),
                    DataAccess.CreateParameterBool( meta.UseDefaultMetaDescription )
                    );
            }
        }


        private Product CreateBaseData( Product product )
        {
            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO Product (Sku, " +
                "    Model, ImageSecondary, Weight, FixedShippingCost, FreeShippingCost, " +
                "    ShippingCost, Manufacturer, " +
                "    Keywords, IsTodaySpecial, MinQuantity, MaxQuantity, " +
                "    Other1, Other2, Other3, Other4, Other5, IsDownloadable, DownloadPath, " +
                "    IsEnabled, RelatedProducts, MerchantRating, DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, " +
                "    TaxClassID, UseInventory,IsAffiliate, IsRecurring, ProductDetailsLayoutPath, IsQBExported, IsCustomPrice, " +
                "    IsCallForPrice, Brand, ReturnTime, IsProductKit, IsDynamicProductKitPrice, IsDynamicProductKitWeight, CreateDate, ViewCount, " +
                "    Length, Width, Height)  " +
                "VALUES (@Sku, " +
                "    @Model, @ImageSecondary, @Weight, @FixedShippingCost, @FreeShippingCost, " +
                "    @ShippingCost, @Manufacturer, " +
                "    @Keywords, @IsTodaySpecial, @MinQuantity, @MaxQuantity, " +
                "    @Other1, @Other2, @Other3, @Other4, @Other5, @IsDownloadable, @DownloadPath, " +
                "    @IsEnabled, @RelatedProducts, @MerchantRating, @DiscountGroupID, " +
                "    @IsGiftcertificate, @IsFixedPrice, @ManufacturerPartNumber, @Upc, " +
                "    @TaxClassID, @UseInventory, @IsAffiliate, @IsRecurring, @ProductDetailsLayoutPath, @IsQBExported, @IsCustomPrice, " +
                "    @IsCallForPrice, @Brand, @ReturnTime, @IsProductKit, @IsDynamicProductKitPrice, @IsDynamicProductKitWeight, @CreateDate, @ViewCount, " +
                "    @Length, @Width, @Height);" +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( product.Sku ),
                DataAccess.CreateParameterString( product.Model ),
                DataAccess.CreateParameterString( product.ImageSecondary ),
                DataAccess.CreateParameterDouble( product.Weight ),
                DataAccess.CreateParameterBool( product.FixedShippingCost ),
                DataAccess.CreateParameterBool( product.FreeShippingCost ),
                DataAccess.CreateParameterDecimalMoney( product.ShippingCost ),
                DataAccess.CreateParameterString( product.Manufacturer ),
                DataAccess.CreateParameterString( product.Keywords ),
                DataAccess.CreateParameterBool( product.IsTodaySpecial ),
                DataAccess.CreateParameterInt32( product.MinQuantity ),
                DataAccess.CreateParameterInt32( product.MaxQuantity ),
                DataAccess.CreateParameterString( product.Other1 ),
                DataAccess.CreateParameterString( product.Other2 ),
                DataAccess.CreateParameterString( product.Other3 ),
                DataAccess.CreateParameterString( product.Other4 ),
                DataAccess.CreateParameterString( product.Other5 ),
                DataAccess.CreateParameterBool( product.IsDownloadable ),
                DataAccess.CreateParameterString( product.DownloadPath ),
                DataAccess.CreateParameterBool( product.IsEnabled ),
                DataAccess.CreateParameterString( product.RelatedProducts ),
                DataAccess.CreateParameterDouble( product.MerchantRating ),
                DataAccess.CreateParameterString( product.DiscountGroupID ),
                DataAccess.CreateParameterBool( product.IsGiftCertificate ),
                DataAccess.CreateParameterBool( product.IsFixedPrice ),
                DataAccess.CreateParameterString( product.ManufacturerPartNumber ),
                DataAccess.CreateParameterString( product.Upc ),
                DataAccess.CreateParameterString( product.TaxClassID ),
                DataAccess.CreateParameterBool( product.UseInventory ),
                DataAccess.CreateParameterBool( product.IsAffiliate ),
                DataAccess.CreateParameterBool( product.IsRecurring ),
                DataAccess.CreateParameterString( product.ProductDetailsLayoutPath ),
                DataAccess.CreateParameterBool( false ),
                DataAccess.CreateParameterBool( product.IsCustomPrice ),
                DataAccess.CreateParameterBool( product.IsCallForPrice ),
                DataAccess.CreateParameterString( product.Brand ),
                DataAccess.CreateParameterInt32( product.ReturnTime ),
                DataAccess.CreateParameterBool( product.IsProductKit ),
                DataAccess.CreateParameterBool( product.IsDynamicProductKitPrice ),
                DataAccess.CreateParameterBool( product.IsDynamicProductKitWeight ),
                DataAccess.CreateParameterDateTime( product.CreateDate ),
                DataAccess.CreateParameterInt32( product.ViewCount ),
                DataAccess.CreateParameterDouble( product.Length ),
                DataAccess.CreateParameterDouble( product.Width ),
                DataAccess.CreateParameterDouble( product.Height )
                );

            product.ProductID = newID;

            LocalesSaveAll( product.ProductID, product.Locales );
            MetaInformationsSaveAll( product.ProductID, product.MetaInformations );

            _productOptionGroupAccess.SaveAll( product.ProductID, product.ProductOptionGroups );

            _productCategoryAccess.SaveAll( product.ProductID, product.CategoryIDs );
            _productDepartmentAccess.Create( product.ProductID, product.DepartmentIDs );

            if (product.ProductStocks.Count == 0)
                product.ProductStocks.Add( new ProductStock() );
            _productStockAccess.SaveAll( product.ProductID, product.ProductStocks );

            _productImageAccess.SaveAll( product.ProductID, product.ProductImages );
            _productShippingCostAccess.SaveAll( product.ProductID, product.ProductShippingCosts );
            _productPriceAccess.SaveAll( product, product.ProductPrices );

            _productSpecificationAccess.SaveAll( product.ProductID, product.ProductSpecifications );

            if (product.IsProductKit)
                _productKitAccess.SaveAll( product.ProductID, product.ProductKits );

            RefreshIsParentVisible( product.ProductID );


            return product;
        }

        private Product CreateLargeProductBaseData(
            Product product, ProductLocale locale, ArrayList categoryVisibleIDList, bool processImage, bool isUseOption )
        {
            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO Product (Sku, " +
                "    Model, ImageSecondary, Weight, FixedShippingCost, FreeShippingCost, " +
                "    ShippingCost, Manufacturer, " +
                "    Keywords, IsTodaySpecial, MinQuantity, MaxQuantity, " +
                "    Other1, Other2, Other3, Other4, Other5, IsDownloadable, DownloadPath, " +
                "    IsEnabled, RelatedProducts, MerchantRating, DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, IsParentVisible, " +
                "    TaxClassID, UseInventory,IsAffiliate, IsRecurring, ProductDetailsLayoutPath, IsQBExported, IsCustomPrice, " +
                "    IsCallForPrice, Brand, ReturnTime, IsProductKit, IsDynamicProductKitPrice, IsDynamicProductKitWeight, CreateDate,ViewCount, " +
                "    Length, Width, Height)  " +
                "VALUES (@Sku, " +
                "    @Model, @ImageSecondary, @Weight, @FixedShippingCost, @FreeShippingCost, " +
                "    @ShippingCost, @Manufacturer, " +
                "    @Keywords, @IsTodaySpecial, @MinQuantity, @MaxQuantity, " +
                "    @Other1, @Other2, @Other3, @Other4, @Other5, @IsDownloadable, @DownloadPath, " +
                "    @IsEnabled, @RelatedProducts, @MerchantRating, @DiscountGroupID, " +
                "    @IsGiftcertificate, @IsFixedPrice, @ManufacturerPartNumber, @Upc, @IsParentVisible, " +
                "    @TaxClassID, @UseInventory, @IsAffiliate, @IsRecurring, @ProductDetailsLayoutPath, @IsQBExported, @IsCustomPrice, " +
                "    @IsCallForPrice, @Brand, @ReturnTime, @IsProductKit, @IsDynamicProductKitPrice, @IsDynamicProductKitWeight,@CreateDate,@ViewCount, " +
                "    @Length, @Width, @Height);" +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( product.Sku ),
                DataAccess.CreateParameterString( product.Model ),
                DataAccess.CreateParameterString( product.ImageSecondary ),
                DataAccess.CreateParameterDouble( product.Weight ),
                DataAccess.CreateParameterBool( product.FixedShippingCost ),
                DataAccess.CreateParameterBool( product.FreeShippingCost ),
                DataAccess.CreateParameterDecimalMoney( product.ShippingCost ),
                DataAccess.CreateParameterString( product.Manufacturer ),
                DataAccess.CreateParameterString( product.Keywords ),
                DataAccess.CreateParameterBool( product.IsTodaySpecial ),
                DataAccess.CreateParameterInt32( product.MinQuantity ),
                DataAccess.CreateParameterInt32( product.MaxQuantity ),
                DataAccess.CreateParameterString( product.Other1 ),
                DataAccess.CreateParameterString( product.Other2 ),
                DataAccess.CreateParameterString( product.Other3 ),
                DataAccess.CreateParameterString( product.Other4 ),
                DataAccess.CreateParameterString( product.Other5 ),
                DataAccess.CreateParameterBool( product.IsDownloadable ),
                DataAccess.CreateParameterString( product.DownloadPath ),
                DataAccess.CreateParameterBool( product.IsEnabled ),
                DataAccess.CreateParameterString( product.RelatedProducts ),
                DataAccess.CreateParameterDouble( product.MerchantRating ),
                DataAccess.CreateParameterString( product.DiscountGroupID ),
                DataAccess.CreateParameterBool( product.IsGiftCertificate ),
                DataAccess.CreateParameterBool( product.IsFixedPrice ),
                DataAccess.CreateParameterString( product.ManufacturerPartNumber ),
                DataAccess.CreateParameterString( product.Upc ),
                DataAccess.CreateParameterBool( true ),
                DataAccess.CreateParameterString( product.TaxClassID ),
                DataAccess.CreateParameterBool( product.UseInventory ),
                DataAccess.CreateParameterBool( product.IsAffiliate ),
                DataAccess.CreateParameterBool( product.IsRecurring ),
                DataAccess.CreateParameterString( product.ProductDetailsLayoutPath ),
                DataAccess.CreateParameterBool( false ),
                DataAccess.CreateParameterBool( product.IsCustomPrice ),
                DataAccess.CreateParameterBool( product.IsCallForPrice ),
                DataAccess.CreateParameterString( product.Brand ),
                DataAccess.CreateParameterInt32( product.ReturnTime ),
                DataAccess.CreateParameterBool( product.IsProductKit ),
                DataAccess.CreateParameterBool( product.IsDynamicProductKitPrice ),
                DataAccess.CreateParameterBool( product.IsDynamicProductKitWeight ),
                DataAccess.CreateParameterDateTime( product.CreateDate ),
                DataAccess.CreateParameterInt32( product.ViewCount ),
                DataAccess.CreateParameterDouble( product.Length ),
                DataAccess.CreateParameterDouble( product.Width ),
                DataAccess.CreateParameterDouble( product.Height )
                );


            product.ProductID = newID;

            InsertLocale( product.ProductID, locale );

            InsertMetaInformations( product.ProductID, product.MetaInformations );

            if (isUseOption)
                _productOptionGroupAccess.SaveAll( product.ProductID, product.ProductOptionGroups );

            bool isParentCategoryVisible;
            _productCategoryAccess.SaveAll( product.ProductID, product.CategoryIDs, categoryVisibleIDList, out isParentCategoryVisible );

            if (!isParentCategoryVisible)
                RefreshIsParentVisible( product.ProductID, isParentCategoryVisible );

            _productDepartmentAccess.Create( product.ProductID, product.DepartmentIDs );

            _productStockAccess.InsertProductStock( product.ProductID, product.ProductStocks );

            if (processImage)
                _productImageAccess.SaveAll( product.ProductID, product.ProductImages );

            if (product.IsProductKit)
                _productKitAccess.SaveAll( product.ProductID, product.ProductKits );

            _productPriceAccess.Create( product, product.ProductPrices );

            return product;
        }

        private Product Create( Product product )
        {
            Product result;

            result = CreateBaseData( product );

            if (product.IsGiftCertificate)
                result = _productGiftCertificateAccess.Create( (GiftCertificateProduct) result );

            if (product.IsRecurring)
                result = _productRecurringAccess.Create( result );

            if (product.IsCustomPrice)
                result = _productCustomPriceAccess.Create( result );

            return result;
        }

        private Product CreateLargeProduct( Product product, ProductLocale locale, ArrayList categoryVisibleIDList, bool processImage, bool isUseOption )
        {
            Product result;

            result = CreateLargeProductBaseData( product, locale, categoryVisibleIDList, processImage, isUseOption );

            if (product.IsGiftCertificate)
                result = _productGiftCertificateAccess.Create( (GiftCertificateProduct) result );

            if (product.IsRecurring)
                result = _productRecurringAccess.Create( result );

            if (product.IsCustomPrice)
                result = _productCustomPriceAccess.Create( result );

            return result;
        }

        private Product Update( Product product )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Product " +
                "SET sku = @sku, " +
                "    model = @model, " +
                "    imageSecondary = @ImageFile, " +
                "    weight = @weight, " +
                "    fixedShippingCost = @fixedShippingCost, " +
                "    freeShippingCost = @FreeShippingCost, " +
                "    shippingCost = @shippingCost, " +
                "    manufacturer = @manufacturer, " +
                "    keywords = @keywords, " +
                "    isTodaySpecial = @isTodaySpecial, " +
                "    MinQuantity = @MinQuantity, " +
                "    MaxQuantity = @MaxQuantity, " +
                "    IsDownloadable = @IsDownloadable, " +
                "    DownloadPath = @DownloadPath, " +
                "    IsEnabled = @isEnable, " +
                "    RelatedProducts = @relatedProducts, " +
                "    MerchantRating = @MerchantRating, " +
                "    DiscountGroupID = @discountGroupID, " +
                "    Other1 = @Other1, " +
                "    Other2 = @Other2, " +
                "    Other3 = @Other3, " +
                "    Other4 = @Other4, " +
                "    Other5 = @Other5, " +
                "    IsFixedPrice = @isFixedPrice, " +
                "    ManufacturerPartNumber = @manufacturerPartNumber, " +
                "    Upc = @upc, " +
                "    TaxClassID = @taxClassID, " +
                "    UseInventory = @useInventory, " +
                "    IsAffiliate = @IsAffiliate, " +
                "    IsRecurring = @IsRecurring, " +
                "    ProductDetailsLayoutPath = @ProductDetailsLayoutPath, " +
                "    QBListID = @QBListID, " +
                "    IsQBExported = @IsQBExported, " +
                "    IsCustomPrice = @IsCustomPrice, " +
                "    IsCallForPrice = @IsCallForPrice, " +
                "    Brand = @Brand, " +
                "    ReturnTime = @ReturnTime, " +
                "    IsProductKit = @IsProductKit, " +
                "    IsDynamicProductKitPrice = @IsDynamicProductKitPrice, " +
                "    IsDynamicProductKitWeight = @IsDynamicProductKitWeight, " +
                "    CreateDate = @CreateDate, " +
                "    ViewCount = @ViewCount, " +
                "    Length = @Length, " +
                "    Width = @Width, " +
                "    Height = @Height " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( product.Sku ),
                DataAccess.CreateParameterString( product.Model ),
                DataAccess.CreateParameterString( product.ImageSecondary ),
                DataAccess.CreateParameterDouble( product.Weight ),
                DataAccess.CreateParameterBool( product.FixedShippingCost ),
                DataAccess.CreateParameterBool( product.FreeShippingCost ),
                DataAccess.CreateParameterDecimalMoney( product.ShippingCost ),
                DataAccess.CreateParameterString( product.Manufacturer ),
                DataAccess.CreateParameterString( product.Keywords ),
                DataAccess.CreateParameterBool( product.IsTodaySpecial ),
                DataAccess.CreateParameterInt32( product.MinQuantity ),
                DataAccess.CreateParameterInt32( product.MaxQuantity ),
                DataAccess.CreateParameterBool( product.IsDownloadable ),
                DataAccess.CreateParameterString( product.DownloadPath ),
                DataAccess.CreateParameterBool( product.IsEnabled ),
                DataAccess.CreateParameterString( product.RelatedProducts ),
                DataAccess.CreateParameterDouble( product.MerchantRating ),
                DataAccess.CreateParameterString( product.DiscountGroupID ),
                DataAccess.CreateParameterString( product.Other1 ),
                DataAccess.CreateParameterString( product.Other2 ),
                DataAccess.CreateParameterString( product.Other3 ),
                DataAccess.CreateParameterString( product.Other4 ),
                DataAccess.CreateParameterString( product.Other5 ),
                DataAccess.CreateParameterBool( product.IsFixedPrice ),
                DataAccess.CreateParameterString( product.ManufacturerPartNumber ),
                DataAccess.CreateParameterString( product.Upc ),
                DataAccess.CreateParameterString( product.TaxClassID ),
                DataAccess.CreateParameterBool( product.UseInventory ),
                DataAccess.CreateParameterBool( product.IsAffiliate ),
                DataAccess.CreateParameterBool( product.IsRecurring ),
                DataAccess.CreateParameterString( product.ProductDetailsLayoutPath ),
                DataAccess.CreateParameterString( product.QBListID ),
                DataAccess.CreateParameterBool( product.IsQBExported ),
                DataAccess.CreateParameterBool( product.IsCustomPrice ),
                DataAccess.CreateParameterBool( product.IsCallForPrice ),
                DataAccess.CreateParameterString( product.Brand ),
                DataAccess.CreateParameterInt32( product.ReturnTime ),
                DataAccess.CreateParameterBool( product.IsProductKit ),
                DataAccess.CreateParameterBool( product.IsDynamicProductKitPrice ),
                DataAccess.CreateParameterBool( product.IsDynamicProductKitWeight ),
                DataAccess.CreateParameterDateTime( product.CreateDate ),
                DataAccess.CreateParameterInt32( product.ViewCount ),
                DataAccess.CreateParameterDouble( product.Length ),
                DataAccess.CreateParameterDouble( product.Width ),
                DataAccess.CreateParameterDouble( product.Height ),
                DataAccess.CreateParameterString( product.ProductID ) );

            _productCategoryAccess.SaveAll( product.ProductID, product.CategoryIDs );
            _productDepartmentAccess.SaveAll( product.ProductID, product.DepartmentIDs );

            LocalesSaveAll( product.ProductID, product.Locales );
            MetaInformationsSaveAll( product.ProductID, product.MetaInformations );

            _productOptionGroupAccess.SaveAll( product.ProductID, product.ProductOptionGroups );

            _productStockAccess.SaveAll( product.ProductID, product.ProductStocks );

            _productImageAccess.SaveAll( product.ProductID, product.ProductImages );
            _productShippingCostAccess.SaveAll( product.ProductID, product.ProductShippingCosts );
            _productPriceAccess.SaveAll( product, product.ProductPrices );

            _productSpecificationAccess.SaveAll( product.ProductID, product.ProductSpecifications );

            _productKitAccess.SaveAll( product.ProductID, product.ProductKits );

            if (product.IsGiftCertificate)
                product = _productGiftCertificateAccess.Update( (GiftCertificateProduct) product );

            if (product.IsRecurring)
                product = _productRecurringAccess.Update( product );

            if (product.IsCustomPrice)
            {
                if (_productCustomPriceAccess.IsExistItem( product.ProductID ))

                    product = _productCustomPriceAccess.Update( product );
                else
                    product = _productCustomPriceAccess.Create( product );
            }

            RefreshIsParentVisible( product.ProductID );



            return product;
        }

        private Product UpdateLargeProduct( Product product, ArrayList categoryVisibleIDList, bool processImage, bool isUseOption )
        {
            DataAccess.ExecuteNonQuery(
              "UPDATE Product " +
              "SET sku = @sku, " +
              "    model = @model, " +
              "    imageSecondary = @ImageFile, " +
              "    weight = @weight, " +
                //      "    fixedShippingCost = @fixedShippingCost, " +
                //    "    shippingCost = @shippingCost, " +
              "    freeShippingCost = @FreeShippingCost, " +
              "    manufacturer = @manufacturer, " +
              "    keywords = @keywords, " +
              "    isTodaySpecial = @isTodaySpecial, " +
              "    MinQuantity = @MinQuantity, " +
              "    MaxQuantity = @MaxQuantity, " +
              "    IsDownloadable = @IsDownloadable, " +
              "    DownloadPath = @DownloadPath, " +
              "    IsEnabled = @isEnable, " +
              "    RelatedProducts = @relatedProducts, " +
              "    MerchantRating = @MerchantRating, " +
              "    DiscountGroupID = @discountGroupID, " +
              "    Other1 = @Other1, " +
              "    Other2 = @Other2, " +
              "    Other3 = @Other3, " +
              "    Other4 = @Other4, " +
              "    Other5 = @Other5, " +
              "    IsFixedPrice = @isFixedPrice, " +
              "    ManufacturerPartNumber = @manufacturerPartNumber, " +
              "    Upc = @upc, " +
              "    TaxClassID = @taxClassID, " +
              "    UseInventory = @useInventory, " +
              "    IsAffiliate = @IsAffiliate, " +
              "    IsRecurring = @IsRecurring, " +
              "    ProductDetailsLayoutPath = @ProductDetailsLayoutPath, " +
              "    QBListID = @QBListID, " +
              "    IsQBExported = @IsQBExported, " +
              "    IsCustomPrice = @IsCustomPrice, " +
              "    IsCallForPrice = @IsCallForPrice, " +
              "    Brand = @Brand, " +
              "    ReturnTime = @ReturnTime, " +
              "    IsProductKit = @IsProductKit, " +
              "    IsDynamicProductKitPrice = @IsDynamicProductKitPrice, " +
              "    IsDynamicProductKitWeight = @IsDynamicProductKitWeight, " +
              "    CreateDate = @CreateDate, " +
              "    ViewCount = @ViewCount, " +
              "    Length = @Length, " +
              "    Width = @Width, " +
              "    Height = @Height " +
              "WHERE ProductID = @ProductID; ",
              DataAccess.CreateParameterString( product.Sku ),
              DataAccess.CreateParameterString( product.Model ),
              DataAccess.CreateParameterString( product.ImageSecondary ),
              DataAccess.CreateParameterDouble( product.Weight ),
                //    DataAccess.CreateParameterBool(product.FixedShippingCost),
                //   DataAccess.CreateParameterDecimalMoney(product.ShippingCost),
              DataAccess.CreateParameterBool( product.FreeShippingCost ),
              DataAccess.CreateParameterString( product.Manufacturer ),
              DataAccess.CreateParameterString( product.Keywords ),
              DataAccess.CreateParameterBool( product.IsTodaySpecial ),
              DataAccess.CreateParameterInt32( product.MinQuantity ),
              DataAccess.CreateParameterInt32( product.MaxQuantity ),
              DataAccess.CreateParameterBool( product.IsDownloadable ),
              DataAccess.CreateParameterString( product.DownloadPath ),
              DataAccess.CreateParameterBool( product.IsEnabled ),
              DataAccess.CreateParameterString( product.RelatedProducts ),
              DataAccess.CreateParameterDouble( product.MerchantRating ),
              DataAccess.CreateParameterString( product.DiscountGroupID ),
              DataAccess.CreateParameterString( product.Other1 ),
              DataAccess.CreateParameterString( product.Other2 ),
              DataAccess.CreateParameterString( product.Other3 ),
              DataAccess.CreateParameterString( product.Other4 ),
              DataAccess.CreateParameterString( product.Other5 ),
              DataAccess.CreateParameterBool( product.IsFixedPrice ),
              DataAccess.CreateParameterString( product.ManufacturerPartNumber ),
              DataAccess.CreateParameterString( product.Upc ),
              DataAccess.CreateParameterString( product.TaxClassID ),
              DataAccess.CreateParameterBool( product.UseInventory ),
              DataAccess.CreateParameterBool( product.IsAffiliate ),
              DataAccess.CreateParameterBool( product.IsRecurring ),
              DataAccess.CreateParameterString( product.ProductDetailsLayoutPath ),
              DataAccess.CreateParameterString( product.QBListID ),
              DataAccess.CreateParameterBool( product.IsQBExported ),
              DataAccess.CreateParameterBool( product.IsCustomPrice ),
              DataAccess.CreateParameterBool( product.IsCallForPrice ),
              DataAccess.CreateParameterString( product.Brand ),
              DataAccess.CreateParameterInt32( product.ReturnTime ),
              DataAccess.CreateParameterBool( product.IsProductKit ),
              DataAccess.CreateParameterBool( product.IsDynamicProductKitPrice ),
              DataAccess.CreateParameterBool( product.IsDynamicProductKitWeight ),
              DataAccess.CreateParameterDateTime( product.CreateDate ),
              DataAccess.CreateParameterInt32( product.ViewCount ),
              DataAccess.CreateParameterDouble( product.Length ),
              DataAccess.CreateParameterDouble( product.Width ),
              DataAccess.CreateParameterDouble( product.Height ),
              DataAccess.CreateParameterString( product.ProductID ) );

            bool isParentCategoryVisible;
            _productCategoryAccess.SaveAll( product.ProductID, product.CategoryIDs, categoryVisibleIDList, out isParentCategoryVisible );
            _productDepartmentAccess.SaveAll( product.ProductID, product.DepartmentIDs );

            LocalesSaveAll( product.ProductID, product.Locales );
            MetaInformationsSaveAll( product.ProductID, product.MetaInformations );

            if (isUseOption)
                _productOptionGroupAccess.SaveAll( product.ProductID, product.ProductOptionGroups );

            _productStockAccess.SaveAll( product.ProductID, product.ProductStocks );

            if (processImage)
                _productImageAccess.SaveAll( product.ProductID, product.ProductImages );

            //      _productShippingCostAccess.SaveAll(product.ProductID, product.ProductShippingCosts);
            _productPriceAccess.SaveAll( product, product.ProductPrices );

            if (product.IsProductKit)
                _productKitAccess.SaveAll( product.ProductID, product.ProductKits );

            if (product.IsGiftCertificate)
                product = _productGiftCertificateAccess.Update( (GiftCertificateProduct) product );

            if (product.IsRecurring)
                product = _productRecurringAccess.Update( product );

            if (product.IsCustomPrice)
            {
                if (_productCustomPriceAccess.IsExistItem( product.ProductID ))

                    product = _productCustomPriceAccess.Update( product );
                else
                    product = _productCustomPriceAccess.Create( product );
            }

            if (product.IsParentVisible != isParentCategoryVisible)
                RefreshIsParentVisible( product.ProductID, isParentCategoryVisible );

            return product;
        }


        private Product GetOneFromDatabase( Culture culture, string productID, string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                SQLSelectProductString +
                "WHERE Product.ProductID = @productID; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( productID )
                );

            if (table.Rows.Count > 0)
                return CreateOneProductFromRow( culture, table.Rows[0], storeID );
            else
                return Product.Null;
        }

        private ProductLocale CreateOneLocaleFromRow( DataRow row )
        {
            ProductLocale locale = new ProductLocale();
            locale.CultureID = ConvertUtilities.ToString( row["CultureID"] );
            locale.Name = row["Name"].ToString();
            locale.ShortDescription = row["ShortDescription"].ToString();
            locale.LongDescription = row["LongDescription"].ToString();
            locale.UrlName = row["UrlName"].ToString();
            return locale;
        }
        private ProductImageLocale CreateOneImageLocaleFromRow( DataRow row )
        {
            ProductImageLocale locale = new ProductImageLocale();
            locale.CultureID = ConvertUtilities.ToString( row["CultureID"] );
            locale.AltTag = row["AltTag"].ToString();
            locale.TitleTag = row["TitleTag"].ToString();

            return locale;
        }

        private ProductMetaInformation CreateOneMetaInformationFromRow( DataRow row )
        {
            ProductMetaInformation meta = new ProductMetaInformation();
            meta.ProductID = ConvertUtilities.ToString( row["ProductID"] );
            meta.CultureID = ConvertUtilities.ToString( row["CultureID"] );
            meta.StoreID = ConvertUtilities.ToString( row["StoreID"] );
            meta.MetaKeyword = row["MetaKeyword"].ToString();
            meta.MetaDescription = row["MetaDescription"].ToString();
            meta.UseDefaultMetaKeyword = ConvertUtilities.ToBoolean( row["UseDefaultMetaKeyword"] );
            meta.UseDefaultMetaDescription = ConvertUtilities.ToBoolean( row["UseDefaultMetaDescription"] );
            return meta;
        }

        private IList<Product> ConvertProductTableToList( Culture culture, DataTable table, string storeID )
        {
            IList<Product> list = new List<Product>();

            foreach (DataRow row in table.Rows)
            {
                list.Add( CreateOneProductFromRow( culture, row, storeID ) );
            }

            return list;
        }

        private IList<Product> GetOnlyIndex( IList<Product> productList, int startIndex, int endIndex )
        {
            IList<Product> list = new List<Product>();
            for (int i = startIndex; i <= endIndex && i < productList.Count; i++)
            {
                list.Add( productList[i] );
            }
            return list;
        }

        private IList<Product> ConvertProductTableToList( Culture culture, DataTable table, int startIndex, int endIndex, string storeID )
        {
            IList<Product> list = new List<Product>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
            {
                list.Add( CreateOneProductFromRow( culture, table.Rows[i], storeID ) );
            }

            return list;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );
            return list;
        }
        private List<ILocale> ConvertImageLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();
            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneImageLocaleFromRow( row ) );

            return list;
        }

        private List<ILocale> ConvertMetaInformationTableToList( DataTable metaInformationTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in metaInformationTable.Rows)
                list.Add( CreateOneMetaInformationFromRow( row ) );

            return list;
        }

        private void DeleteImageAll( string productID )
        {
            Product product = GetOne( Culture.Null, productID, Store.Null.StoreID );
            foreach (ProductImage productImage in product.ProductImages)
            {
                DeleteImageFiles( productImage );
            }

            if (!IsImageSecondaryExisting( product.ImageSecondary ) && !String.IsNullOrEmpty( product.ImageSecondary ))
                FileUtilities.Delete( GetLocalPath( product.ImageSecondary ) );
        }

        private void DeleteImageFiles( ProductImage productImage )
        {
            if (!DataAccessContext.ProductRepository.IsRegularImageExisting( productImage.RegularImage ) && !String.IsNullOrEmpty( productImage.RegularImage ))
                FileUtilities.Delete( GetLocalPath( productImage.RegularImage ) );

            if (!DataAccessContext.ProductRepository.IsLargeImageExisting( productImage.LargeImage ) && !String.IsNullOrEmpty( productImage.LargeImage ))
                FileUtilities.Delete( GetLocalPath( productImage.LargeImage ) );

            if (!DataAccessContext.ProductRepository.IsThumbnailImageExisting( productImage.ThumbnailImage ) && !String.IsNullOrEmpty( productImage.ThumbnailImage ))
                FileUtilities.Delete( GetLocalPath( productImage.ThumbnailImage ) );
        }

        private string GetLocalPath( string relativePath )
        {
            if (relativePath.Trim().StartsWith( "~/" ))
                return HttpContext.Current.Server.MapPath( relativePath.Trim() );
            else
                return HttpContext.Current.Server.MapPath( "~/" + relativePath.Trim() );
        }

        private void DeleteProduct( string productID )
        {
            DeleteImageAll( productID );

            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM Product " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );

            DeleteLocales( productID );
            DeleteMetaInformations( productID );
            _productCategoryAccess.DeleteByProductID( productID );
            _productDepartmentAccess.DeleteByProductID( productID );
            _productStockAccess.DeleteByProductID( productID );
            _productShippingCostAccess.DeleteByProductID( productID );
            _productGiftCertificateAccess.DeleteByProductID( productID );
            _productImageAccess.DeleteByProductID( productID );
            _customerReviewAccess.DeleteByProductID( productID );
            _productPriceAccess.DeleteByProductID( productID );
            _productCustomPriceAccess.DeleteByProductID( productID );
            _productSpecificationAccess.DeleteByProductID( productID );
            _productKitAccess.DeleteByProductID( productID );
            _productKitItemAccess.DeleteByProductID( productID );
        }

        private static int ProductIDMax( int outOfStockValue, bool useStockInventoryControl )
        {
            ArrayList parameterList = new ArrayList();
            string strSQL = "SELECT MAX(Product.ProductID) FROM (Product INNER JOIN " +
                "(SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock GROUP BY ProductID) " +
                "AS S ON Product.ProductID = S.ProductID ) WHERE ";
            if (useStockInventoryControl)
            {
                strSQL += "((SumStock > @outOfStockValue AND UseInventory = @trueCondition) OR UseInventory = @falseCondotion) AND ";
                parameterList.Add( DataAccess.CreateParameterInt32( outOfStockValue ) );
                parameterList.Add( DataAccess.CreateParameterBool( true ) );
                parameterList.Add( DataAccess.CreateParameterBool( false ) );
            }
            strSQL += "IsEnabled = @isEnable AND IsParentVisible = @isParentVisible ";
            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            string maxID = DataAccess.ExecuteScalar( strSQL, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            int result = 0;
            if (int.TryParse( maxID, out result ))
                return result;
            else
                return 0;
        }

        private static int ProductIDMin( int outOfStockValue, bool useStockInventoryControl )
        {
            ArrayList parameterList = new ArrayList();
            string strSQL = "SELECT MIN(Product.ProductID) FROM (Product INNER JOIN " +
                "(SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock GROUP BY ProductID) " +
                "AS S ON Product.ProductID = S.ProductID ) WHERE ";
            if (useStockInventoryControl)
            {
                strSQL += "((SumStock > @outOfStockValue AND UseInventory = @trueCondition) OR UseInventory = @falseCondotion) AND ";
                parameterList.Add( DataAccess.CreateParameterInt32( outOfStockValue ) );
                parameterList.Add( DataAccess.CreateParameterBool( true ) );
                parameterList.Add( DataAccess.CreateParameterBool( false ) );
            }
            strSQL += " IsEnabled = @isEnable AND IsParentVisible = @isParentVisible ";
            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            string minID = DataAccess.ExecuteScalar( strSQL, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            int result = 0;
            if (int.TryParse( minID, out result ))
                return result;
            else
                return 0;
        }

        private static int GetProductTotalAmount( int outOfStockValue, bool useStockInventoryControl )
        {
            ArrayList parameterList = new ArrayList();

            string strSQL = "SELECT COUNT(Product.ProductID) FROM Product INNER JOIN " +
                "(SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock GROUP BY ProductID ) AS S ON " +
                "Product.ProductID = S.ProductID " +
                "WHERE ";
            if (useStockInventoryControl)
            {
                strSQL += "((SumStock > @outOfStockValue AND UseInventory = @trueCondition) OR UseInventory = @falseCondotion) AND ";
                parameterList.Add( DataAccess.CreateParameterInt32( outOfStockValue ) );
                parameterList.Add( DataAccess.CreateParameterBool( true ) );
                parameterList.Add( DataAccess.CreateParameterBool( false ) );
            }
            strSQL += " IsEnabled = @isEnable AND IsParentVisible = @IsParentVisible";
            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            string amount = DataAccess.ExecuteScalar( strSQL, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            int result;
            if (int.TryParse( amount, out result ))
                return result;
            else
                return 0;
        }

        private void DeleteAllFiles( string folderName )
        {
            DirectoryInfo dir = new DirectoryInfo( folderName );
            foreach (FileInfo fi in dir.GetFiles())
            {
                fi.Delete();
            }
            foreach (DirectoryInfo di in dir.GetDirectories())
            {
                DeleteAllFiles( di.FullName );
            }
        }

        private int CountTopProduct( Culture culture, string storeID, int limitResult, string sortBy, BoolFilter isEnabled, string rootCategoryID )
        {
            string query = "SELECT TOP( " + limitResult.ToString() + " ) ROW_NUMBER() over ( ORDER BY " + sortBy + " DESC ) as rownum, " +
                           "* FROM (( " + SQLSelectProductString +
                           FlagFilterHelper.CreateFlagFilterWithWhere( "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                           ") AS Product JOIN " +
                           "(SELECT productCategory.ProductID AS ID from category,productCategory " +
                            "WHERE category.rootID = @rootID " +
                            "AND category.CategoryID = productCategory.CategoryID) " +
                            "AS ProductByRoot ON  ProductByRoot.ID = Product.ProductID )  ";

            DataTable table = DataAccess.ExecuteSelect(
                "with SearchResult as (" + query + ") select * from SearchResult;",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( rootCategoryID)
            );

            return table.Rows.Count;
        }

        public int GetProductCountByCategoryID( string categoryID, BoolFilter isEnabled )
        {
            string productCount = DataAccess.ExecuteScalar(
                "SELECT COUNT(distinct P.ProductID) FROM Product P, ProductCategory PC, Category C WHERE PC.CategoryID=" + categoryID +
                " AND P.ProductID = PC.ProductID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "C.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                FlagFilterHelper.CreateFlagFilterWithAnd( "P.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) );

            return Int32.Parse( productCount );
        }

        public int GetProductCountByCategoryID( string[] categoryIDs )
        {
            string query = String.Empty;
            ArrayList parameterList = new ArrayList();

            for (int i = 0; i <= categoryIDs.Length - 1; i++)
            {
                query += "C.CategoryID = @ID" + i.ToString();
                parameterList.Add( DataAccess.CreateParameterString( categoryIDs[i].ToString() ) );
                if (i < categoryIDs.Length - 1)
                {
                    query += " or ";
                }
            }
            if (!String.IsNullOrEmpty( query ))
            {
                query = " WHERE " + " ( " + query + " ) and P.ProductID = PC.ProductID and P.IsEnabled = 1" +
                    " AND PC.CategoryID = C.CategoryID and C.IsEnabled = 1";
            }


            string productCount = DataAccess.ExecuteScalar(
                "SELECT COUNT(distinct P.ProductID) FROM Product P, ProductCategory PC, Category C " +
                query, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return Int32.Parse( productCount );
        }

        public int GetProductCountByDepartmentID( string departmentID, BoolFilter isEnabled )
        {
            string productCount = DataAccess.ExecuteScalar(
                "SELECT COUNT(distinct P.ProductID) FROM Product P, ProductDepartment PD, Department D WHERE PD.DepartmentID=" + departmentID +
                " AND P.ProductID = PD.ProductID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "D.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                FlagFilterHelper.CreateFlagFilterWithAnd( "P.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) );
            return Int32.Parse( productCount );
        }

        public int GetProductCountByManufacturerID( string manufacturerID, BoolFilter isEnabled )
        {
            string productCount = DataAccess.ExecuteScalar(
                "SELECT COUNT(distinct P.ProductID) FROM Product P WHERE Manufacturer =" + manufacturerID +
                FlagFilterHelper.CreateFlagFilterWithAnd( "P.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) );
            return Int32.Parse( productCount );
        }

        public int GetProductCountByDepartmentID( string[] departmentIDs )
        {
            string query = String.Empty;
            ArrayList parameterList = new ArrayList();

            for (int i = 0; i <= departmentIDs.Length - 1; i++)
            {
                query += "D.DepartmentID = @ID" + i.ToString();
                parameterList.Add( DataAccess.CreateParameterString( departmentIDs[i].ToString() ) );
                if (i < departmentIDs.Length - 1)
                {
                    query += " or ";
                }
            }
            if (!String.IsNullOrEmpty( query ))
            {
                query = " WHERE " + " ( " + query + " ) and P.ProductID = PD.ProductID and P.IsEnabled = 1 " +
                    " AND PD.DepartmentID = D.DepartmentID and D.IsEnabled = 1 ";
            }


            string productCount = DataAccess.ExecuteScalar(
                "SELECT COUNT(distinct P.ProductID) FROM Product P, ProductDepartment PD, Department D " +
                query, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return Int32.Parse( productCount );
        }

        public Dictionary<string, int> GetProductCountGroupByCategoryID()
        {
            DataTable table = DataAccess.ExecuteSelect( "SELECT CategoryID, COUNT(ProductID) AS ProductCount FROM " +
                "(SELECT Product.ProductID,CategoryID,IsEnabled from Product,ProductCategory " +
                "WHERE Product.IsEnabled = 1 AND Product.ProductID = ProductCategory.ProductID) as ProductCategoryEnabledOnly" +
                " GROUP BY CategoryID" );
            Dictionary<string, int> productCountByCategoryID = new Dictionary<string, int>();

            foreach (DataRow row in table.Rows)
            {
                productCountByCategoryID.Add( row["CategoryID"].ToString(), Int32.Parse( row["ProductCount"].ToString() ) );
            }
            return productCountByCategoryID;
        }


        public Dictionary<string, int> GetProductCountGroupByDepartmentID()
        {
            DataTable table = DataAccess.ExecuteSelect( "SELECT DepartmentID, COUNT(ProductID) AS ProductCount FROM " +
                "(SELECT Product.ProductID,DepartmentID,IsEnabled from Product,ProductDepartment " +
                "WHERE Product.IsEnabled = 1 AND Product.ProductID = ProductDepartment.ProductID) as ProductDepartmentEnabledOnly" +
              " GROUP BY DepartmentID" );
            Dictionary<string, int> productCountByDepartmentID = new Dictionary<string, int>();

            foreach (DataRow row in table.Rows)
            {
                productCountByDepartmentID.Add( row["DepartmentID"].ToString(), Int32.Parse( row["ProductCount"].ToString() ) );
            }
            return productCountByDepartmentID;
        }

        private int GetProductCount()
        {
            string productCount = DataAccess.ExecuteScalar( "SELECT COUNT(ProductID) FROM Product" );
            return Int32.Parse( productCount );
        }

        private int GetSearchProductCount(
            Culture culture,
            string categoryID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            string storeID,
            string rootCategoryID
            )
        {
            string productFieldName = ConvertFieldName( searchFilter.FieldName );
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, productFieldName, parameterList );

            string query = "SELECT COUNT (DISTINCT  Product.ProductID) " +
                "FROM " +
                "    ((( ( Product INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock " +
                "    GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "    (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "    ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "    WHERE CultureID = @CultureID2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID) " +

                CreateCategoryInnerJoinSql( categoryID, rootCategoryID ) +
                commandSearch;

            string searchProductCount = DataAccess.ExecuteScalar(
                query, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return Convert.ToInt32( searchProductCount );
        }

        private IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            BoolFilter isEnabled,
            bool isIncludeProductKit,
            string storeID,
            int startIndex,
            int endIndex
     )
        {
            DataTable table = _categoryHelper.GetByCategoryDepartmentID( culture, categoryID, sortBy, isEnabled, isIncludeProductKit, storeID, startIndex, endIndex );
            return ConvertProductTableToList( culture, table, storeID );
        }

        private IList<Product> GetByCategoryID(
           Culture culture,
           string categoryID,
           string sortBy,
           BoolFilter isEnabled,
           string storeID,
           int startIndex,
           int endIndex
        )
        {
            DataTable table = _categoryHelper.GetByCategoryDepartmentID( culture, categoryID, sortBy, isEnabled, storeID, startIndex, endIndex );
            return ConvertProductTableToList( culture, table, storeID );
        }

        private IList<Product> GetFacetResultByCategoryID(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            string minPrice,
            string maxPrice,
            IList<SpecificationItemValue> specValueList,
            string sortBy,
            BoolFilter isEnabled,
            string storeID,
            int startIndex,
            int endIndex
            )
        {
            DataTable table = _categoryHelper.GetFacetResultByCategoryID( culture, categoryIDs, departmentIDs, manufacturerID, minPrice, maxPrice, specValueList, sortBy, isEnabled, storeID, startIndex, endIndex );
            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetNewArrivalByCategoryDepartmentID(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            int limitCount,
            string storeID )
        {
            DataTable table = _categoryHelper.GetNewArrivalByCategoryDepartmentID( culture, categoryIDs, departmentIDs, limitCount, storeID );
            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetNewArrivalByManufacturerID(
            Culture culture,
            string manufacturerID,
            int limitCount,
            string storeID )
        {
            DataTable table;
            string sql = "SELECT DISTINCT TOP " + limitCount.ToString() + " RandID = NEWID()," +
                AllProductFields +
                "FROM ((( ( Product INNER JOIN ( SELECT ProductID, SUM( Stock ) " +
                "AS SumStock FROM ProductStock " +
                "GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) LEFT JOIN " +
                "(SELECT * FROM ProductLocale WHERE CultureID = @CultureID1) AS L " +
                "ON Product.ProductID = L.ProductID) " +
                "INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @CultureID2 AND StoreID = @StoreID2 ) M ON Product.ProductID = M.ProductID) ";
            if (manufacturerID == "0")
            {
                table = DataAccess.ExecuteSelect( sql,
                    DataAccess.CreateParameterString( GetCultureID( culture ) ),
                    DataAccess.CreateParameterString( storeID ),
                    DataAccess.CreateParameterString( GetCultureID( culture ) ),
                    DataAccess.CreateParameterString( storeID )
                );
            }
            else
            {
                sql += "WHERE Manufacturer = @ManufacturerID ";
                sql += " ORDER BY CreateDate DESC;";
                table = DataAccess.ExecuteSelect( sql,
                    DataAccess.CreateParameterString( GetCultureID( culture ) ),
                    DataAccess.CreateParameterString( storeID ),
                    DataAccess.CreateParameterString( GetCultureID( culture ) ),
                    DataAccess.CreateParameterString( storeID ),
                    DataAccess.CreateParameterString( manufacturerID )
                );
            }


            return ConvertProductTableToList( culture, table, storeID );
        }

        private IList<Product> GetByDepartmentID(
            Culture culture,
            string departmentID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID,
            int startIndex,
            int endIndex
)
        {
            DataTable table = _departmentHelper.GetByCategoryDepartmentID( culture, departmentID, sortBy, isEnabled, true, storeID, startIndex, endIndex );
            return ConvertProductTableToList( culture, table, storeID );
        }

        private IList<Product> GetByDepartmentID(
            Culture culture,
            string[] departmentIDs,
            string sortBy,
            BoolFilter isEnabled,
            string storeID,
            int startIndex,
            int endIndex
        )
        {
            DataTable table = _departmentHelper.GetByCategoryDepartmentID( culture, departmentIDs, sortBy, isEnabled, true, storeID, startIndex, endIndex );
            return ConvertProductTableToList( culture, table, storeID );
        }

        private string CreateSortByTextGetByManufacturer( string sortBy )
        {
            string field = "ManufacturerID";
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( field ))
                return "ORDER BY " + "Manufacturer." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "name" ))
                return "ORDER BY " + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "sortorder" ))
                return "ORDER BY C." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private IList<Product> GetByManufacturerID(
            Culture culture,
            string manufacturerID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID,
            int startIndex,
            int endIndex )
        {
            string query = "SELECT DISTINCT ROW_NUMBER() over ( " + CreateSortByTextGetByManufacturer( "Product.ProductID" ) + ") as rownum," +
                "Product.ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, FreeShippingCost, " +
                "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, " +
                "    ShippingCost, Manufacturer, Keywords,PP.Price, " +
                "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
                "    RelatedProducts, MerchantRating, " +
                "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                 "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
                "    ProductDetailsLayoutPath, IsAffiliate, IsCustomPrice, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount " +
                "    FROM " +
                "    ((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock  " +
                "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
               "WHERE Manufacturer = @ID " +
                    FlagFilterHelper.CreateFlagFilterWithAnd(
                "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) )
                    + " ";

            string extQuery = "with DataByCategoryDepartmentID as (" + query + ") " +
                    " select * from DataByCategoryDepartmentID where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect(
                extQuery,
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( manufacturerID ),
                DataAccess.CreateParameterBool( true ) );

            return ConvertProductTableToList( culture, table, storeID );
        }

        #endregion


        #region Aggregate Methods

        public IEntityList<ProductImage> GetProductImages( string productID )
        {
            return _productImageAccess.GetByProductID( productID );
        }

        public IList<ProductOptionGroup> GetProductOptionGroups( Culture culture, string productID )
        {
            return _productOptionGroupAccess.GetByProductID( culture, productID );
        }

        public IList<ProductStock> GetProductStocks( string productID )
        {
            return _productStockAccess.GetByProductID( productID );
        }

        public IList<ProductShippingCost> GetProductShippingCosts( string productID )
        {
            return _productShippingCostAccess.GetByProductID( productID );
        }

        public IList<ProductPrice> GetProductPrices( string productID )
        {
            return _productPriceAccess.GetByProductID( productID );
        }

        public IList<ProductPrice> GetDefaultProductPriceList()
        {
            return _productPriceAccess.GetDefaultProductPriceList();
        }

        public ProductPrice GetDefaultProductPrice( string productID )
        {
            return _productPriceAccess.GetDefaultProductPrice( productID );
        }

        public IList<ProductSpecification> GetProductSpecifications( string productID )
        {
            return _productSpecificationAccess.GetByProductID( productID );
        }

        public IList<string> GetProductSpecificationsIDByProductIDs( IList<string> productIDs )
        {
            return _productSpecificationAccess.GetSpecificationsIDListByProductIDList( productIDs );
        }

        public IList<ProductSpecification> GetAllProductSpecifications()
        {
            return _productSpecificationAccess.GetAll();
        }

        public void DeleteProductSpecification()
        {
            _productSpecificationAccess.Delete();
        }

        public IList<ProductKit> GetProductKits( Culture culture, string productID )
        {
            return _productKitAccess.GetByProductID( culture, productID );
        }

        public void DeleteProductPricesByStoreID( string storeID )
        {
            _productPriceAccess.DeleteByStoreID( storeID );
        }

        public void CreateProductPricesFromDefaultByStoreID( string storeID ) { _productPriceAccess.CreateFromDefaultByStoreID( storeID ); }

        public IList<ILocale> GetLocales( string productID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllProductLocaleFields +
                "FROM ProductLocale " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<ILocale> GetImageLocales( string productImageID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllProductImageLocaleFields +
                "FROM ProductImageLocale " +
                "WHERE ProductImageID = @ProductImageID; ",
                DataAccess.CreateParameterString( productImageID ) );

            return ConvertImageLocaleTableToList( localeTable );
        }

        public IList<ILocale> GetMetaInformationsByStoreID( string productID, string storeID )
        {
            DataTable metaInformationTable = DataAccess.ExecuteSelect(
                "SELECT " + AllProductMetaInformationFields +
                "FROM ProductMetaInformation " +
                "WHERE ProductID = @ProductID AND StoreID = @StoreID ORDER BY ProductID, CultureID, StoreID; ",
                DataAccess.CreateParameterString( productID ), DataAccess.CreateParameterString( storeID ) );

            return ConvertMetaInformationTableToList( metaInformationTable );
        }

        public IList<ILocale> GetMetaInformations( string productID )
        {
            DataTable metaInformationTable = DataAccess.ExecuteSelect(
                "SELECT " + AllProductMetaInformationFields +
                "FROM ProductMetaInformation " +
                "WHERE ProductID = @ProductID ORDER BY ProductID, CultureID, StoreID; ",
                DataAccess.CreateParameterString( productID ) );

            return ConvertMetaInformationTableToList( metaInformationTable );
        }

        #endregion


        #region Public Methods

        public Product Save( Product product )
        {
            Product result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (product.ProductID == "0")
                    result = Create( product );
                else
                    result = Update( product );

                transaction.Complete();
            }

            return result;
        }

        public void Delete( string productID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DeleteProduct( productID );
                transaction.Complete();
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ProductLocale " +
                "WHERE CultureID = @CultureID; ",
                DataAccess.CreateParameterString( cultureID )
            );
        }

        public void DeleteAll()
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery( "DELETE FROM Product" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductLocale" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductMetaInformation" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductCategory" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductDepartment" );
                DataAccess.ExecuteNonQuery( "DELETE FROM OptionCombination" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductStock" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductShippingCost" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductGiftCertificate" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductImage" );
                DataAccess.ExecuteNonQuery( "DELETE FROM CustomerReview" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductPrice" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductCustomPrice" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductKit" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ProductKitItem" );
                transaction.Complete();
            }

        }

        public void DeleteProductMetaInformationByStoreID( string storeID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ProductMetaInformation " +
                "WHERE StoreID = @StoreID; ",
                DataAccess.CreateParameterString( storeID ) );
        }

        public void RefreshProductUrlNameAll( string productName )
        {

        }

        public void RefreshProductUrlNameAll()
        {

        }

        public string GetProductIDBySku( string sku )
        {
            string productID = DataAccess.ExecuteScalar(
                "SELECT ProductID " +
                "FROM Product " +
                "WHERE Sku = @sku; ",
                DataAccess.CreateParameterString( sku ) );

            return productID;
        }

        public Dictionary<string, string> GetProductIDAndSku()
        {
            Dictionary<string, string> skuProductID = new Dictionary<string, string>();
            DataTable table = DataAccess.ExecuteSelect( "Select ProductID,Sku from Product" );
            foreach (DataRow row in table.Rows)
            {
                skuProductID.Add( row["Sku"].ToString(), row["ProductID"].ToString() );
            }
            return skuProductID;
        }

        public Product GetOne( Culture culture, string productID, string storeID )
        {
            return GetOneFromDatabase( culture, productID, storeID );
        }

        public IList<Product> GetAllByBestSelling(
            Culture culture,
            int numberOfResult,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit
            )
        {
            string categoryListCondition = EnabledCategoryListCondition( culture, rootCategoryID );
            DataTable tableBestSelling = _categoryHelper.GetAllByBestSelling( culture, numberOfResult, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, categoryListCondition, isIncludeProductKit );

            if (tableBestSelling.Rows.Count > numberOfResult)
            {
                int numberOfRemove = tableBestSelling.Rows.Count - numberOfResult;
                for (int i = 0; i < numberOfRemove; i++)
                    tableBestSelling.Rows.RemoveAt( numberOfResult );
            }

            return ConvertProductTableToList( culture, tableBestSelling, storeID );
        }

        public IList<Product> GetAllByBestSelling(
            Culture culture,
            int numberOfResult,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
        )
        {
            string categoryListCondition = EnabledCategoryListCondition( culture, rootCategoryID );
            DataTable tableBestSelling = _categoryHelper.GetAllByBestSelling( culture, numberOfResult, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, categoryListCondition, true );

            if (tableBestSelling.Rows.Count > numberOfResult)
            {
                int numberOfRemove = tableBestSelling.Rows.Count - numberOfResult;
                for (int i = 0; i < numberOfRemove; i++)
                    tableBestSelling.Rows.RemoveAt( numberOfResult );
            }

            return ConvertProductTableToList( culture, tableBestSelling, storeID );
        }

        public IList<Product> GetAllByRandom(
                Culture culture,
                int numberOfRandomProducts,
                int outOfStockValue,
                bool useStockInventoryControl,
                string storeID,
                string rootCategoryID,
            bool isIncludeProductKit
            )
        {
            string categoryListCondition = EnabledCategoryListCondition( culture, rootCategoryID );
            DataTable table = _categoryHelper.GetAllByRandom( culture, numberOfRandomProducts, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, categoryListCondition, isIncludeProductKit );

            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetAllByRandom(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
        )
        {
            string categoryListCondition = EnabledCategoryListCondition( culture, rootCategoryID );
            DataTable table = _categoryHelper.GetAllByRandom( culture, numberOfRandomProducts, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, categoryListCondition, true );

            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetAllByNewArrival(
                Culture culture,
                int numberOfRandomProducts,
                int outOfStockValue,
                bool useStockInventoryControl,
                string storeID,
                string rootCategoryID,
            bool isIncludeProductKit
            )
        {
            string categoryListCondition = EnabledCategoryListCondition( culture, rootCategoryID );
            DataTable table = _categoryHelper.GetAllByNewArrival( culture, numberOfRandomProducts, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, categoryListCondition, isIncludeProductKit );

            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetAllByNewArrival(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
        )
        {
            string categoryListCondition = EnabledCategoryListCondition( culture, rootCategoryID );
            DataTable table = _categoryHelper.GetAllByNewArrival( culture, numberOfRandomProducts, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, categoryListCondition, true );

            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetAllByTodaySpecial(
            Culture culture, int outOfStockValue, bool useStockInventoryControl, string storeID, string rootCategoryID, bool isIncludeProductKit )
        {
            string categoryListCondition = EnabledCategoryListCondition( culture, rootCategoryID );

            ArrayList parameterList = new ArrayList();
            String sqlString = "SELECT Product.ProductID, ImageSecondary, C.CategoryID, L.UrlName " +
                "FROM ((Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock " +
                "FROM ProductStock GROUP BY ProductID) " +
                "AS S ON Product.ProductID = S.ProductID ) " +
                "INNER JOIN ProductCategory C ON Product.ProductID = C.ProductID " +
                "RIGHT JOIN (SELECT CategoryID, RootID FROM Category WHERE RootID = @RootID " +
                categoryListCondition + " ) AS D ON C.CategoryID = D.CategoryID) " +
                "INNER JOIN (SELECT UrlName, CultureID, ProductID FROM ProductLocale WHERE CultureID = @CultureID) AS L ON Product.ProductID = L.ProductID " +
                "WHERE IsTodaySpecial = @IsTodaySpecial ";

            parameterList.Add( DataAccess.CreateParameterString( rootCategoryID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            if (useStockInventoryControl)
            {
                sqlString += "AND ((SumStock > @outOfStockValue AND UseInventory = @trueCondition) OR " +
                             "UseInventory = @falseCondotion) ";
                parameterList.Add( DataAccess.CreateParameterInt32( outOfStockValue ) );
                parameterList.Add( DataAccess.CreateParameterBool( true ) );
                parameterList.Add( DataAccess.CreateParameterBool( false ) );
            }

            sqlString += "AND Product.IsEnabled = @IsEnabled " +
                         "AND IsParentVisible = @IsParentVisible ";
            if (!isIncludeProductKit)
                sqlString += " AND Product.IsProductKit = 0 ";
            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );


            DataTable table = DataAccess.ExecuteSelect( sqlString, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            DataTable filterIDTable = new DataTable();
            filterIDTable = table.Clone();
            foreach (DataRow row in table.Rows)
            {
                if (filterIDTable.Select( "ProductID = " + row["ProductID"].ToString() ).Length == 0)
                {
                    filterIDTable.ImportRow( row );
                }
            }

            return ConvertProductTableToList( culture, filterIDTable, storeID );
        }

        public IList<Product> GetAllByTodaySpecial(
            Culture culture, int outOfStockValue, bool useStockInventoryControl, string storeID, string rootCategoryID )
        {
            string categoryListCondition = EnabledCategoryListCondition( culture, rootCategoryID );

            ArrayList parameterList = new ArrayList();
            String sqlString = "SELECT Product.ProductID, ImageSecondary, C.CategoryID, L.UrlName " +
                "FROM ((Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock " +
                "FROM ProductStock GROUP BY ProductID) " +
                "AS S ON Product.ProductID = S.ProductID ) " +
                "INNER JOIN ProductCategory C ON Product.ProductID = C.ProductID " +
                "RIGHT JOIN (SELECT CategoryID, RootID FROM Category WHERE RootID = @RootID " +
                categoryListCondition + " ) AS D ON C.CategoryID = D.CategoryID) " +
                "INNER JOIN (SELECT UrlName, CultureID, ProductID FROM ProductLocale WHERE CultureID = @CultureID) AS L ON Product.ProductID = L.ProductID " +
                "WHERE IsTodaySpecial = @IsTodaySpecial ";

            parameterList.Add( DataAccess.CreateParameterString( rootCategoryID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            if (useStockInventoryControl)
            {
                sqlString += "AND ((SumStock > @outOfStockValue AND UseInventory = @trueCondition) OR " +
                             "UseInventory = @falseCondotion) ";
                parameterList.Add( DataAccess.CreateParameterInt32( outOfStockValue ) );
                parameterList.Add( DataAccess.CreateParameterBool( true ) );
                parameterList.Add( DataAccess.CreateParameterBool( false ) );
            }

            sqlString += "AND Product.IsEnabled = @IsEnabled " +
                         "AND IsParentVisible = @IsParentVisible ";

            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );


            DataTable table = DataAccess.ExecuteSelect( sqlString, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            DataTable filterIDTable = new DataTable();
            filterIDTable = table.Clone();
            foreach (DataRow row in table.Rows)
            {
                if (filterIDTable.Select( "ProductID = " + row["ProductID"].ToString() ).Length == 0)
                {
                    filterIDTable.ImportRow( row );
                }
            }

            return ConvertProductTableToList( culture, filterIDTable, storeID );
        }

        public IList<Product> GetAllByTaxClassID( Culture culture, string taxClassID, string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * FROM Product " +
                "WHERE TaxClassID = @TaxClassID;",
                DataAccess.CreateParameterString( taxClassID )
            );
            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetAllBySubscriptionLevelID(
            Culture culture,
            string subscriptionID,
            string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT * FROM Product LEFT JOIN (SELECT ProductID,UrlName FROM ProductLocale WHERE CultureID = @cultureID ) as PL ON Product.ProductID = PL.ProductID " +
               "INNER JOIN ProductSubscription PS ON Product.ProductID = PS.ProductID " +
               "WHERE PS.SubscriptionLevelID = @SubscriptionLevelID;",
               DataAccess.CreateParameterString( culture.CultureID ),
               DataAccess.CreateParameterString( subscriptionID )
               );
            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT Product.ProductID, Name, ShortDescription, LongDescription, Sku, Manufacturer, " +
                "    Model, ImageSecondary, Stock, Weight, FixedShippingCost, FreeShippingCost, ShippingCost, " +
                "    PP.WholesalePrice, PP.Price, PP.RetailPrice, Keywords, Other1, Other2, Other3, Other4, Other5, " +
                "    UrlName, IsTodaySpecial, MinQuantity, MaxQuantity, IsDownloadable, DownloadPath, IsEnabled, " +
                "    RelatedProducts, DiscountGroupID, PP.WholesalePrice2, PP.WholesalePrice3, " +
                "    PP.UseDefaultPrice, PP.UseDefaultRetailPrice, PP.UseDefaultWholeSalePrice, PP.UseDefaultWholeSalePrice2, PP.UseDefaultWholeSalePrice3, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, UseInventory, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                "    IsDynamicProductKitPrice, IsDynamicProductKitWeight, CreateDate,ViewCount, " +
                "    M.MetaKeyword AS MetaKeyword , M.MetaDescription AS MetaDescription, M.UseDefaultMetaKeyword AS UseDefaultMetaKeyword, M.UseDefaultMetaDescription AS UseDefaultMetaDescription " +
                "FROM (((Product INNER JOIN ProductStock ON Product.ProductID = ProductStock.ProductID ) " +
                "    INNER JOIN ProductLocale L " +
                "    ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice ) as PP ON Product.ProductID = PP.ProductID) " +
                "    INNER JOIN ProductMetaInformation M ON Product.ProductID = M.ProductID " +
                "WHERE 0 = 1; " );

            DataColumn[] keys = new DataColumn[] { table.Columns["ProductID"] };
            table.PrimaryKey = keys;
            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public string GetProductIDFromUrlName( string urlName )
        {
            string productID = DataAccess.ExecuteScalar(
                "SELECT ProductID " +
                "FROM ProductLocale " +
                "WHERE UrlName = @urlName; ",
                DataAccess.CreateParameterString( urlName ) );

            return productID;
        }

        public void RefreshIsParentVisibleByCategoryID( string categoryID, string storeID )
        {
            IList<Category> childCategories = DataAccessContext.CategoryRepository.GetByParentID(
                Culture.Null, categoryID, "CategoryID", BoolFilter.ShowAll );

            if (childCategories.Count == 0)
            {
                IList<Product> products = GetByCategoryID(
                    Culture.Null, categoryID, "ProductID", BoolFilter.ShowAll, storeID );

                foreach (Product product in products)
                {
                    RefreshIsParentVisible( product.ProductID );
                }
            }
            else
            {
                foreach (Category category in childCategories)
                {
                    RefreshIsParentVisibleByCategoryID( category.CategoryID, storeID );
                }
            }
        }

        public void RefreshIsParentVisibleByDepartmentID( string departmentID, string storeID )
        {
            IList<Department> childDepartments = DataAccessContext.DepartmentRepository.GetByParentID(
                Culture.Null, departmentID, "DepartmentID", BoolFilter.ShowAll );

            if (childDepartments.Count == 0)
            {
                IList<Product> products = GetByDepartmentID(
                    Culture.Null, departmentID, "ProductID", BoolFilter.ShowAll, storeID );

                foreach (Product product in products)
                {
                    RefreshIsParentVisible( product.ProductID );
                }
            }
            else
            {
                foreach (Department department in childDepartments)
                {
                    RefreshIsParentVisibleByCategoryID( department.DepartmentID, storeID );
                }
            }
        }

        public IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
            )
        {
            DataTable table = _categoryHelper.GetByCategoryDepartmentID( culture, categoryID, sortBy, isEnabled, storeID );
            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            BoolFilter isEnabled,
            bool isAffiliate,
            string storeID
    )
        {
            DataTable table = _categoryHelper.GetByCategoryDepartmentID( culture, categoryID, sortBy, isEnabled, isAffiliate, storeID );
            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetByDepartmentID(
            Culture culture,
            string departmentID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
        )
        {
            DataTable table = _departmentHelper.GetByCategoryDepartmentID( culture, departmentID, sortBy, isEnabled, storeID );
            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            bool isIncludeProductKit,
            out int howmanyItems,
            string storeID
            )
        {

            IList<Product> productList = GetByCategoryID( culture, categoryID, sortBy, isEnabled, isIncludeProductKit, storeID, startIndex, endIndex );
            howmanyItems = GetProductCountByCategoryID( categoryID, isEnabled );
            return productList;
        }

        public IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID
            )
        {

            IList<Product> productList = GetByCategoryID( culture, categoryID, sortBy, isEnabled, storeID, startIndex, endIndex );
            howmanyItems = GetProductCountByCategoryID( categoryID, isEnabled );
            return productList;
        }

        public IList<Product> GetFacetResultByCategoryID(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            string minPrice,
            string maxPrice,
            IList<SpecificationItemValue> specValueList,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID
            )
        {

            IList<Product> productList = GetFacetResultByCategoryID( culture, categoryIDs, departmentIDs, manufacturerID, minPrice, maxPrice, specValueList, sortBy, isEnabled, storeID, startIndex, endIndex );
            howmanyItems = _categoryHelper.GetFacetCount( culture, categoryIDs, departmentIDs, manufacturerID, minPrice, maxPrice, specValueList, sortBy, isEnabled, storeID );
            return productList;
        }

        public decimal GetMaxPriceByCategoryDepartment(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            BoolFilter isEnabled,
            string storeID
            )
        {
            return _categoryHelper.GetMaxPriceByCategoryDepartment( culture, categoryIDs, departmentIDs, isEnabled, storeID );
        }

        public IList<String> GetProductIDListByCategoryDepartment(
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            BoolFilter isEnabled,
            string storeID )
        {
            return _categoryHelper.GetProductIDListByCategoryDepartment( categoryIDs, departmentIDs, isEnabled, storeID );
        }

        public int GetFacetCount(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            string minPrice,
            string maxPrice,
            IList<SpecificationItemValue> specValueList,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
            )
        {
            return _categoryHelper.GetFacetCount( culture, categoryIDs, departmentIDs, manufacturerID, minPrice, maxPrice, specValueList, sortBy, isEnabled, storeID );
        }

        public IList<Product> GetByDepartmentID(
            Culture culture,
            string departmentID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID )
        {
            IList<Product> productList = GetByDepartmentID( culture, departmentID, sortBy, isEnabled, storeID, startIndex, endIndex );
            howmanyItems = GetProductCountByDepartmentID( departmentID, isEnabled );
            return productList;
        }

        public IList<Product> GetByDepartmentID(
            Culture culture,
            string[] departmentIDs,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID )
        {
            IList<Product> productList = GetByDepartmentID( culture, departmentIDs, sortBy, isEnabled, storeID, startIndex, endIndex );
            howmanyItems = GetProductCountByDepartmentID( departmentIDs );
            return productList;
        }

        public IList<Product> GetByManufacturerID(
           Culture culture,
           string manufacturerID,
           string sortBy,
           int startIndex,
           int endIndex,
           BoolFilter isEnabled,
           out int howmanyItems,
           string storeID )
        {
            IList<Product> productList = GetByManufacturerID( culture, manufacturerID, sortBy, isEnabled, storeID, startIndex, endIndex );
            howmanyItems = GetProductCountByManufacturerID( manufacturerID, isEnabled );
            return productList;
        }

        public bool ImageIsUsedByAnotherProduct( string urlLargeImage )
        {
            return _productImageAccess.ImageIsUsedByAnotherProduct( urlLargeImage );
        }

        public bool IsRegularImageExisting( string regularImagePath )
        {
            return _productImageAccess.IsRegularImageExisting( regularImagePath );
        }

        public bool IsLargeImageExisting( string largeImagePath )
        {
            return _productImageAccess.IsLargeImageExisting( largeImagePath );
        }

        public bool IsThumbnailImageExisting( string thumbnailImagePath )
        {
            return _productImageAccess.IsThumbnailImageExisting( thumbnailImagePath );
        }

        public bool IsImageSecondaryExisting( string imageSecondary )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * " +
                "FROM Product " +
                "WHERE ImageSecondary = @ImageSecondary",
                DataAccess.CreateParameterString( imageSecondary ) );

            if (table.Rows.Count > 1)
                return true;
            else
                return false;
        }

        public int GetAllProductCount()
        {
            string amount = DataAccess.ExecuteScalar(
                "SELECT COUNT(Product.ProductID) FROM Product INNER JOIN " +
                "(SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock GROUP BY ProductID ) AS S ON " +
                "Product.ProductID = S.ProductID;" );
            int result;
            if (int.TryParse( amount, out result ))
                return result;
            else
                return 0;
        }

        public int GetAllProductCountBySku( string sku )
        {
            string amount = DataAccess.ExecuteScalar(
                "SELECT COUNT(Product.ProductID) AS 'Count' FROM Product INNER JOIN " +
                "(SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock GROUP BY ProductID ) AS S ON " +
                "Product.ProductID = S.ProductID WHERE Product.Sku = @sku",
                DataAccess.CreateParameterString( sku ) );

            int result;
            if (int.TryParse( amount, out result ))
                return result;
            else
                return 0;
        }

        public int GetAllProductInactiveCount()
        {
            string amount = DataAccess.ExecuteScalar(
                "SELECT COUNT(Product.ProductID) FROM Product INNER JOIN " +
                "(SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock GROUP BY ProductID ) AS S ON " +
                "Product.ProductID = S.ProductID " +
                "WHERE IsEnabled = @IsEnabled;",
                DataAccess.CreateParameterBool( false ) );
            int result;
            if (int.TryParse( amount, out result ))
                return result;
            else
                return 0;
        }

        public int GetAllProductInLowStockCount( int lowStock, bool useStockInventoryControl )
        {
            if (!useStockInventoryControl)
                return 0;

            string amount = DataAccess.ExecuteScalar(
                "SELECT COUNT(Product.ProductID) FROM Product INNER JOIN " +
                "(SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock GROUP BY ProductID ) AS S ON " +
                "Product.ProductID = S.ProductID " +
                "WHERE SumStock <= @outOfStockValue AND Product.UseInventory = @UseInventory;",
                DataAccess.CreateParameterInt32( lowStock ),
                DataAccess.CreateParameterBool( true ) );
            int result;
            if (int.TryParse( amount, out result ))
                return result;
            else
                return 0;
        }

        public bool HasProductInDiscountGroup( string discountgroupID )
        {
            object result = DataAccess.ExecuteScalar(
                "SELECT Count(*) FROM Product " +
                "WHERE DiscountGroupID = @DiscountGroupID;",
                DataAccess.CreateParameterString( discountgroupID ) );

            if (ConvertUtilities.ToInt32( result ) == 0)
                return false;
            else
                return true;
        }

        public IList<Product> SearchProduct(
            Culture culture,
            string categoryID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID
            )
        {
            string productFieldName = ConvertFieldName( searchFilter.FieldName );
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, productFieldName, parameterList );

            string query = "SELECT DISTINCT  ROW_NUMBER() over (" +
                CreateSortByText( sortBy ) + ") as rownum, * FROM ( SELECT DISTINCT " + AllProductFields +
                "FROM " +
                "    ((( ( Product INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock " +
                "    GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "    (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "    ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "    WHERE CultureID = @CultureID2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID) " +

                CreateCategoryInnerJoinSql( categoryID, rootCategoryID ) +
                commandSearch +
                ") AS Product";

            string extQuery = "with DataSearch as (" + query + ") " +
                " select * from DataSearch where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable full = DataAccess.ExecuteSelect(
                extQuery,
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = GetSearchProductCount( culture, categoryID, sortBy, searchFilter, startIndex, endIndex, storeID, rootCategoryID );

            return ConvertProductTableToList( culture, full, storeID );
        }

        public IList<Product> QuickSearch(
            Culture culture,
            string searchTerm,
            string[] searchFieldList,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            string searchType
            )
        {

            IList<Product> full = ExecuteQuickSearch( culture, searchTerm, searchFieldList, sortBy, storeID, rootCategoryID, startIndex, endIndex, searchType );
            howManyItems = CountQuickSearchResult( culture, searchTerm, searchFieldList, sortBy, storeID, rootCategoryID, searchType );

            return full;
        }

        public IList<Product> AdvancedSearch(
            Culture culture,
            string categoryIDs,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            string searchType
            )
        {
            IList<Product> searchResult = ExecuteAdvancedSearch(
                culture, categoryIDs, searchTerm, price1, price2, searchFieldList, sortBy, storeID, rootCategoryID, startIndex, endIndex, true, searchType );
            howManyItems = CountAdvanceSearchResult(
                culture, categoryIDs, searchTerm, price1, price2, searchFieldList, sortBy, storeID, rootCategoryID, searchType );

            return searchResult;
        }

        public IList<Product> AdvancedSearch(
            Culture culture,
            string categoryIDs,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit,
            string searchType
            )
        {
            IList<Product> searchResult = ExecuteAdvancedSearch(
                culture, categoryIDs, searchTerm, price1, price2, searchFieldList, sortBy, storeID, rootCategoryID, startIndex, endIndex, isIncludeProductKit, searchType );
            howManyItems = CountAdvanceSearchResult(
                culture, categoryIDs, searchTerm, price1, price2, searchFieldList, sortBy, storeID, rootCategoryID, searchType );

            return searchResult;
        }

        public IList<Product> AdvancedSearch(
            Culture culture,
            string categoryIDs,
            string departmentIDs,
            string manufacturerID,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            string rootDepartmentID,
            string searchType
            )
        {
            IList<Product> searchResult = ExecuteAdvancedSearch(
                culture, categoryIDs, departmentIDs, manufacturerID, searchTerm, price1, price2, searchFieldList, sortBy, storeID, rootCategoryID, rootDepartmentID, startIndex, endIndex, true, searchType );
            howManyItems = CountAdvanceSearchResult(
                culture, categoryIDs, departmentIDs, manufacturerID, searchTerm, price1, price2, searchFieldList, sortBy, storeID, rootCategoryID, rootDepartmentID, searchType );

            return searchResult;
        }

        public IList<Product> AdvancedSearchDepartment(
            Culture culture,
            string departmentIDs,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootDepartmentID,
            string searchType
            )
        {
            IList<Product> result = ExecuteAdvancedSearchDepartment(
                culture, departmentIDs, searchTerm, price1, price2, searchFieldList, sortBy, storeID, rootDepartmentID, startIndex, endIndex, searchType );
            howManyItems = CountAdvanceSearchDepartmentResult( culture, departmentIDs, searchTerm, price1, price2, searchFieldList, sortBy, storeID, rootDepartmentID, searchType );
            return result;
        }

        public DataTable GetStockOptionLine( Culture culture, string[] optionGroupIDs )
        {
            string sql = String.Empty;

            for (int i = 0; i < optionGroupIDs.Length; i++)
            {
                string index = i.ToString();
                if (i < optionGroupIDs.Length - 1)
                {
                    index += ",";
                }

                sql += GenerateSqlSelectByOptionGroup(
                  culture.CultureID, optionGroupIDs[i], String.Format( "G{0}", index ) );
            }

            if (sql == String.Empty)
                return null;

            DataTable table = DataAccess.ExecuteSelect( String.Format( "SELECT * FROM {0}", sql ) );

            DataColumn myCol = new DataColumn( "CossGroup", Type.GetType( "System.String" ) );

            int totalCol = table.Columns.Count;

            table.Columns.Add( myCol );

            foreach (DataRow row in table.Rows)
            {
                string nameDisplay = String.Empty;

                for (int i = 1; i < totalCol; i = i + 2)
                {
                    if (nameDisplay == String.Empty)
                        nameDisplay = row[i].ToString();
                    else
                        nameDisplay += "-" + row[i].ToString();
                }

                row["CossGroup"] = nameDisplay;
            }
            return table;
        }

        public void UpdateSortOrder( string categoryID, string[] sortedProductIDs )
        {
            _categoryHelper.UpdateSortOrder( categoryID, sortedProductIDs );
        }

        public void UpdateSortOrderDepartment( string departmentID, string[] sortedProductIDs )
        {
            _departmentHelper.UpdateSortOrder( departmentID, sortedProductIDs );
        }

        public IList<Product> GetByIsQBExported(
            Culture culture,
            bool isQBExported,
            string rootCategoryID,
            string storeID,
            string existList )
        {
            string existListQuery = "";

            if (!string.IsNullOrEmpty( existList ))
            {
                existListQuery = "AND QBListID NOT IN ( " + existList + "  ) ";
            }

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * " +
                "FROM ((( product " +
                "INNER JOIN (" +
                "   SELECT * " +
                "   FROM productlocale " +
                "   WHERE cultureid = @CultureID) as c ON product.productid = c.productid " +
                "       AND IsQBExported = @IsQBExported " + existListQuery + ") " +
                "   INNER JOIN (" +
                "       SELECT DISTINCT productID " +
                "       FROM productcategory pc, category c " +
                "       WHERE pc.categoryid = c.categoryID AND rootid = @RootID) as pc ON product.productid = pc.productid) " +
                "       INNER JOIN (" +
                "           SELECT * " +
                "           FROM ProductPrice " +
                "           WHERE storeid = @StoreID ) as PP ON Product.ProductID = PP.ProductID);",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterBool( isQBExported ),
                DataAccess.CreateParameterString( rootCategoryID ),
                DataAccess.CreateParameterString( storeID )
            );

            return ConvertProductTableToList( culture, table, storeID );
        }

        public IList<Product> GetAllQBProduct( Culture culture, string rootCategoryID, string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * " +
                "FROM ((( product " +
                "INNER JOIN (" +
                "   SELECT * " +
                "   FROM productlocale " +
                "   WHERE cultureid = @CultureID) as c ON product.productid = c.productid) " +
                "   INNER JOIN (" +
                "       SELECT DISTINCT productID " +
                "       FROM productcategory pc, category c " +
                "       WHERE pc.categoryid = c.categoryID AND rootid = @RootID) as pc ON product.productid = pc.productid) " +
                "       INNER JOIN (" +
                "           SELECT * " +
                "           FROM ProductPrice " +
                "           WHERE storeid = @StoreID ) as PP ON Product.ProductID = PP.ProductID);",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( rootCategoryID ),
                DataAccess.CreateParameterString( storeID )
            );

            return ConvertProductTableToList( culture, table, storeID );
        }

        public void UpdateQBData( string productID, string qbListID, bool isQBExported )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Product " +
                "SET QBListID = @QBListID, " +
                "IsQBExported = @IsQBExported " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterString( qbListID ),
                DataAccess.CreateParameterBool( isQBExported ),
                DataAccess.CreateParameterString( productID )
            );
        }

        public void SetDefaultQBData( string qbListID )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Product " +
                "SET QBListID = '', " +
                "IsQBExported = @IsQBExported " +
                "WHERE QBListID = @QBListID;",
                DataAccess.CreateParameterBool( false ),
                DataAccess.CreateParameterString( qbListID )
            );
        }

        public Product GetOneLargeProduct( Culture culture, string productID, string storeID )
        {
            //  return GetOneFromDatabase(culture, productID, storeID);
            DataTable table = DataAccess.ExecuteSelect(
                SQLSelectProductString +
                "WHERE Product.ProductID = @productID; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( productID )
                );

            if (table.Rows.Count > 0)
                return CreateOneLargeProductFromRow( culture, table.Rows[0], storeID );
            else
                return Product.Null;

        }

        public Product CreateLargeProductItem(
            Product product, ProductLocale locale, ArrayList categoryVisibleIDList, bool processImage, bool isUseOption )
        {
            Product result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                result = CreateLargeProduct( product, locale, categoryVisibleIDList, processImage, isUseOption );
                transaction.Complete();
            }

            return result;
        }

        public Product UpdateLargeProductItem( Product product, ArrayList categoryVisibleIDList, bool processImage, bool isUseOption )
        {
            Product result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                result = UpdateLargeProduct( product, categoryVisibleIDList, processImage, isUseOption );
                transaction.Complete();
            }

            return result;
        }

        public Product GetOneBySku( Culture culture, string storeID, string sku )
        {
            DataTable table = DataAccess.ExecuteSelect(
                SQLSelectProductString +
                "WHERE Product.Sku = @sku; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( sku )
                );

            if (table.Rows.Count > 0)
                return CreateOneProductFromRow( culture, table.Rows[0], storeID );
            else
                return Product.Null;
        }

        public Product GetOneByUrlName( Culture culture, string storeID, string name )
        {
            DataTable table = DataAccess.ExecuteSelect(
                SQLSelectProductString +
                "WHERE UrlName = @name; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( name )
                );

            if (table.Rows.Count > 0)
                return CreateOneProductFromRow( culture, table.Rows[0], storeID );
            else
                return Product.Null;
        }

        public IList<Product> GetProductByName( Culture culture, string storeID, string name )
        {
            DataTable table = DataAccess.ExecuteSelect(
                SQLSelectProductString +
                "WHERE Name = @name; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( name )
            );

            return ConvertProductTableToList( culture, table, storeID );
        }

        public bool IsUseCustomPrice( string productID )
        {
            return _productCustomPriceAccess.IsExistItem( productID );
        }

        public IList<Product> GetAllProductKit( Culture culture )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT p.*, pl.CultureID, pl.Name " +
                "FROM Product p LEFT JOIN ( " +
                "	SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS pl ON p.ProductID = pl.ProductID " +
                "WHERE IsProductKit = @IsProductKit",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterBool( true ) );

            return ConvertProductTableToList( culture, table, "0" );
        }

        public IList<Product> GetProductKitItems( Culture culture, string ProductKitGroupID, string sortBy )
        {
            if (String.IsNullOrEmpty( sortBy ))
            {
                sortBy = "SortOrder";
            }

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT np.* " +
                "FROM ProductKitItem pki LEFT JOIN ( " +
                "	SELECT p.*, pl.CultureID, pl.Name " +
                "	FROM Product p LEFT JOIN ( " +
                "		SELECT * " +
                "		FROM ProductLocale " +
                "		WHERE CultureID = @CultureID ) AS pl ON p.ProductID = pl.ProductID ) AS np ON pki.ProductID = np.ProductID " +
                "WHERE ProductKitGroupID = @ProductKitGroupID " +
                "ORDER BY " + sortBy + "; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( ProductKitGroupID ) );

            return ConvertProductTableToList( culture, table, "0" );
        }

        public bool CheckIsProductKitByProductID( string productID )
        {
            string result = DataAccess.ExecuteScalar(
                "SELECT IsProductKit FROM Product WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );

            return ConvertUtilities.ToBoolean( result );
        }

        public void IncreaseViewCount( string productID )
        {
            string result = DataAccess.ExecuteScalar(
                "UPDATE Product SET ViewCount = ViewCount + 1 WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );
        }

        public Dictionary<string, string> GetMultipleCategoryProduct()
        {
            return _categoryHelper.GetMultipleCategoryDepartmentProduct();
        }


        public Dictionary<string, string> GetMultipleDepartmentProduct()
        {
            return _departmentHelper.GetMultipleCategoryDepartmentProduct();
        }

        public IList<Product> GetProductNewArrival(
            Culture culture,
            string storeID,
            int limitResult,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            string rootCategoryID,
            out int howManyItems )
        {
            howManyItems = CountTopProduct( culture, storeID, limitResult, sortBy, isEnabled, rootCategoryID );

            string query = "SELECT TOP( " + limitResult.ToString() + " ) ROW_NUMBER() over ( ORDER BY " + sortBy + " DESC ) as rownum, " +
                           "* FROM ( " + SQLSelectProductString +
                           FlagFilterHelper.CreateFlagFilterWithWhere( "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                           ") AS Product JOIN " +
                            "(SELECT productCategory.ProductID AS ID from category,productCategory " +
                            "WHERE category.rootID = @rootID " +
                            "AND category.CategoryID = productCategory.CategoryID) " +
                            "AS ProductByRoot ON  ProductByRoot.ID = Product.ProductID   ";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect(
                extQuery,
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( rootCategoryID)
            );

            return ConvertProductTableToList( culture, table, storeID );
        }

        #endregion
    }
}
