using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;

using Vevo.Data.Implementation;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Shipping;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;

namespace Vevo.Data
{
    public class ShippingOptionRepositoryNoCache : IShippingOptionRepository
    {
        #region Private

        private ShippingZoneAccess _shippingZoneAccess = new ShippingZoneAccess();

        private const string AllShippingFields =
            " Shipping.ShippingID, Shipping.ShippingOptionTypeID, ShippingName, " +
            "FreeShipping, FreeShippingValue, " +
            "FixedShippingCost, ShippingCostFirstItem, " +
            "ShippingCostNextItem, IsEnabled, HandlingFee, " +
            "AllowedType  ";

        private const string AllShippingLocaleFields = " ShippingID, CultureID, ShippingName ";


        private ShippingOptionLocale CreateOneLocaleFromRow( DataRow row )
        {
            ShippingOptionLocale locale = new ShippingOptionLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.ShippingName = row["ShippingName"].ToString();

            return locale;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "shippingid":
                    return "Shipping.ShippingID";
                default:
                    return fieldName;
            }
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "shippingid" ))
                return "ORDER BY Shipping." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private void DeleteLocales( string shippingID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ShippingLocale " +
                "WHERE ShippingID = @ShippingID; ",
                DataAccess.CreateParameterString( shippingID ) );
        }

        private void InsertLocales( ShippingOption shippingOption )
        {
            foreach (ShippingOptionLocale locale in shippingOption.Locales)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO ShippingLocale ( ShippingID, CultureID, ShippingName ) " +
                    "VALUES ( @ShippingID, @CultureID, @ShippingName )",
                    DataAccess.CreateParameterString( shippingOption.ShippingID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.ShippingName ) );
            }
        }

        private ShippingOption Create( ShippingOption shippingOption )
        {
            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO Shipping (ShippingOptionTypeID, FreeShipping, FreeShippingValue, " +
                "    FixedShippingCost, ShippingCostFirstItem, ShippingCostNextItem, IsEnabled, " +
                "    HandlingFee, AllowedType) " +
                "VALUES (@ShippingOptionTypeID, @FreeShipping, @FreeShippingValue, " +
                "    @FixedShippingCost, @ShippingCostFirstItem, @ShippingCostNextItem, @IsEnabled, " +
                "    @HandlingFee, @AllowedType); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( shippingOption.ShippingOptionTypeID ),
                DataAccess.CreateParameterBool( shippingOption.FreeShipping ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.FreeShippingValue ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.FixedShippingCost ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.ShippingCostFirstItem ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.ShippingCostNextItem ),
                DataAccess.CreateParameterBool( shippingOption.IsEnabled ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.HandlingFee ),
                DataAccess.CreateParameterString( shippingOption.AllowedType.ToString() )
                );

            shippingOption.ShippingID = newID;

            InsertLocales( shippingOption );
            _shippingZoneAccess.SaveAll( shippingOption.ShippingID, shippingOption.ShippingZone );

            return shippingOption;
        }

        private ShippingOption Update( ShippingOption shippingOption )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Shipping " +
                "SET " +
                "ShippingOptionTypeID = @ShippingOptionTypeID, " +
                "FreeShipping = @FreeShipping, " +
                "FreeShippingValue = @FreeShippingValue, " +
                "FixedShippingCost = @FixedShippingCost, " +
                "ShippingCostFirstItem = @ShippingCostFirstItem, " +
                "ShippingCostNextItem = @ShippingCostNextItem, " +
                "IsEnabled = @IsEnabled, " +
                "HandlingFee = @HandlingFee, " +
                "AllowedType = @AllowedType " +
                "WHERE ShippingID = @ShippingID;",
                DataAccess.CreateParameterString( shippingOption.ShippingOptionTypeID ),
                DataAccess.CreateParameterBool( shippingOption.FreeShipping ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.FreeShippingValue ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.FixedShippingCost ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.ShippingCostFirstItem ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.ShippingCostNextItem ),
                DataAccess.CreateParameterBool( shippingOption.IsEnabled ),
                DataAccess.CreateParameterDecimalMoney( shippingOption.HandlingFee ),
                DataAccess.CreateParameterString( shippingOption.AllowedType.ToString() ),
                DataAccess.CreateParameterString( shippingOption.ShippingID )
                );

            DeleteLocales( shippingOption.ShippingID );
            InsertLocales( shippingOption );
            _shippingZoneAccess.SaveAll( shippingOption.ShippingID, shippingOption.ShippingZone );

            return shippingOption;
        }

        private ShippingOption.ShippingZoneAllowedType GetAllowedType( string allowedType )
        {
            if (String.IsNullOrEmpty( allowedType ))
                return ShippingOption.ShippingZoneAllowedType.Unknown;
            else
                return EnumUtilities<ShippingOption.ShippingZoneAllowedType>.Parse( allowedType.ToString() );
        }

        private ShippingOption CreateOneFromRow( Culture culture, DataRow row )
        {
            ShippingOption shippingOption = new ShippingOption( culture );

            shippingOption.ShippingID = row["ShippingID"].ToString();
            shippingOption.ShippingOptionTypeID = row["ShippingOptionTypeID"].ToString();
            shippingOption.ShippingName = row["ShippingName"].ToString();
            shippingOption.FreeShipping = ConvertUtilities.ToBoolean( row["FreeShipping"] );
            shippingOption.FreeShippingValue = ConvertUtilities.ToDecimal( row["FreeShippingValue"] );
            shippingOption.FixedShippingCost = ConvertUtilities.ToDecimal( row["FixedShippingCost"] );
            shippingOption.ShippingCostFirstItem = ConvertUtilities.ToDecimal( row["ShippingCostFirstItem"] );
            shippingOption.ShippingCostNextItem = ConvertUtilities.ToDecimal( row["ShippingCostNextItem"] );
            shippingOption.IsEnabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );
            shippingOption.HandlingFee = ConvertUtilities.ToDecimal( row["HandlingFee"] );
            shippingOption.AllowedType = GetAllowedType( row["AllowedType"].ToString() );

            return shippingOption;
        }

        private IList<ShippingOption> ConvertTableToList( Culture culture, DataTable table )
        {
            return ConvertTableToList( culture, table, 0, table.Rows.Count );
        }

        private IList<ShippingOption> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<ShippingOption> list = new List<ShippingOption>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }

        private void CopyShippingOptionLocales( ShippingOption shippingOption, IList<ILocale> locales )
        {
            foreach (ShippingOptionLocale locale in locales)
            {
                shippingOption.Locales.Add( locale );
            }
        }

        #endregion


        #region Public Methods

        public ShippingOption Save( ShippingOption shippingOption )
        {
            ShippingOption result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (shippingOption.ShippingID == "0")
                    result = Create( shippingOption );
                else
                    result = Update( shippingOption );

                transaction.Complete();
            }

            return result;
        }

        public void Delete( string shippingID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                    "DELETE " +
                    "FROM Shipping " +
                    "WHERE shippingID = @shippingID; ",
                    DataAccess.CreateParameterString( shippingID ) );

                DeleteLocales( shippingID );
                _shippingZoneAccess.DeleteByShippingID( shippingID );

                transaction.Complete();
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ShippingLocale " +
                "WHERE CultureID = @CultureID ;",
                DataAccess.CreateParameterString( cultureID ) );
        }

        public ShippingOption GetOne( Culture culture, string shippingID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT " + AllShippingFields +
               "FROM Shipping LEFT JOIN " +
               "     (SELECT * FROM ShippingLocale WHERE CultureID = @CultureID) AS L " +
               "     ON Shipping.ShippingID = L.ShippingID " +
               "WHERE Shipping.ShippingID = @ShippingID; ",
               DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
               DataAccess.CreateParameterString( shippingID )
               );

            if (table.Rows.Count > 0)
            {
                ShippingOption shippingOption = CreateOneFromRow( culture, table.Rows[0] );
                CopyShippingOptionLocales( shippingOption, GetLocales( shippingID ) );
                return shippingOption;
            }
            else
                return ShippingOption.Null;
        }

        public IList<ILocale> GetLocales( string shippingID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingLocaleFields +
                "FROM ShippingLocale " +
                "WHERE ShippingID = @ShippingID; ",
                DataAccess.CreateParameterString( shippingID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<ShippingOption> SearchShipping(
            Culture culture,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            DataTable full = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingFields +
                "FROM Shipping LEFT JOIN " +
                "     (SELECT * FROM ShippingLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Shipping.ShippingID = L.ShippingID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["Shipping.ShippingID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( culture, full, startIndex, endIndex );
        }

        public string GetIDFromName( string name )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * FROM ShippingLocale " +
                "WHERE ShippingName = @ShippingName",
                DataAccess.CreateParameterString( name.Trim() ) );

            if (table.Rows.Count > 0)
                return table.Rows[0]["ShippingID"].ToString();
            else
                return "0";
        }

        public IList<ShippingOption> GetShipping( Culture culture, BoolFilter isRealTimeShipping )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingFields +
                "FROM (Shipping LEFT JOIN " +
                "     (SELECT * FROM ShippingLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Shipping.ShippingID = L.ShippingID) " +
                "LEFT JOIN ShippingOptionType " +
                "     ON Shipping.ShippingOptionTypeID = ShippingOptionType.ShippingOptionTypeID " +
                "WHERE IsEnabled = @IsEnabled " +
                FlagFilterHelper.CreateFlagFilterWithAnd(
                    "IsRealTime", RepositoryUtilities.ConvertToFlagFilter( isRealTimeShipping ) ),
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterBool( true )
                );

            DataColumn[] keys = new DataColumn[1] { table.Columns["ShippingID"] };
            table.PrimaryKey = keys;
            return ConvertTableToList( culture, table );
        }

        public IList<ShippingOption> GetAllByShippingZoneGroupID( Culture culture, string shippingZoneGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                 "SELECT DISTINCT " + AllShippingFields +
                 "FROM (Shipping LEFT JOIN " +
                 "     (SELECT * FROM ShippingLocale WHERE CultureID = @CultureID) AS L " +
                 "     ON Shipping.ShippingID = L.ShippingID) " +
                 "RIGHT JOIN (SELECT ZoneGroupID, ShippingID AS SID FROM ShippingZone WHERE ZoneGroupID = @ZoneGroupID) AS SZ " +
                 "     ON Shipping.ShippingID = SZ.SID ",
                 DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                 DataAccess.CreateParameterString( shippingZoneGroupID )
                 );

            DataColumn[] keys = new DataColumn[1] { table.Columns["Shipping.ShippingID"] };
            table.PrimaryKey = keys;

            return ConvertTableToList( culture, table );

        }

        public IList<ShippingZone> GetShippingZone( string shippingID )
        {
            return _shippingZoneAccess.GetByShippingID( shippingID );
        }

        #endregion
    }
}
