﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using LumenWorks.Framework.IO.Csv;
using Vevo.Domain.Products;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.SystemServices;

namespace Vevo.Domain.ImportExport
{
    public class ProductKitGroupItemImporter
    {
        #region Private

        private Dictionary<string, string> _skuDatabase;
        private Culture _culture;
        private IFileManager _fileManager;
        private ProductImportStatus _status;
        private DateTime _timeStart;
        private DateTime _timeEnd;

        private string _storeID;

        private CsvReader CreateCsvReader( string relativePath, out TextReader reader )
        {
            Encoding encoding;
            if (_fileManager.ContainsUnicodeBom( relativePath ))
                encoding = Encoding.Unicode;
            else
                encoding = Encoding.Default;

            reader = _fileManager.CreateTextReader( relativePath, encoding );
            CsvReader csv = new CsvReader( reader, true );
            csv.MissingFieldAction = MissingFieldAction.ReplaceByEmpty;
            csv.ParseError += ParseErrorHandler;

            return csv;
        }

        private void ParseErrorHandler( object sender, ParseErrorEventArgs e )
        {
            _status.AddError();
        }

        private bool ConvertToBool( string value )
        {
            int intValue;
            if (Int32.TryParse( value, out intValue ))
                return (intValue != 0);

            switch (value.ToLower())
            {
                case "true":
                case "yes":
                    return true;

                default:
                    return false;
            }
        }

        private bool IsProductKitGroupValid( string productKitGroupID )
        {
            if (DataAccessContext.ProductKitGroupRepository.GetOne( _culture, productKitGroupID ).IsNull)
                return false;
            return true;
        }

        private bool IsProductExist( string sku )
        {
            if (_skuDatabase.ContainsKey( sku ))
                return true;
            else
                return false;
        }

        private bool IsQuantityValid( string quantity )
        {
            bool result = true;

            if (String.IsNullOrEmpty( quantity ))
                result = false;
            uint dummyQTY = 0;
            if (!uint.TryParse( quantity, out dummyQTY ))
                result = false;

            return result;
        }

        private void DeleteAllProductKitGroupItem()
        {
            IList<ProductKitGroup> groupList = DataAccessContext.ProductKitGroupRepository.GetAll( _culture );
            foreach (ProductKitGroup group in groupList)
            {
                DataAccessContext.ProductKitGroupRepository.DeleteProductKitItemByGroupID( group.ProductKitGroupID );
            }
        }

        private ProductImportStatus ImportProductKitItem(
          string filePath,
          out TimeSpan timeDiff,
          string storeID
          )
        {
            TextReader reader = null;
            CsvReader csv = null;
            _status = new ProductImportStatus();
            try
            {
                csv = CreateCsvReader( filePath, out reader );

                // Read data line-by-line
                int i = 2;
                while (csv.ReadNextRecord())
                {
                    try
                    {
                        if (!IsProductExist( csv["Sku"] ))
                        {
                            _status.AddProductSKUError( i.ToString() );
                        }
                        else if (!IsProductKitGroupValid( csv["ProductKitGroupID"] ))
                        {
                            _status.AddProductKitGroupError( i.ToString() );
                        }
                        else if (!IsQuantityValid( csv["Quantity"] ))
                        {
                            _status.AddProductKitQuantityError( i.ToString() );
                        }
                        else
                        {
                            if (ImportProductKitGroupItem( csv, storeID ))
                            {
                                _status.AddSuccess();
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        SaveLogFile.SaveLog( ex );
                        _status.AddOtherError( i.ToString(), ex );
                    }
                    i++;
                }

                _status.BuildOtherError();
                _status.BuildProductKitGroupError();
                _status.BuildProductSKUError();
                _status.BuildProductKitQuantityError();
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }
            finally
            {
                if (csv != null)
                    csv.Dispose();

                if (reader != null)
                    reader.Dispose();
            }

            _timeEnd = DateTime.Now;
            timeDiff = _timeEnd.Subtract( _timeStart );
            return _status;
        }

        private bool ImportProductKitGroupItem( CsvReader csv, string storeID )
        {
            string productKitGroupID = csv["ProductKitGroupID"];
            bool result = false;

            string productID = DataAccessContext.ProductRepository.GetOneBySku( _culture, storeID, csv["Sku"].ToString() ).ProductID;

            ProductKitItem kitItem = new ProductKitItem();
            kitItem.IsDefault = ConvertToBool( csv["IsDefault"] );
            kitItem.IsUserDefinedQuantity = ConvertToBool( csv["IsUserDefinedQuantity"] );
            kitItem.Quantity = Convert.ToInt32( csv["Quantity"] );
            kitItem.ProductID = productID;
            kitItem.ProductKitGroupID = productKitGroupID;

            ProductKitGroup group = DataAccessContext.ProductKitGroupRepository.GetOne( _culture, productKitGroupID );

            if (!DataAccessContext.ProductKitGroupRepository.GetOneItem( group.ProductKitGroupID, productID ).IsNull)
            {
                DataAccessContext.ProductKitGroupRepository.DeleteProductKitItem( productID, productKitGroupID );
            }

            group.ProductKitItems.Add( kitItem );
            DataAccessContext.ProductKitGroupRepository.Save( group );
            result = true;

            return result;
        }

        #endregion

        #region Constructors

        public ProductKitGroupItemImporter( Culture culture, IFileManager fileManager )
        {
            _culture = culture;
            _fileManager = fileManager;
        }

        public ProductKitGroupItemImporter( Culture culture )
        {
            _culture = culture;
            _fileManager = new FileManager();
        }

        #endregion

        #region Public Methods

        public ProductImportStatus ImportProductKitItemPurgeAll(
            string filePath, out TimeSpan timeDiff, string storeID )
        {
            _timeStart = DateTime.Now;
            _storeID = storeID;
            _skuDatabase = DataAccessContext.ProductRepository.GetProductIDAndSku();
            DeleteAllProductKitGroupItem();
            return ImportProductKitItem( filePath, out timeDiff, storeID );
        }

        public ProductImportStatus ImportProductKitItemOverwrite(
            string filePath, out TimeSpan timeDiff, string storeID )
        {
            _timeStart = DateTime.Now;
            _storeID = storeID;
            _skuDatabase = DataAccessContext.ProductRepository.GetProductIDAndSku();
            return ImportProductKitItem( filePath, out timeDiff, storeID );
        }

        #endregion
    }
}
