using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain.Configurations;
using Vevo.Domain.Discounts;
using Vevo.Domain.Payments;
using Vevo.Domain.Shipping;
using Vevo.Domain.Tax;
using Vevo.Domain.Users;
using Vevo.Domain.Stores;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Orders
{
    public class OrderCalculator
    {
        #region Private

        private decimal GetDiscount(
            bool couponNeverExpired,
            CartItemGroup cartItemGroup,
            Coupon coupon,
            Customer customer,
            out IList<decimal> discountLine )
        {
            DiscountCalculator discountCalculator = new DiscountCalculator();

            if (!couponNeverExpired)
            {
                return discountCalculator.GetDiscount(
                    cartItemGroup, coupon, customer, out discountLine );
            }
            else
            {
                return discountCalculator.GetDiscountNoCouponExpiration(
                    cartItemGroup, coupon, customer, out discountLine );
            }
        }

        private OrderAmount CalculateNoGiftCertificate(
            CheckoutDetails checkoutDetails,
            CartItemGroup cartItemGroup,
            Customer customer,
            bool couponNeverExpired
            )
        {
            WholesaleStatus wholesaleStatus = customer.GetWholesaleStatus();

            decimal subtotal = cartItemGroup.GetSubtotal( wholesaleStatus );

            IList<decimal> discountLines;
            decimal discount = GetDiscount(
               couponNeverExpired, cartItemGroup, checkoutDetails.Coupon, customer, out discountLines );

            decimal shippingCost;

            IList<ICartItem> cartItems = new List<ICartItem>();
            foreach (ICartItem item in cartItemGroup.CartItems)
            {
                if (!item.IsPromotion)
                    cartItems.Add( item );
            }
            CartItemGroup groupProductOnly = new CartItemGroup( cartItems );

            if (checkoutDetails.Coupon.DiscountType == Coupon.DiscountTypeEnum.FreeShipping)
                shippingCost = 0;
            else
                shippingCost = checkoutDetails.ShippingMethod.GetShippingCost(
                    groupProductOnly, wholesaleStatus, discount );

            decimal handlingFee = checkoutDetails.ShippingMethod.GetHandlingFee(
                groupProductOnly, wholesaleStatus );

            TaxCalculator taxCalculator = new TaxCalculator();
            decimal tax = taxCalculator.GetTax(
                groupProductOnly,
                checkoutDetails,
                customer,
                discountLines,
                shippingCost,
                handlingFee
                );

            decimal redeemPrice = checkoutDetails.RedeemPrice;

            return OrderAmount.GetFactory()
                .WithSubtotal( subtotal )
                .WithDiscount( discount )
                .WithPointDiscount( redeemPrice )
                .WithShippingCost( shippingCost )
                .WithHandlingFee( handlingFee )
                .WithTax( tax )
                .Create();
        }

        #endregion


        #region Constructors

        public OrderCalculator()
        {
        }

        #endregion


        #region Public Methods

        public decimal CalculateGoogleCheckoutTax(
            CheckoutDetails checkoutDetails,
            IList<CartItemGroup> cartItemGroups,
            Customer customer,
            decimal shippingRate )
        {
            decimal tax = 0;

            foreach (CartItemGroup cartItemGroup in cartItemGroups)
            {
                WholesaleStatus wholesaleStatus = customer.GetWholesaleStatus();

                decimal subtotal = cartItemGroup.GetSubtotal( wholesaleStatus );

                IList<decimal> discountLines;

                decimal discount = GetDiscount(
                   false, cartItemGroup, checkoutDetails.Coupon, customer, out discountLines );

                decimal handlingFee = checkoutDetails.ShippingMethod.GetHandlingFee(
                    cartItemGroup, wholesaleStatus );

                TaxCalculator taxCalculator = new TaxCalculator();
                tax = tax + taxCalculator.GetTax(
                    cartItemGroup,
                    checkoutDetails,
                    customer,
                    discountLines,
                    shippingRate,
                    handlingFee
                    );
            }

            return tax;
        }

        public OrderAmount Calculate(
            CheckoutDetails checkoutDetails,
            CartItemGroup cartItemGroup,
            Customer customer,
            decimal reservedGiftAmount
            )
        {
            OrderAmount partialAmount = CalculateNoGiftCertificate(
                checkoutDetails, cartItemGroup, customer, false );

            decimal giftCertificateAmount = checkoutDetails.GiftCertificate.GetApplicableAmount(
                DataAccessContext.Configurations,
                cartItemGroup,
                partialAmount.Subtotal,
                partialAmount.Discount,
                partialAmount.PointDiscount,
                partialAmount.ShippingCost,
                partialAmount.HandlingFee,
                partialAmount.Tax,
                reservedGiftAmount );

            return OrderAmount.GetFactory()
                .WithSubtotal( partialAmount.Subtotal )
                .WithDiscount( partialAmount.Discount )
                .WithPointDiscount( partialAmount.PointDiscount )
                .WithShippingCost( partialAmount.ShippingCost )
                .WithHandlingFee( partialAmount.HandlingFee )
                .WithTax( partialAmount.Tax )
                .WithGiftCertificate( giftCertificateAmount )
                .Create();
        }

        public OrderAmount Calculate(
            CheckoutDetails checkoutDetails,
            IList<CartItemGroup> cartItemGroups,
            Customer customer
            )
        {
            OrderAmount totalOrderAmount = new OrderAmount();

            foreach (CartItemGroup cartItemGroup in cartItemGroups)
            {               
                totalOrderAmount = totalOrderAmount.Add(
                    Calculate( checkoutDetails, cartItemGroup, customer, totalOrderAmount.GiftCertificate ) );
            }

            return totalOrderAmount;
        }

        public OrderAmountPackage CalculatePackage(
            CheckoutDetails checkoutDetails,
            CartItemGroup cartItemGroup,
            Customer customer
            )
        {
            OrderAmount checkoutAmount = Calculate( checkoutDetails, cartItemGroup, customer, 0 );
            OrderAmount recurringRegularAmount;
            OrderAmount recurringTrialAmount;

            if (cartItemGroup.IsRecurring)
            {
                if (cartItemGroup.HasTrialPeriod)
                {
                    recurringTrialAmount = checkoutAmount;
                    recurringRegularAmount = Calculate(
                        checkoutDetails,
                        cartItemGroup.CreateRegularRecurringGroup(),
                        customer,
                        0 );
                }
                else
                {
                    recurringTrialAmount = new OrderAmount();
                    recurringRegularAmount = checkoutAmount;
                }
            }
            else
            {
                recurringRegularAmount = new OrderAmount();
                recurringTrialAmount = new OrderAmount();
            }


            return new OrderAmountPackage( checkoutAmount, recurringRegularAmount, recurringTrialAmount );
        }

        public OrderAmount CalculateForOrderEditing(
            CheckoutDetails checkoutDetails,
            IList<CartItemGroup> cartItemGroups,
            Customer customer,
            decimal orderGiftCertificateAmount
            )
        {
            OrderAmount totalOrderAmount = new OrderAmount();

            foreach (CartItemGroup cartItemGroup in cartItemGroups)
            {
                totalOrderAmount = totalOrderAmount.Add(
                    CalculateNoGiftCertificate( checkoutDetails, cartItemGroup, customer, true ) );
            }

            OrderAmount giftOrderAmount = OrderAmount.GetFactory()
                .WithGiftCertificate( orderGiftCertificateAmount )
                .Create();

            return totalOrderAmount.Add( giftOrderAmount );
        }

        // This amount includes taxes (if included) and discount. 
        // It does not incude shipping/handling and gift certificate. Also, the included tax
        // might not be the actual amount of tax depending on the shipping destination.

        public decimal GetShoppingCartTotal(
            Customer customer,
            IList<CartItemGroup> cartItemGroups,
            Coupon coupon )
        {
            WholesaleStatus wholesaleStatus = customer.GetWholesaleStatus( DataAccessContext.Configurations );
            decimal sum = 0;
            foreach (CartItemGroup cartItemGroup in cartItemGroups)
            {
                foreach (ICartItem cartItem in cartItemGroup.CartItems)
                {
                    sum += cartItem.GetShoppingCartUnitPrice( wholesaleStatus )
                        * cartItem.Quantity;
                }
            }
            return sum - GetPreCheckoutDiscount( customer, cartItemGroups, coupon );
        }

        public decimal GetPreCheckoutDiscount(
            Customer customer,
            IList<CartItemGroup> cartItemGroups,
            Coupon coupon )
        {
            DiscountCalculator discountCalculator = new DiscountCalculator();
            IList<decimal> discountLines;
            decimal discount = 0;
            foreach (CartItemGroup cartItemGroup in cartItemGroups)
            {
                discount += discountCalculator.GetDiscount( cartItemGroup, coupon, customer, out discountLines );
            }
            return discount;
        }

        public decimal GetShippingCost(
            ShippingMethod shippingMethod,
            IList<CartItemGroup> cartItemGroups,
            WholesaleStatus wholesaleStatus,
            decimal discount )
        {
            decimal shippingCost = 0;
            foreach (CartItemGroup cartItemGroup in cartItemGroups)
            {
                IList<ICartItem> cartItems = new List<ICartItem>();
                foreach (ICartItem item in cartItemGroup.CartItems)
                {
                    if (!item.IsPromotion)
                        cartItems.Add( item );
                }
                CartItemGroup group = new CartItemGroup( cartItems );
                shippingCost += shippingMethod.GetShippingCost( group, wholesaleStatus, discount );
            }

            return shippingCost;
        }

        public decimal GetHandlingFee(
            ShippingMethod shippingMethod,
            IList<CartItemGroup> cartItemGroups,
            WholesaleStatus wholesaleStatus )
        {
            decimal handlingFee = 0;
            foreach (CartItemGroup cartItemGroup in cartItemGroups)
            {
                handlingFee += shippingMethod.GetHandlingFee( cartItemGroup, wholesaleStatus );
            }
            return handlingFee;
        }

        public int ConvertPriceToRewardPoint( decimal price, decimal pointValue )
        {
            return ConvertUtilities.ToInt32( Math.Ceiling( price / pointValue ) );
        }

        public int GetPointFromPrice( decimal price, decimal rewardPoints )
        {
            return ConvertUtilities.ToInt32( Math.Floor( price * rewardPoints ) );
        }

        #endregion
    }
}
