using System;
using System.Collections.Generic;
using System.Text;

namespace Vevo.Domain.Orders
{
    public class RecurringProfile : ICloneable
    {
        #region Private

        private enum RecurringUnit
        {
            day,
            week,
            month,
            year
        }

        private enum RecurringDate
        {
            RegularBegin,
            RegularEnd,
            TrialBegin,
            TrialEnd
        }

        private string _recurringID = "0";
        private string _referenceID = "0";
        private string _recurringStatus = String.Empty;
        private int _recurringInterval = 0;
        private string _recurringIntervalUnit = String.Empty;
        private int _recurringNumberOfCycles = 0;
        private int _recurringNumberOfTrialCycles = 0;
        private DateTime _beginRecurringDate = DateTime.Now;
        private DateTime _endRecurringDate = DateTime.Now;
        private DateTime _beginTrialDate = DateTime.Now;
        private DateTime _endTrialDate = DateTime.Now;
        private int _recurringSequence = 0;
        private DateTime _updateTime = DateTime.Now;
        private decimal _trialPrice = 0;
        private decimal _regularPrice = 0;
        private string _recurringDescription = String.Empty;
        private decimal _regularShipping = 0;
        private decimal _regularTax = 0;
        private decimal _regularDiscount = 0;
        private decimal _trialShipping = 0;
        private decimal _trialTax = 0;
        private decimal _trialDiscount = 0;


        private RecurringUnit ParseRecurringUnit( string unit )
        {
            return (RecurringUnit) Enum.Parse( typeof( RecurringUnit ), unit.ToLower() );
        }

        private DateTime CalculateRecurringDate(
            DateTime dateTimeNow,
            int recurringInterval,
            string recurringIntervalUnit,
            int recurringIntervalNumberOfCycle,
            int recurringIntervalNumberOfTrialCycle,
            RecurringDate recurringDateType )
        {
            switch (recurringDateType)
            {
                case RecurringDate.RegularBegin:
                    if (recurringIntervalNumberOfTrialCycle == 0)
                        return dateTimeNow;
                    else
                        return RecurringAddTime(
                            dateTimeNow,
                            recurringInterval,
                            recurringIntervalNumberOfTrialCycle,
                            ParseRecurringUnit( recurringIntervalUnit ) );

                case RecurringDate.RegularEnd:
                    return RecurringAddTime(
                        RecurringAddTime(
                            dateTimeNow,
                            recurringInterval,
                            recurringIntervalNumberOfTrialCycle - 1,
                            ParseRecurringUnit( recurringIntervalUnit ) ),
                        recurringInterval,
                        recurringIntervalNumberOfCycle,
                        ParseRecurringUnit( recurringIntervalUnit ) );

                case RecurringDate.TrialBegin:
                    return dateTimeNow;

                case RecurringDate.TrialEnd:
                    return RecurringAddTime(
                        dateTimeNow,
                        recurringInterval,
                        recurringIntervalNumberOfTrialCycle - 1,
                        ParseRecurringUnit( recurringIntervalUnit ) );

                default:
                    throw new Exception( "Unknow period : " + recurringDateType.ToString() );
            }
        }

        private DateTime RecurringAddTime(
            DateTime dateTimeNow,
            int recurringInterval,
            int recurringIntervalCycle,
            RecurringUnit recurringUnit )
        {
            switch (recurringUnit)
            {
                case RecurringUnit.day:
                    return dateTimeNow.AddDays( recurringInterval * recurringIntervalCycle );
                case RecurringUnit.week:
                    return dateTimeNow.AddDays( recurringInterval * recurringIntervalCycle * 7 );
                case RecurringUnit.month:
                    return dateTimeNow.AddMonths( recurringInterval * recurringIntervalCycle );
                case RecurringUnit.year:
                    return dateTimeNow.AddYears( recurringInterval * recurringIntervalCycle );
                default:
                    throw new Exception( "Unknow period unit: " + recurringUnit.ToString() );
            }

        }

        #endregion

        #region Public Constructors

        public RecurringProfile()
        {
        }

        #endregion

        #region Public Properties

        public static RecurringProfile Null
        {
            get { return NullRecurringProfile.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string RecurringID { get { return _recurringID; } set { _recurringID = value; } }
        public string ReferenceID { get { return _referenceID; } set { _referenceID = value; } }
        public string RecurringStatus { get { return _recurringStatus; } set { _recurringStatus = value; } }
        public int RecurringInterval { get { return _recurringInterval; } set { _recurringInterval = value; } }
        public string RecurringIntervalUnit { get { return _recurringIntervalUnit; } set { _recurringIntervalUnit = value; } }
        public int RecurringNumberOfCycles { get { return _recurringNumberOfCycles; } set { _recurringNumberOfCycles = value; } }
        public int RecurringNumberOfTrialCycles { get { return _recurringNumberOfTrialCycles; } set { _recurringNumberOfTrialCycles = value; } }
        public DateTime BeginRecurringDate { get { return _beginRecurringDate; } set { _beginRecurringDate = value; } }
        public DateTime EndRecurringDate { get { return _endRecurringDate; } set { _endRecurringDate = value; } }
        public DateTime BeginTrialDate { get { return _beginTrialDate; } set { _beginTrialDate = value; } }
        public DateTime EndTrialDate { get { return _endTrialDate; } set { _endTrialDate = value; } }
        public int RecurringSequence { get { return _recurringSequence; } set { _recurringSequence = value; } }
        public DateTime UpdateTime { get { return _updateTime; } set { _updateTime = value; } }
        public Decimal TrialPrice { get { return _trialPrice; } set { _trialPrice = value; } }
        public Decimal RegularPrice { get { return _regularPrice; } set { _regularPrice = value; } }
        public String RecurringDescription { get { return _recurringDescription; } set { _recurringDescription = value; } }
        public decimal RegularShipping { get { return _regularShipping; } set { _regularShipping = value; } }
        public decimal RegularTax { get { return _regularTax; } set { _regularTax = value; } }
        public decimal RegularDiscount { get { return _regularDiscount; } set { _regularDiscount = value; } }
        public decimal TrialShipping { get { return _trialShipping; } set { _trialShipping = value; } }
        public decimal TrialTax { get { return _trialTax; } set { _trialTax = value; } }
        public decimal TrialDiscount { get { return _trialDiscount; } set { _trialDiscount = value; } }

        #endregion

        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            RecurringProfile newObject = (RecurringProfile) MemberwiseClone();
            return newObject;
        }

        #endregion

        #region Public Method

        public void SetRecurringPeriod(
            int interval,
            string intervalUnit,
            int numberOfRegularCycles,
            int numberOfTrialCycles )
        {
            _beginRecurringDate = CalculateRecurringDate(
                DateTime.Today,
                interval,
                intervalUnit,
                numberOfRegularCycles,
                numberOfTrialCycles,
                RecurringDate.RegularBegin );
            _endRecurringDate = CalculateRecurringDate(
                DateTime.Today,
                interval,
                intervalUnit,
                numberOfRegularCycles,
                numberOfTrialCycles,
                RecurringDate.RegularEnd );
            _beginTrialDate = CalculateRecurringDate(
                DateTime.Today,
                interval,
                intervalUnit,
                numberOfRegularCycles,
                numberOfTrialCycles,
                RecurringDate.TrialBegin );
            _endTrialDate = CalculateRecurringDate(
                DateTime.Today,
                interval,
                intervalUnit,
                numberOfRegularCycles,
                numberOfTrialCycles,
                RecurringDate.TrialEnd );
        }

        #endregion
    }
}
